@@ -19,6 +19,7 @@
 
 #include "GxPaintBox.h"
 #include "GxGradient.h"
+#include "GxLevelSlider.h"
 #include <gtk/gtkprivate.h>
 #include <gtk/gtk.h>
 #include <cmath>
@@ -31,12 +32,7 @@
 
 enum {
 	PROP_PAINT_FUNC = 1,
-	PROP_BORDER_WIDTH = 2,
-	PROP_SPACING = 3,
-	PROP_HOMOGENEOUS = 4,
-	PROP_ICON_SET = 5,
-	PROP_WIDTH = 6,
-	PROP_HEIGHT = 7,
+	PROP_ICON_SET = 2,
 };
 
 static void gx_paint_box_destroy(GtkObject *object);
@@ -47,10 +43,13 @@ static void gx_paint_box_get_property(
 static gboolean gx_paint_box_expose(GtkWidget *widget, GdkEventExpose *event);
 static void gx_paint_box_style_set (GtkWidget *widget, GtkStyle  *previous_style);
 
-G_DEFINE_TYPE(GxPaintBox, gx_paint_box, GTK_TYPE_HBOX)
+G_DEFINE_TYPE(GxPaintBox, gx_paint_box, GTK_TYPE_BOX)
 
 #define get_stock_id(widget) (GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(widget))->stock_id)
 #define get_main_image_id(widget) (GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(widget))->main_image_id)
+#define get_widget_id(widget) (GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(widget))->widget_id)
+#define get_widget_id2(widget) (GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(widget))->widget_id2)
+
 
 static void gx_paint_box_class_init (GxPaintBoxClass *klass)
 {
@@ -64,6 +63,8 @@ static void gx_paint_box_class_init (GxPaintBoxClass *klass)
 	widget_class->expose_event = gx_paint_box_expose;
 	klass->stock_id = "gxhead";
 	klass->main_image_id = "main_image";
+    klass->widget_id = "gxplate";
+    klass->widget_id2 = "gxplate2";
 	g_object_class_install_property(
 		gobject_class, PROP_PAINT_FUNC,
 		g_param_spec_string("paint-func",
@@ -112,15 +113,6 @@ static void gx_paint_box_class_init (GxPaintBoxClass *klass)
 						 G_MAXINT,
 						 0,
 		                 GParamFlags(GTK_PARAM_READABLE)));
-	g_object_class_install_property(
-		gobject_class,PROP_WIDTH,
-	    g_param_spec_int ("width",
-						 P_("Width"),
-						 P_("size.width request for paintbox"),
-						 0,
-						 G_MAXINT,
-						 0,
-						 GParamFlags(GTK_PARAM_READWRITE)));
 	gtk_widget_class_install_style_property(
 		GTK_WIDGET_CLASS(klass),
 		g_param_spec_int("width",
@@ -130,15 +122,6 @@ static void gx_paint_box_class_init (GxPaintBoxClass *klass)
 						 G_MAXINT,
 						 0,
 		                 GParamFlags(GTK_PARAM_READABLE)));
-	g_object_class_install_property(
-		gobject_class,PROP_HEIGHT,
-	    g_param_spec_int ("height",
-						 P_("Height"),
-						 P_("size.height request for paintbox"),
-						 0,
-						 G_MAXINT,
-						 0,
-						 GParamFlags(GTK_PARAM_READWRITE)));
 	gtk_widget_class_install_style_property(
 		GTK_WIDGET_CLASS(klass),
 		g_param_spec_int("height",
@@ -184,9 +167,12 @@ static void gx_paint_box_style_set(GtkWidget *widget, GtkStyle  *previous_style)
 
 static void gx_paint_box_init (GxPaintBox *paint_box)
 {
-	paint_box->paint_func = g_strdup("");
 	gtk_widget_set_redraw_on_allocate(GTK_WIDGET(paint_box), TRUE);
+	paint_box->paint_func = g_strdup("");
 	set_paint_func(paint_box, NULL);
+	paint_box->gxh_image = NULL;
+	paint_box->gxr_image = NULL;
+	paint_box->icon_set = 0;
 }
 
 static void gx_paint_box_destroy(GtkObject *object)
@@ -196,13 +182,15 @@ static void gx_paint_box_destroy(GtkObject *object)
 		g_free(paint_box->paint_func);
 		paint_box->paint_func = NULL;
 	}
-	GtkWidget *wi = GTK_WIDGET( g_object_new (GX_TYPE_PAINT_BOX, NULL ));
-	if (G_IS_OBJECT(GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(wi))-> gxh_image)) {
-		g_object_unref(GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(wi))->gxh_image);
+	if (G_IS_OBJECT(paint_box->gxh_image)) {
+		g_object_unref(paint_box->gxh_image);
 	}
-	if (G_IS_OBJECT(GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(wi))-> gxr_image)) {
-		g_object_unref(GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(wi))->gxr_image);
+	paint_box->gxh_image = NULL;
+	if (G_IS_OBJECT(paint_box->gxr_image)) {
+		g_object_unref(paint_box->gxr_image);
 	}
+	paint_box->gxr_image = NULL;
+	GTK_OBJECT_CLASS(gx_paint_box_parent_class)->destroy(object);
 }
 
 static gboolean gx_paint_box_expose(GtkWidget *widget, GdkEventExpose *event)
@@ -220,18 +208,6 @@ static void set_icon(GxPaintBox *paint_box, int value)
 	gtk_widget_style_get(GTK_WIDGET(paint_box), "icon-set", &spf, NULL);
 	 paint_box->icon_set = spf;
 }
-static void set_width(GxPaintBox *paint_box, int value)
-{
-	int spf;
-	gtk_widget_style_get(GTK_WIDGET(paint_box), "width", &spf, NULL);
-	 paint_box->width = spf;
-}
-static void set_height(GxPaintBox *paint_box, int value)
-{
-	int spf;
-	gtk_widget_style_get(GTK_WIDGET(paint_box), "height", &spf, NULL);
-	 paint_box->height = spf;
-}
 static void gx_paint_box_set_property(
 	GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
@@ -240,24 +216,9 @@ static void gx_paint_box_set_property(
 	case PROP_PAINT_FUNC:
 		set_paint_func(paint_box, g_value_get_string(value));
 		break;
-	case PROP_BORDER_WIDTH:
-		gtk_container_set_border_width (GTK_CONTAINER(paint_box), g_value_get_uint (value));
-		break;
-	case PROP_SPACING:
-		gtk_box_set_spacing (GTK_BOX(paint_box), g_value_get_int (value));
-		break;
-	case PROP_HOMOGENEOUS:
-		gtk_box_set_homogeneous (GTK_BOX(paint_box), g_value_get_boolean (value));
-		break;
 	case PROP_ICON_SET:
 		set_icon(paint_box, g_value_get_int(value));
 		break;
-	case PROP_WIDTH:
-		set_width(paint_box, g_value_get_int(value));
-		break;
-	case PROP_HEIGHT:
-		set_height(paint_box, g_value_get_int(value));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		break;
@@ -271,67 +232,21 @@ static void gx_paint_box_get_property(
 	case PROP_PAINT_FUNC:
 		g_value_set_string(value, GX_PAINT_BOX(object)->paint_func);
 		break;
-	case PROP_BORDER_WIDTH:
-		g_value_set_uint (value, GX_PAINT_BOX(object)->border_width);
-		break;
-	case PROP_SPACING:
-		g_value_set_int (value, GX_PAINT_BOX(object)->spacing);
-		break;
-	case PROP_HOMOGENEOUS:
-		g_value_set_boolean (value, GX_PAINT_BOX(object)->homogeneous);
-		break;
 	case PROP_ICON_SET:
 		g_value_set_int (value, GX_PAINT_BOX(object)->icon_set);
 		break;
-	case PROP_WIDTH:
-		g_value_set_int (value, GX_PAINT_BOX(object)->width);
-		break;
-	case PROP_HEIGHT:
-		g_value_set_int (value, GX_PAINT_BOX(object)->height);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		break;
 	}
 }
 
-void gx_box_pack_start (GxPaintBox *box, GtkWidget *child, 
-	gboolean expand, gboolean fill, guint padding)
-{
-	gtk_box_pack_start (GTK_BOX(box), child, expand, fill, padding);
-}
-
-void gx_box_pack_end (GxPaintBox *box, GtkWidget *child,
-	gboolean expand, gboolean fill, guint padding)
-{
-	gtk_box_pack_end (GTK_BOX(box), child, expand, fill, padding);
-}
-
-void gx_box_add (GxPaintBox *container, GtkWidget *widget)
-{
-	gtk_container_add (GTK_CONTAINER(container), widget);
-}
-
-void gx_box_remove (GxPaintBox *container,GtkWidget *widget)
-{
-	gtk_container_remove (GTK_CONTAINER(container), widget);
-}
-
-void gx_box_set_border_width (GxPaintBox *container, guint border_width)
-{
-	gtk_container_set_border_width (GTK_CONTAINER(container), border_width);
-}
-
-GList*  gx_box_get_children(GxPaintBox *container)
-{
-	return gtk_container_get_children (GTK_CONTAINER(container));
-}
-
-GtkWidget *gx_paint_box_new (gboolean homogeneous, gint spacing)
+GtkWidget *gx_paint_box_new (GtkOrientation orientation, gboolean homogeneous, gint spacing)
 {
 	return (GtkWidget*)g_object_new(
 		GX_TYPE_PAINT_BOX,
-		"spacing", spacing,
+		"orientation", orientation,
+		"spacing",     spacing,
 		"homogeneous", homogeneous ? TRUE : FALSE,
 		NULL);
 }
@@ -396,7 +311,7 @@ static void set_box_color(GtkWidget *wi, cairo_pattern_t *pat)
 }
 
 //----- paint boxes with cairo -----
-static gboolean amp_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void amp_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	cr = gdk_cairo_create(wi->window);
@@ -444,10 +359,9 @@ static gboolean amp_expose(GtkWidget *wi, GdkEventExpose *ev)
 
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean conv_widget_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void conv_widget_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	cairo_pattern_t *pat;
@@ -506,10 +420,9 @@ static gboolean conv_widget_expose(GtkWidget *wi, GdkEventExpose *ev)
 	cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean upper_widget_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void upper_widget_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	cairo_pattern_t *pat;
@@ -546,11 +459,9 @@ static gboolean upper_widget_expose(GtkWidget *wi, GdkEventExpose *ev)
 	cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
-
 }
 
-static gboolean rectangle_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void rectangle_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	/* create a cairo context */
@@ -582,10 +493,9 @@ static gboolean rectangle_expose(GtkWidget *wi, GdkEventExpose *ev)
 	cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean led_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void led_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	/* create a cairo context */
@@ -614,12 +524,12 @@ static gboolean led_expose(GtkWidget *wi, GdkEventExpose *ev)
 
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean rectangle_skin_color_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void rectangle_skin_color_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
+    cairo_pattern_t*pat;
 	/* create a cairo context */
 	cr = gdk_cairo_create(wi->window);
 	GdkRegion *region;
@@ -632,26 +542,96 @@ static gboolean rectangle_skin_color_expose(GtkWidget *wi, GdkEventExpose *ev)
 	double y0      = wi->allocation.y+1;
 	double rect_width  = wi->allocation.width-2;
 	double rect_height = wi->allocation.height-2;
+    
+    double radius = 12.;
+	if (rect_width<12) radius = rect_width;
+	else if (rect_height<12) radius = rect_height;
+	double x1,y1;
 
-	cairo_rectangle (cr, x0,y0,rect_width,rect_height+3);
-	cairo_set_source_rgb (cr, 0, 0, 0);
-	cairo_fill (cr);
+	x1=x0+rect_width-2;
+	y1=y0+rect_height-2;
+    x0+=1;
+    y0+=1;
+    
+    int spf;
+	gtk_widget_style_get(GTK_WIDGET(wi), "icon-set", &spf, NULL);
 
-	cairo_pattern_t*pat =
-	cairo_pattern_create_linear (0, y0, 0, y0+rect_height);
-		//cairo_pattern_create_radial (-50, y0, 5,rect_width-10,  rect_height, 20.0);
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 0.0);
+	cairo_paint(cr);
+	cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+    if(spf == 6 || spf == 7) {
+        GdkPixbuf * stock_image =
+            gtk_widget_render_icon(wi,get_widget_id(wi),(GtkIconSize)-1,NULL);
+    
+        guchar *pb_pixel = gdk_pixbuf_get_pixels (stock_image);
+        gint pixbuf_rowstride = gdk_pixbuf_get_rowstride (stock_image);
+        gint width = gdk_pixbuf_get_width (stock_image);
+        gint height = gdk_pixbuf_get_height (stock_image);
+        cairo_surface_t *s_image =
+            cairo_image_surface_create_for_data
+            (pb_pixel,CAIRO_FORMAT_RGB24 ,width, height,pixbuf_rowstride);
+
+        pat = cairo_pattern_create_for_surface(s_image);
+        cairo_set_source (cr, pat);
+        cairo_pattern_set_extend(cairo_get_source(cr), CAIRO_EXTEND_REPEAT);
+        
+        cairo_move_to  (cr, x0, y0 + radius);
+        cairo_curve_to (cr, x0 , y0, x0 , y0, x0 + radius, y0);
+        cairo_line_to (cr, x1 - radius, y0);
+        cairo_curve_to (cr, x1, y0, x1, y0, x1, y0 + radius);
+        cairo_line_to (cr, x1 , y1 - radius);
+        cairo_curve_to (cr, x1, y1, x1, y1, x1 - radius, y1);
+        cairo_line_to (cr, x0 + radius, y1);
+        cairo_curve_to (cr, x0, y1, x0, y1, x0, y1- radius);
+        cairo_close_path (cr);
+        
+        cairo_fill(cr);
+        g_object_unref(stock_image);
+        cairo_surface_destroy(s_image);
+        
+    } else {
+        cairo_move_to  (cr, x0, y0 + radius);
+	cairo_curve_to (cr, x0 , y0, x0 , y0, x0 + radius, y0);
+	cairo_line_to (cr, x1 - radius, y0);
+	cairo_curve_to (cr, x1, y0, x1, y0, x1, y0 + radius);
+	cairo_line_to (cr, x1 , y1 - radius);
+	cairo_curve_to (cr, x1, y1, x1, y1, x1 - radius, y1);
+	cairo_line_to (cr, x0 + radius, y1);
+	cairo_curve_to (cr, x0, y1, x0, y1, x0, y1- radius);
+	cairo_close_path (cr);
+        cairo_set_source_rgb (cr, 0, 0, 0);
+        cairo_fill (cr);
+    }
+	pat = cairo_pattern_create_linear (x0, y0, x0, y0+rect_height);
+    //cairo_pattern_create_radial (-50, y0, 5,rect_width-10,  rect_height, 20.0);
 	set_skin_color(wi, pat);
 	cairo_set_source (cr, pat);
-	cairo_rectangle (cr, x0+1,y0+1,rect_width-2,rect_height-1);
+    
+    
+    x1+=2;
+	y1+=2;
+    x0-=1;
+    y0-=1;
+	cairo_move_to  (cr, x0, y0 + radius);
+	cairo_curve_to (cr, x0 , y0, x0 , y0, x0 + radius, y0);
+	cairo_line_to (cr, x1 - radius, y0);
+	cairo_curve_to (cr, x1, y0, x1, y0, x1, y0 + radius);
+	cairo_line_to (cr, x1 , y1 - radius);
+	cairo_curve_to (cr, x1, y1, x1, y1, x1 - radius, y1);
+	cairo_line_to (cr, x0 + radius, y1);
+	cairo_curve_to (cr, x0, y1, x0, y1, x0, y1- radius);
+	cairo_close_path (cr);
+    
+    
+	//cairo_rectangle (cr, x0+1,y0+1,rect_width-2,rect_height-1);
 	cairo_fill (cr);
-
 	cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean convolver_icon_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void convolver_icon_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	/* create a cairo black arc to given widget */
@@ -698,12 +678,10 @@ static gboolean convolver_icon_expose(GtkWidget *wi, GdkEventExpose *ev)
 	cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean eq_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void eq_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
-	//float p = gx_gui::parameter_map["eq.f31_25"].getFloat().value;
     cairo_t *cr;
     cairo_text_extents_t extents;
 	/* create a cairo context */
@@ -760,10 +738,9 @@ static gboolean eq_expose(GtkWidget *wi, GdkEventExpose *ev)
 	cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean zac_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void zac_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	/* create a cairo context */
@@ -790,10 +767,9 @@ static gboolean zac_expose(GtkWidget *wi, GdkEventExpose *ev)
 	cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean AmpBox_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void AmpBox_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	/* create a cairo context */
@@ -837,7 +813,6 @@ static gboolean AmpBox_expose(GtkWidget *wi, GdkEventExpose *ev)
     cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 /*
 static void set_rack_color(const gchar * title, cairo_pattern_t *pat)
@@ -948,10 +923,15 @@ static void set_rack_color(const gchar * title, cairo_pattern_t *pat)
 	
 }
 */
-static gboolean RackBox_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void RackBox_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
+    int spf;
+	gtk_widget_style_get(GTK_WIDGET(wi), "icon-set", &spf, NULL);
+    if(spf == 7) return;
+    
 	cairo_t *cr;
 	cairo_text_extents_t extents;
+    cairo_pattern_t*pat;
 
 	/* create a cairo context */
 	cr = gdk_cairo_create(wi->window);
@@ -967,8 +947,28 @@ static gboolean RackBox_expose(GtkWidget *wi, GdkEventExpose *ev)
 	double rect_width  = wi->allocation.width;
 	double rect_height = wi->allocation.height;
 	double x,y;
-	
-	
+
+    if (spf == 6) {
+        GdkPixbuf * stock_image =
+            gtk_widget_render_icon(wi,get_widget_id2(wi),(GtkIconSize)-1,NULL);
+        
+        guchar *pb_pixel = gdk_pixbuf_get_pixels (stock_image);
+        gint pixbuf_rowstride = gdk_pixbuf_get_rowstride (stock_image);
+        gint width = gdk_pixbuf_get_width (stock_image);
+        gint height = gdk_pixbuf_get_height (stock_image);
+        cairo_surface_t *s_image = cairo_image_surface_create_for_data (pb_pixel,CAIRO_FORMAT_RGB24 ,width, height,pixbuf_rowstride);
+
+        pat = cairo_pattern_create_for_surface(s_image);
+        cairo_set_source (cr, pat);
+        cairo_pattern_set_extend(cairo_get_source(cr), CAIRO_EXTEND_REPEAT);
+        
+        cairo_rectangle(cr,x0+4,y0+4,rect_width-8,rect_height-8);
+        cairo_fill(cr);
+        g_object_unref(stock_image);
+        cairo_surface_destroy(s_image);
+    }
+
+
 	cairo_select_font_face (cr, "URW Chancery L", CAIRO_FONT_SLANT_NORMAL,
                                CAIRO_FONT_WEIGHT_BOLD);
 	cairo_set_font_size (cr, rect_width/12);
@@ -988,7 +988,7 @@ static gboolean RackBox_expose(GtkWidget *wi, GdkEventExpose *ev)
     cairo_stroke (cr);
     
 	cairo_rectangle (cr, x0+4,y0+4,rect_width-8,rect_height-8);
-	cairo_pattern_t*pat = cairo_pattern_create_linear (0, y0, 0, y0+rect_height);
+	pat = cairo_pattern_create_linear (0, y0, 0, y0+rect_height);
 	//set_rack_color(title, pat);
 	set_box_color(wi, pat);
 	//cairo_pattern_add_color_stop_rgba (pat, 1, 0, 0, 0.2, 0.8);
@@ -1015,10 +1015,9 @@ static gboolean RackBox_expose(GtkWidget *wi, GdkEventExpose *ev)
     cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean crybaby_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void crybaby_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	cairo_text_extents_t extents;
@@ -1114,10 +1113,9 @@ static gboolean crybaby_expose(GtkWidget *wi, GdkEventExpose *ev)
     cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean ir_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void ir_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	cairo_text_extents_t extents;
@@ -1213,10 +1211,9 @@ static gboolean ir_expose(GtkWidget *wi, GdkEventExpose *ev)
     cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean compressor_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void compressor_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	cairo_text_extents_t extents;
@@ -1316,10 +1313,9 @@ static gboolean compressor_expose(GtkWidget *wi, GdkEventExpose *ev)
     cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean seq_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void seq_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	cairo_text_extents_t extents;
@@ -1417,10 +1413,9 @@ static gboolean seq_expose(GtkWidget *wi, GdkEventExpose *ev)
     cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean lhfilter_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void lhfilter_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	/* create a cairo context */
@@ -1470,10 +1465,9 @@ static gboolean lhfilter_expose(GtkWidget *wi, GdkEventExpose *ev)
 	g_object_unref(frame);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean tribal_box_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void tribal_box_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	GdkPixbuf *_image, *stock_image;
 	cairo_t *cr;
@@ -1530,10 +1524,9 @@ static gboolean tribal_box_expose(GtkWidget *wi, GdkEventExpose *ev)
 
 	g_object_unref(_image);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean vbox_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void vbox_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	/* create a cairo context */
@@ -1565,10 +1558,9 @@ static gboolean vbox_expose(GtkWidget *wi, GdkEventExpose *ev)
 	cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean filter_box_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void filter_box_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	GdkPixbuf *_image, *stock_image;
 	cairo_t *cr;
@@ -1625,10 +1617,9 @@ static gboolean filter_box_expose(GtkWidget *wi, GdkEventExpose *ev)
 	cairo_destroy(cr);
 	g_object_unref(_image);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean plug_box_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void plug_box_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	/* create a cairo context */
@@ -1685,11 +1676,10 @@ static gboolean plug_box_expose(GtkWidget *wi, GdkEventExpose *ev)
     cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
 
-static gboolean info_box_expose_base(
+static void info_box_expose_base(
 	GtkWidget *wi, GdkEventExpose *ev,
 	void (*set_grad)(GtkWidget*,cairo_pattern_t*))
 {
@@ -1759,7 +1749,6 @@ static gboolean info_box_expose_base(
     cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
 static void set_grad_info_on(GtkWidget *wi, cairo_pattern_t *pat)
@@ -1769,18 +1758,18 @@ static void set_grad_info_on(GtkWidget *wi, cairo_pattern_t *pat)
 	cairo_pattern_add_color_stop_rgba (pat, 1, 0.2, 0.8, 0.2, 0.4);
 }
 
-static gboolean info_box_expose_on(GtkWidget *wi, GdkEventExpose *ev)
+static void info_box_expose_on(GtkWidget *wi, GdkEventExpose *ev)
 {
-	return info_box_expose_base(wi, ev, set_grad_info_on);
+	info_box_expose_base(wi, ev, set_grad_info_on);
 }
 
-static gboolean info_box_expose_off(GtkWidget *wi, GdkEventExpose *ev)
+static void info_box_expose_off(GtkWidget *wi, GdkEventExpose *ev)
 {
-	return info_box_expose_base(wi, ev, set_skin_color);
+	info_box_expose_base(wi, ev, set_skin_color);
 }
 
 
-static gboolean slooper_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void slooper_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
 	/* create a cairo context */
@@ -1829,91 +1818,90 @@ static gboolean slooper_expose(GtkWidget *wi, GdkEventExpose *ev)
     cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
-	return FALSE;
 }
 
-static gboolean gxhead_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void gxhead_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
-	GxPaintBoxClass *klass =  GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(wi));
+	GxPaintBox *paintbox =  GX_PAINT_BOX(wi);
 	
 	gint x0      = wi->allocation.x+1;
 	gint y0      = wi->allocation.y+1;
 	gint rect_width  = wi->allocation.width-2;
 	gint rect_height = wi->allocation.height-3;
-	
+
 	static int nf = 0;
 	static double ne_w1 = 0.;
-	static int spf;
-	
+	int spf;
+
 	gtk_widget_style_get(GTK_WIDGET(wi), "icon-set", &spf, NULL);
-	
-	if (nf != spf || ne_w1 != rect_width*rect_height || !(GDK_IS_PIXBUF (klass-> gxh_image))) {
+
+	if (nf != spf || ne_w1 != rect_width*rect_height || !(GDK_IS_PIXBUF (paintbox-> gxh_image))) {
 		ne_w1 = rect_width*rect_height;
 		nf = spf;
-		if (G_IS_OBJECT(klass-> gxh_image)) {
-			g_object_unref(klass->gxh_image);
+		if (G_IS_OBJECT(paintbox-> gxh_image)) {
+			g_object_unref(paintbox->gxh_image);
 		}
 		GdkPixbuf  *stock_image, *frame;
 		stock_image = gtk_widget_render_icon(wi,get_stock_id(wi),(GtkIconSize)-1,NULL);
 		double scalew = rect_width/double(gdk_pixbuf_get_width(stock_image)-48);
 		double scaleh = rect_height/double(gdk_pixbuf_get_height(stock_image)-48);
 		
-		klass->gxh_image = gdk_pixbuf_scale_simple(
+		paintbox->gxh_image = gdk_pixbuf_scale_simple(
 			stock_image, rect_width, rect_height, GDK_INTERP_NEAREST);
 		// upper border
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,24,0,gdk_pixbuf_get_width(stock_image)-48,12);
 		gdk_pixbuf_scale (
-			frame, klass->gxh_image,0,0,rect_width,12,0,0,scalew,1,GDK_INTERP_BILINEAR);
+			frame, paintbox->gxh_image,0,0,rect_width,12,0,0,scalew,1,GDK_INTERP_BILINEAR);
 		// under border
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,24,gdk_pixbuf_get_height(stock_image)-12,
 			gdk_pixbuf_get_width(stock_image)-48,12);
 		gdk_pixbuf_scale (
-			frame,klass->gxh_image,0,gdk_pixbuf_get_height(klass->gxh_image)-12,
-			rect_width,12,0,gdk_pixbuf_get_height(klass->gxh_image)-12,
+			frame,paintbox->gxh_image,0,gdk_pixbuf_get_height(paintbox->gxh_image)-12,
+			rect_width,12,0,gdk_pixbuf_get_height(paintbox->gxh_image)-12,
 			scalew,1,GDK_INTERP_BILINEAR);
 		// left border
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,0,24,12,gdk_pixbuf_get_height(stock_image)-48);	
 		gdk_pixbuf_scale(
-			frame, klass->gxh_image,0,12,12,rect_height-24,0,0,1,scaleh,GDK_INTERP_BILINEAR);
+			frame, paintbox->gxh_image,0,12,12,rect_height-24,0,0,1,scaleh,GDK_INTERP_BILINEAR);
 		// right border	
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,gdk_pixbuf_get_width(stock_image)-12,
 			24,12,gdk_pixbuf_get_height(stock_image)-48);
 		gdk_pixbuf_scale(
-			frame,klass->gxh_image,gdk_pixbuf_get_width(klass->gxh_image)-12,
-			12,12,rect_height-24,gdk_pixbuf_get_width(klass->gxh_image)-12,
+			frame,paintbox->gxh_image,gdk_pixbuf_get_width(paintbox->gxh_image)-12,
+			12,12,rect_height-24,gdk_pixbuf_get_width(paintbox->gxh_image)-12,
 			0,1,scaleh,GDK_INTERP_BILINEAR);
 		//left upper corner
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,0,0,20,20);
 		gdk_pixbuf_scale (
-			frame, klass->gxh_image,0,0,20,20,0,0,1,1,GDK_INTERP_BILINEAR);
+			frame, paintbox->gxh_image,0,0,20,20,0,0,1,1,GDK_INTERP_BILINEAR);
 		//right upper corner
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,gdk_pixbuf_get_width(stock_image)-20,0,20,20);
 		gdk_pixbuf_scale (
-			frame, klass->gxh_image,gdk_pixbuf_get_width(klass->gxh_image)-20,
-			0,20,20,gdk_pixbuf_get_width(klass->gxh_image)-20,0,1,1,
+			frame, paintbox->gxh_image,gdk_pixbuf_get_width(paintbox->gxh_image)-20,
+			0,20,20,gdk_pixbuf_get_width(paintbox->gxh_image)-20,0,1,1,
 			GDK_INTERP_BILINEAR);
 		//left under corner
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,0,gdk_pixbuf_get_height(stock_image)-20,20,20);
 		gdk_pixbuf_scale (
-			frame, klass->gxh_image,0,gdk_pixbuf_get_height(klass->gxh_image)-20,
-			20,20,0,gdk_pixbuf_get_height(klass->gxh_image)-20,1,1,
+			frame, paintbox->gxh_image,0,gdk_pixbuf_get_height(paintbox->gxh_image)-20,
+			20,20,0,gdk_pixbuf_get_height(paintbox->gxh_image)-20,1,1,
 			GDK_INTERP_BILINEAR);
 		//right under corner
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,gdk_pixbuf_get_width(stock_image)-20,
 			gdk_pixbuf_get_height(stock_image)-20,20,20);
 		gdk_pixbuf_scale (
-			frame, klass->gxh_image,gdk_pixbuf_get_width(klass->gxh_image)-20,
-			gdk_pixbuf_get_height(klass->gxh_image)-20,
-			20,20,gdk_pixbuf_get_width(klass->gxh_image)-20,
-			gdk_pixbuf_get_height(klass->gxh_image)-20,1,1,
+			frame, paintbox->gxh_image,gdk_pixbuf_get_width(paintbox->gxh_image)-20,
+			gdk_pixbuf_get_height(paintbox->gxh_image)-20,
+			20,20,gdk_pixbuf_get_width(paintbox->gxh_image)-20,
+			gdk_pixbuf_get_height(paintbox->gxh_image)-20,1,1,
 			GDK_INTERP_BILINEAR);
 				
 		// base 
@@ -1921,7 +1909,7 @@ static gboolean gxhead_expose(GtkWidget *wi, GdkEventExpose *ev)
 			stock_image,24,24,gdk_pixbuf_get_width(stock_image)-68,
 			gdk_pixbuf_get_height(stock_image)-48);
 		gdk_pixbuf_scale (
-			frame, klass->gxh_image,12,12,rect_width-24,rect_height-24,
+			frame, paintbox->gxh_image,12,12,rect_width-24,rect_height-24,
 			12,12,scalew,scaleh,GDK_INTERP_TILES);
 		//g_object_unref(_image);
 		g_object_unref(stock_image);
@@ -1929,17 +1917,21 @@ static gboolean gxhead_expose(GtkWidget *wi, GdkEventExpose *ev)
 	}
 	// draw to display
 	gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
-	                klass->gxh_image, 0, 0,
+	                paintbox->gxh_image, 0, 0,
 	                x0, y0, rect_width,rect_height,
 	                GDK_RGB_DITHER_NORMAL, 0, 0);
 	
-	return FALSE;
 }
 
-static gboolean gxrack_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void gxrack_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
-	GxPaintBoxClass *klass =  GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(wi));
+	gint rect_width  = wi->allocation.width-2;
+	gint rect_height = wi->allocation.height-3;
+	if (rect_width <= 0 || rect_height <= 0) {
+	    return;
+	}
 	cairo_t *cr;
+	GxPaintBox *paintbox = GX_PAINT_BOX(wi);
 	/* create a cairo context */
 	cr = gdk_cairo_create(wi->window);
 	GdkRegion *region;
@@ -1950,76 +1942,74 @@ static gboolean gxrack_expose(GtkWidget *wi, GdkEventExpose *ev)
 	
 	gint x0      = wi->allocation.x+1;
 	gint y0      = wi->allocation.y+1;
-	gint rect_width  = wi->allocation.width-2;
-	gint rect_height = wi->allocation.height-3;
-	
+
 	static double ne_w = 0.;
-	if (ne_w != rect_width*rect_height || !(GDK_IS_PIXBUF (klass-> gxr_image))) {
+	if (ne_w != rect_width*rect_height || !(GDK_IS_PIXBUF (paintbox-> gxr_image))) {
 		ne_w = rect_width*rect_height;
-		if (G_IS_OBJECT(klass-> gxr_image)) {
-			g_object_unref(klass->gxr_image);
+		if (G_IS_OBJECT(paintbox-> gxr_image)) {
+			g_object_unref(paintbox->gxr_image);
 		}
 		GdkPixbuf  *stock_image, *frame;
 		stock_image = gtk_widget_render_icon(wi,get_stock_id(wi),(GtkIconSize)-1,NULL);
 		double scalew = rect_width/double(gdk_pixbuf_get_width(stock_image)-48);
 		double scaleh = rect_height/double(gdk_pixbuf_get_height(stock_image)-48);
 		
-		klass->gxr_image = gdk_pixbuf_scale_simple(
+		paintbox->gxr_image = gdk_pixbuf_scale_simple(
 			stock_image, rect_width, rect_height, GDK_INTERP_NEAREST);
 		// upper border
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,24,0,gdk_pixbuf_get_width(stock_image)-48,12);
 		gdk_pixbuf_scale (
-			frame, klass->gxr_image,0,0,rect_width,12,0,0,scalew,1,GDK_INTERP_BILINEAR);
+			frame, paintbox->gxr_image,0,0,rect_width,12,0,0,scalew,1,GDK_INTERP_BILINEAR);
 		// under border
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,24,gdk_pixbuf_get_height(stock_image)-12,
 			gdk_pixbuf_get_width(stock_image)-48,12);
 		gdk_pixbuf_scale (
-			frame,klass->gxr_image,0,gdk_pixbuf_get_height(klass->gxr_image)-12,
-			rect_width,12,0,gdk_pixbuf_get_height(klass->gxr_image)-12,
+			frame,paintbox->gxr_image,0,gdk_pixbuf_get_height(paintbox->gxr_image)-12,
+			rect_width,12,0,gdk_pixbuf_get_height(paintbox->gxr_image)-12,
 			scalew,1,GDK_INTERP_BILINEAR);
 		// left border
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,0,24,12,gdk_pixbuf_get_height(stock_image)-48);
 		gdk_pixbuf_scale(
-			frame, klass->gxr_image,0,12,12,rect_height-24,0,0,1,scaleh,GDK_INTERP_BILINEAR);
+			frame, paintbox->gxr_image,0,12,12,rect_height-24,0,0,1,scaleh,GDK_INTERP_BILINEAR);
 		// right border	
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,gdk_pixbuf_get_width(stock_image)-12,
 			24,12,gdk_pixbuf_get_height(stock_image)-48);
 		gdk_pixbuf_scale(
-			frame,klass->gxr_image,gdk_pixbuf_get_width(klass->gxr_image)-12,
-			12,12,rect_height-24,gdk_pixbuf_get_width(klass->gxr_image)-12,
+			frame,paintbox->gxr_image,gdk_pixbuf_get_width(paintbox->gxr_image)-12,
+			12,12,rect_height-24,gdk_pixbuf_get_width(paintbox->gxr_image)-12,
 			0,1,scaleh,GDK_INTERP_BILINEAR);
 		//left upper corner
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,0,0,20,20);
 		gdk_pixbuf_scale (
-			frame, klass->gxr_image,0,0,20,20,0,0,1,1,GDK_INTERP_BILINEAR);
+			frame, paintbox->gxr_image,0,0,20,20,0,0,1,1,GDK_INTERP_BILINEAR);
 		//right upper corner
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,gdk_pixbuf_get_width(stock_image)-20,0,20,20);
 		gdk_pixbuf_scale (
-			frame, klass->gxr_image,gdk_pixbuf_get_width(klass->gxr_image)-20,
-			0,20,20,gdk_pixbuf_get_width(klass->gxr_image)-20,0,1,1,
+			frame, paintbox->gxr_image,gdk_pixbuf_get_width(paintbox->gxr_image)-20,
+			0,20,20,gdk_pixbuf_get_width(paintbox->gxr_image)-20,0,1,1,
 			GDK_INTERP_BILINEAR);
 		//left under corner
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,0,gdk_pixbuf_get_height(stock_image)-20,20,20);
 		gdk_pixbuf_scale (
-			frame, klass->gxr_image,0,gdk_pixbuf_get_height(klass->gxr_image)-20,
-			20,20,0,gdk_pixbuf_get_height(klass->gxr_image)-20,1,1,
+			frame, paintbox->gxr_image,0,gdk_pixbuf_get_height(paintbox->gxr_image)-20,
+			20,20,0,gdk_pixbuf_get_height(paintbox->gxr_image)-20,1,1,
 			GDK_INTERP_BILINEAR);
 		//right under corner
 		frame = gdk_pixbuf_new_subpixbuf(
 			stock_image,gdk_pixbuf_get_width(stock_image)-20,
 			gdk_pixbuf_get_height(stock_image)-20,20,20);
 		gdk_pixbuf_scale (
-			frame, klass->gxr_image,gdk_pixbuf_get_width(klass->gxr_image)-20,
-			gdk_pixbuf_get_height(klass->gxr_image)-20,
-			20,20,gdk_pixbuf_get_width(klass->gxr_image)-20,
-			gdk_pixbuf_get_height(klass->gxr_image)-20,1,1,
+			frame, paintbox->gxr_image,gdk_pixbuf_get_width(paintbox->gxr_image)-20,
+			gdk_pixbuf_get_height(paintbox->gxr_image)-20,
+			20,20,gdk_pixbuf_get_width(paintbox->gxr_image)-20,
+			gdk_pixbuf_get_height(paintbox->gxr_image)-20,1,1,
 			GDK_INTERP_BILINEAR);
 		g_object_unref(stock_image);
 		g_object_unref(frame);
@@ -2027,10 +2017,10 @@ static gboolean gxrack_expose(GtkWidget *wi, GdkEventExpose *ev)
 	
 	// draw to display
 	gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
-	                klass->gxr_image, 0, 0,
+	                paintbox->gxr_image, 0, 0,
 	                x0, y0, rect_width,rect_height,
 	                GDK_RGB_DITHER_NORMAL, 0, 0);
-	                
+
 	// base 
 	x0      += 12;
 	y0      += 12;
@@ -2066,10 +2056,9 @@ static gboolean gxrack_expose(GtkWidget *wi, GdkEventExpose *ev)
 	cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);                
-	return FALSE;
 }
 
-static gboolean main_expose(GtkWidget *wi, GdkEventExpose *ev)
+static void main_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	
 	cairo_t *cr;
@@ -2080,11 +2069,11 @@ static gboolean main_expose(GtkWidget *wi, GdkEventExpose *ev)
 	gdk_region_intersect (region, ev->region);
 	gdk_cairo_region (cr, region);
 	cairo_clip (cr);
-	static int w,h;
+	int w,h;
 	
-	gtk_widget_style_get(GTK_WIDGET(wi), "width", &w, NULL);
+	gtk_widget_style_get(GTK_WIDGET(wi), "width", &w, NULL); // FIXME
 	gtk_widget_style_get(GTK_WIDGET(wi), "height", &h, NULL);
-	gtk_widget_set_size_request (GTK_WIDGET (wi),w,h);
+	//gtk_widget_set_size_request (GTK_WIDGET (wi),w,h);
 	gint x0      = wi->allocation.x;
 	gint y0      = wi->allocation.y;
 	
@@ -2095,8 +2084,82 @@ static gboolean main_expose(GtkWidget *wi, GdkEventExpose *ev)
 	                main_image, 0, 0,
 	                x0, y0, w,h,
 	                GDK_RGB_DITHER_NORMAL, 0, 0);
-	return FALSE;
+
+    g_object_unref(main_image);
+    cairo_destroy(cr);
+	gdk_region_destroy (region);   
 }
+
+static void level_meter_expose(GtkWidget *wi, GdkEventExpose *ev)
+{
+	cairo_t *cr;
+	/* create a cairo context */
+	cr = gdk_cairo_create(wi->window);
+	GdkRegion *region;
+	region = gdk_region_rectangle (&wi->allocation);
+	gdk_region_intersect (region, ev->region);
+	gdk_cairo_region (cr, region);
+	cairo_clip (cr);
+	cairo_set_font_size (cr, 7.0);
+
+	double x0      = wi->allocation.x+1;
+	double y0      = wi->allocation.y+2;
+	double rect_width  = wi->allocation.width-2;
+	double rect_height = wi->allocation.height-4;
+
+	int  db_points[] = { -50, -40, -30, -20, -10, -3, 0, 4 };
+	char  buf[32];
+
+	cairo_rectangle (cr, x0,y0,rect_width,rect_height+2);
+	cairo_set_source_rgb (cr, 0, 0, 0);
+	cairo_fill (cr);
+
+	cairo_pattern_t*pat = cairo_pattern_create_linear (0, y0, 0, y0+rect_height);
+		//cairo_pattern_create_radial (-50, y0, 5,rect_width-10,  rect_height, 20.0);
+	cairo_pattern_add_color_stop_rgb (pat, 0, 0.1, 0.1, 0.2);
+	cairo_pattern_add_color_stop_rgb (pat, 0.3, 0.06, 0.06, 0.07);
+	cairo_pattern_add_color_stop_rgb (pat, 1, 0.03, 0.03, 0.03);
+	cairo_set_source (cr, pat);
+	cairo_rectangle (cr, x0+1,y0+1,rect_width-2,rect_height-2);
+	cairo_fill_preserve (cr);
+	
+	pat = cairo_pattern_create_linear (x0, 0, x0+rect_width, 0);
+	cairo_pattern_add_color_stop_rgba (pat, 0.3, 0.01, 0.01, 0.02, 0.3);
+	cairo_pattern_add_color_stop_rgba (pat, 0.5, 0.6, 0.6, 0.7, 0.5);
+	cairo_pattern_add_color_stop_rgba (pat, 0.7, 0.01, 0.01, 0.01, 0.3);
+	cairo_set_source (cr, pat);
+	cairo_rectangle (cr, x0+1,y0+1,rect_width-2,rect_height-2);
+	cairo_fill (cr);
+
+	for (unsigned int i = 0; i < sizeof (db_points)/sizeof (db_points[0]); ++i)
+	{
+		float fraction = log_meter(db_points[i]);
+		cairo_set_source_rgb (cr, 0.12*i, 1, 0.1);
+
+		cairo_move_to (cr, x0+rect_width*0.2,y0+rect_height - (rect_height * fraction));
+		cairo_line_to (cr, x0+rect_width*0.8 ,y0+rect_height -  (rect_height * fraction));
+		if (i<6)
+		{
+			snprintf (buf, sizeof (buf), "%d", db_points[i]);
+			cairo_move_to (cr, x0+rect_width*0.32,y0+rect_height - (rect_height * fraction));
+		}
+		else
+		{
+			snprintf (buf, sizeof (buf), " %d", db_points[i]);
+			cairo_move_to (cr, x0+rect_width*0.34,y0+rect_height - (rect_height * fraction));
+		}
+		cairo_show_text (cr, buf);
+	}
+
+	cairo_set_source_rgb (cr, 0.4, 0.8, 0.4);
+	cairo_set_line_width (cr, 0.5);
+	cairo_stroke (cr);
+
+	cairo_pattern_destroy (pat);
+	cairo_destroy(cr);
+	gdk_region_destroy (region);
+}
+
 static void set_expose_func(GxPaintBox *paint_box, const gchar *paint_func)
 {
 	if (strcmp(paint_func, "amp_expose") == 0) {
@@ -2151,6 +2214,8 @@ static void set_expose_func(GxPaintBox *paint_box, const gchar *paint_func)
 		paint_box->expose_func = ir_expose;
 	} else if (strcmp(paint_func, "main_expose") == 0) {
 		paint_box->expose_func = main_expose;
+	} else if (strcmp(paint_func, "level_meter_expose") == 0) {
+	    paint_box->expose_func = level_meter_expose;
 	} else {
 		paint_box->expose_func = 0;
 	}
