@@ -47,7 +47,6 @@ struct _GxReglerPrivate
 
 enum {
   VALUE_ENTRY,
-  VALUES_ENTRY,
   FORMAT_VALUE,
   LAST_SIGNAL
 };
@@ -67,6 +66,7 @@ static void gx_regler_class_init(GxReglerClass *klass);
 static void gx_regler_init(GxRegler *regler);
 static void gx_control_parameter_interface_init (GxControlParameterIface *iface);
 static void gx_regler_finalize(GObject*);
+static void gx_regler_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
 static void gx_regler_style_set (GtkWidget *widget, GtkStyle  *previous_style);
 static void gx_regler_destroy(GtkObject *object);
 static void gx_regler_set_property(
@@ -319,6 +319,7 @@ static void gx_regler_class_init(GxReglerClass *klass)
 	widget_class->style_set = gx_regler_style_set;
 	widget_class->button_release_event = gx_regler_button_release;
 	widget_class->scroll_event = gx_regler_scroll;
+	widget_class->size_allocate = gx_regler_size_allocate;
 
 	range_class->value_changed = gx_regler_value_changed;
 	range_class->change_value = gx_regler_change_value;
@@ -346,6 +347,13 @@ static void gx_regler_class_init(GxReglerClass *klass)
 					  G_TYPE_STRING, 1,
 					  G_TYPE_DOUBLE);
 
+	gtk_widget_class_install_style_property(
+		widget_class,
+		g_param_spec_boolean("show-value",
+		                     P_("show value"),
+		                     P_("display the value"),
+		                     TRUE,
+		                     GParamFlags(GTK_PARAM_READABLE)));
 	gtk_widget_class_install_style_property(
 		widget_class,
 		g_param_spec_int("value-spacing",P_("Value spacing"),
@@ -538,6 +546,17 @@ static void gx_regler_finalize(GObject *object)
 	G_OBJECT_CLASS(gx_regler_parent_class)->finalize(object);
 }
 
+static void gx_regler_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
+{
+	widget->allocation = *allocation;
+	if (gtk_widget_get_realized(widget))
+		gdk_window_move_resize(GTK_RANGE(widget)->event_window,
+							   widget->allocation.x,
+							   widget->allocation.y,
+							   widget->allocation.width,
+							   widget->allocation.height);
+}
+
 static void step_back(GtkRange *range)
 {
 	gdouble newval;
@@ -797,7 +816,9 @@ void _gx_regler_get_positions(GxRegler *regler, GdkRectangle *image_rect,
 	gint y = widget->allocation.y;
 	gint width = image_rect->width;
 	gint height =  image_rect->height;
-	if (regler->show_value) {
+	gboolean show_value;
+	gtk_widget_style_get(widget, "show-value", &show_value, NULL);
+	if (regler->show_value && show_value) {
 		GxReglerPrivate *priv = regler->priv;
 		gint text_width = priv->value_req.width;
 		gint text_height = priv->value_req.height;
@@ -845,6 +866,34 @@ void _gx_regler_get_positions(GxRegler *regler, GdkRectangle *image_rect,
 	}
 }
 
+gboolean _gx_regler_check_display_popup(GxRegler *regler, GdkRectangle *image_rect,
+					GdkRectangle *value_rect, GdkEventButton *event)
+{
+	// check if value entry popup requested
+	gdouble x = event->x + GTK_WIDGET(regler)->allocation.x;
+	gdouble y = event->y + GTK_WIDGET(regler)->allocation.y;
+	GdkRectangle *rect = NULL;
+	if (image_rect && _approx_in_rectangle(x, y, image_rect)) {
+		if (event->button == 3) {
+			rect = image_rect;
+		}
+	} else if (value_rect && _approx_in_rectangle(x, y, value_rect)) {
+		if (event->button == 1 || event->button == 3) {
+			rect = value_rect;
+		} else {
+			return TRUE;
+		}
+	} else {
+		return TRUE;
+	}
+	if (rect) {
+		gboolean ret;
+		g_signal_emit_by_name(regler, "value-entry", rect, event, &ret);
+		return TRUE;
+	}
+	return FALSE;
+}
+
 static gchar* _gx_regler_format_value(GxRegler *regler, gdouble value)
 {
 	gchar *fmt = NULL;
@@ -857,11 +906,31 @@ static gchar* _gx_regler_format_value(GxRegler *regler, gdouble value)
 	}
 }
 
+static const GtkBorder default_value_border = { 6, 6, 3, 3 };
+
+static void get_value_border(GtkWidget *widget, GtkBorder *value_border)
+{
+	GtkBorder *tmp_border;
+
+	gtk_widget_style_get(widget, "value-border", &tmp_border, NULL);
+	if (tmp_border) {
+		*value_border = *tmp_border;
+		gtk_border_free(tmp_border);
+	} else {
+		*value_border = default_value_border;
+    }
+}
+
 void _gx_regler_simple_display_value(GxRegler *regler, GdkRectangle *rect)
 {
 	if (!regler->show_value) {
 		return;
 	}
+	gboolean show_value;
+	gtk_widget_style_get(GTK_WIDGET(regler), "show-value", &show_value, NULL);
+	if (!show_value) {
+		return;
+	}
     GtkWidget *widget = GTK_WIDGET(regler);
     PangoLayout *l = regler->value_layout;
     PangoRectangle logical_rect;
@@ -881,14 +950,23 @@ void _gx_regler_display_value(GxRegler *regler, GdkRectangle *rect)
 	if (!regler->show_value) {
 		return;
 	}
+	gboolean show_value;
+	gtk_widget_style_get(GTK_WIDGET(regler), "show-value", &show_value, NULL);
+	if (!show_value) {
+		return;
+	}
 	cairo_t *cr = gdk_cairo_create(GTK_WIDGET(regler)->window);
-	double x0 = rect->x + 1;
-	double y0 = rect->y + 1;
-	double rect_width  =  rect->width - 2;
-	double rect_height =  rect->height - 2;
-	gint border_width = 8;
+	GtkBorder border;
+	get_value_border(GTK_WIDGET(regler), &border);
+	gint inset = max(0, min(2, min(min(border.left-4, border.right-4), min(border.top-1, border.bottom-1))));
+	gint frm = inset < 2 ? 0 : 1;
+	double x0 = rect->x + frm;
+	double y0 = rect->y + frm;
+	double rect_width  =  rect->width - 2 * frm;
+	double rect_height =  rect->height - 2 * frm;
+	gint border_width = 2;
 
-    cairo_rectangle (cr, x0-1,y0-1,rect_width+2,rect_height+2);
+    cairo_rectangle (cr, rect->x,rect->y,rect->width,rect->height);
     cairo_set_source_rgba(cr, 0, 0, 0, 0.4);
     cairo_fill (cr);
 
@@ -897,21 +975,23 @@ void _gx_regler_display_value(GxRegler *regler, GdkRectangle *rect)
     cairo_pattern_add_color_stop_rgba (pat, 1, 0., 0., 0., 0.8);
     cairo_pattern_add_color_stop_rgba (pat, 0, 0, 0, 0, 0.4);
     cairo_set_source (cr, pat);
-    cairo_rectangle (cr, x0+2,y0+2,rect_width-4,rect_height-4);
+    cairo_rectangle (cr, x0+inset,y0+inset,rect_width-2*inset,rect_height-2*inset);
     cairo_fill (cr);
 
-    cairo_set_source_rgb(cr,  0.2, 0.2, 0.2);
-    cairo_set_line_width(cr, 2.0);
-    cairo_move_to(cr,x0+rect_width-3, y0+3);
-    cairo_line_to(cr, x0+rect_width-3, y0+rect_height-2);
-    cairo_line_to(cr, x0+2, y0+rect_height-2);
-    cairo_stroke(cr);
+	// line on right and bottom side
+	cairo_set_source_rgb(cr,  0.2, 0.2, 0.2);
+	cairo_set_line_width(cr, 2.0);
+	cairo_move_to(cr,x0+rect_width-1-inset, y0+1+inset);
+	cairo_line_to(cr, x0+rect_width-1-inset, y0+rect_height-inset);
+	cairo_line_to(cr, x0+inset, y0+rect_height-inset);
+	cairo_stroke(cr);
 
+	// line on left and top side
     cairo_set_source_rgb(cr,  0.1, 0.1, 0.1);
     cairo_set_line_width(cr, 2.0);
-    cairo_move_to(cr,x0+3, y0+rect_height-1);
-    cairo_line_to(cr, x0+3, y0+3);
-    cairo_line_to(cr, x0+rect_width-3, y0+3);
+    cairo_move_to(cr,x0+1+inset, y0+rect_height-inset+1);
+    cairo_line_to(cr, x0+1+inset, y0+1+inset);
+    cairo_line_to(cr, x0+rect_width-1-inset, y0+1+inset);
     cairo_stroke(cr);
 
 	gchar *txt;
@@ -923,71 +1003,61 @@ void _gx_regler_display_value(GxRegler *regler, GdkRectangle *rect)
     g_free(txt);
     PangoRectangle logical_rect;
     pango_layout_get_pixel_extents(l, NULL, &logical_rect);
-    gdouble off = border_width + (rect->width - 2*border_width
-                                  - logical_rect.width) * regler->value_xalign;
+    gdouble off = border_width + border.left + regler->value_xalign *
+		(rect->width - logical_rect.width - (2*border_width + border.left + border.right));
     cairo_move_to(cr, x0-1+off, y0+3);
     pango_cairo_show_layout(cr, l);
 
 	cairo_destroy(cr);
 }
 
-static const GtkBorder default_value_border = { 6, 6, 3, 3 };
-
-static void get_value_border(GtkWidget *widget, GtkBorder *value_border)
-{
-	GtkBorder *tmp_border;
-
-	gtk_widget_style_get(widget, "value-border", &tmp_border, NULL);
-	if (tmp_border) {
-		*value_border = *tmp_border;
-		gtk_border_free(tmp_border);
-	} else {
-		*value_border = default_value_border;
-    }
-}
-
 void _gx_regler_calc_size_request(GxRegler *regler, GtkRequisition *requisition)
 {
+	if (!regler->show_value) {
+		return;
+	}
+	gboolean show_value;
+	gtk_widget_style_get(GTK_WIDGET(regler), "show-value", &show_value, NULL);
+	if (!show_value) {
+		return;
+	}
 	gx_regler_ensure_layout(regler);
-	if (regler->show_value) {
-		PangoRectangle logical_rect1, logical_rect2;
-		GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(regler));
-		gint value_spacing;
-		ensure_digits(regler);
-		GtkBorder border;
-		get_value_border(GTK_WIDGET(regler), &border);
-		gtk_widget_style_get(GTK_WIDGET(regler), "value-spacing", &value_spacing, NULL);
-		gchar *txt;
-		ensure_digits(regler);
-		txt = _gx_regler_format_value(regler, gtk_adjustment_get_lower(adj));
-		pango_layout_set_text(regler->value_layout, txt, -1);
-		g_free(txt);
-		pango_layout_get_pixel_extents(regler->value_layout, NULL, &logical_rect1);
-		txt = _gx_regler_format_value(regler, gtk_adjustment_get_upper(adj));
-		pango_layout_set_text(regler->value_layout, txt, -1);
-		g_free(txt);
-		pango_layout_get_pixel_extents(regler->value_layout, NULL, &logical_rect2);
-		gint height = max(logical_rect1.height,logical_rect2.height) + border.top + border.bottom;
-		gint width = max(logical_rect1.width,logical_rect2.width) + border.left + border.right;
-		GxReglerPrivate *priv = regler->priv;
-		priv->value_req.width = width;
-		priv->value_req.height = height;
-		switch (regler->value_position) {
-		case GTK_POS_LEFT:
-		case GTK_POS_RIGHT:
-			requisition->width += width + value_spacing;
-			if (height > requisition->height) {
-				requisition->height = height;
-			}
-			break;
-		case GTK_POS_TOP:
-		case GTK_POS_BOTTOM:
-			requisition->height += height + value_spacing;
-			if (width > requisition->width) {
-				requisition->width = width;
-			}
-			break;
+	PangoRectangle logical_rect1, logical_rect2;
+	GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(regler));
+	GtkBorder border;
+	get_value_border(GTK_WIDGET(regler), &border);
+	gint value_spacing;
+	gtk_widget_style_get(GTK_WIDGET(regler), "value-spacing", &value_spacing, NULL);
+	gchar *txt;
+	ensure_digits(regler);
+	txt = _gx_regler_format_value(regler, gtk_adjustment_get_lower(adj));
+	pango_layout_set_text(regler->value_layout, txt, -1);
+	g_free(txt);
+	pango_layout_get_pixel_extents(regler->value_layout, NULL, &logical_rect1);
+	txt = _gx_regler_format_value(regler, gtk_adjustment_get_upper(adj));
+	pango_layout_set_text(regler->value_layout, txt, -1);
+	g_free(txt);
+	pango_layout_get_pixel_extents(regler->value_layout, NULL, &logical_rect2);
+	gint height = max(logical_rect1.height,logical_rect2.height) + border.top + border.bottom;
+	gint width = max(logical_rect1.width,logical_rect2.width) + border.left + border.right;
+	GxReglerPrivate *priv = regler->priv;
+	priv->value_req.width = width;
+	priv->value_req.height = height;
+	switch (regler->value_position) {
+	case GTK_POS_LEFT:
+	case GTK_POS_RIGHT:
+		requisition->width += width + value_spacing;
+		if (height > requisition->height) {
+			requisition->height = height;
 		}
+		break;
+	case GTK_POS_TOP:
+	case GTK_POS_BOTTOM:
+		requisition->height += height + value_spacing;
+		if (width > requisition->width) {
+			requisition->width = width;
+		}
+		break;
 	}
 }
 
