@@ -378,7 +378,7 @@ bool GxConvolver::compute(int count, float* input1, float *input2, float *output
     memcpy(inpdata(0), input1, count * sizeof(float));
     memcpy(inpdata(1), input2, count * sizeof(float));
 
-    int flags = process();
+    int flags = process(sync);
 
     memcpy(output1, outdata(0), count * sizeof(float));
     memcpy(output2, outdata(1), count * sizeof(float));
@@ -400,9 +400,13 @@ public:
 	if (imprate != samplerate) {
 	    vec = resamp.process(imprate, *count, impresp, samplerate, count);
 	    if (!vec) {
-		gx_system::gx_print_error(
-		    "convolver",
-		    boost::format("failed to resample %1% -> %2%") % imprate % samplerate);
+		boost::format msg = boost::format("failed to resample %1% -> %2%") % imprate % samplerate;
+		if (samplerate) {
+		    gx_system::gx_print_error("convolver", msg);
+		} else {
+		    // not need for extra error when no samplerate (probably not connected to jack)
+		    gx_system::gx_print_warning("convolver", msg);
+		}
 		return 0;
 	    }
 	    return vec;
@@ -467,7 +471,7 @@ bool GxSimpleConvolver::compute(int count, float* input, float *output) {
     }
     memcpy(inpdata(0), input, count * sizeof(float));
 
-    int flags = process();
+    int flags = process(sync);
 
     memcpy(output, outdata(0), count * sizeof(float));
     return flags == 0;
