@@ -26,9 +26,6 @@
 
 #include "engine.h"     // NOLINT
 
-#include "gx_faust_plugins.h"
-#include "../plugins/pluginlib.h"
-
 namespace gx_engine {
 
 AudioVariables audio;
@@ -67,8 +64,58 @@ void ProcessingChainBase::set_stopped(bool v) {
     }
 }
 
+bool ProcessingChainBase::wait_rt_finished() {
+    if (stopped) {
+	return true;
+    }
+    timespec ts;
+    clock_gettime(CLOCK_REALTIME, &ts);
+    const long ns_in_sec = 1000000000;
+    ts.tv_nsec += ns_in_sec / 10;
+    if (ts.tv_nsec >= ns_in_sec) {
+	ts.tv_nsec -= ns_in_sec;
+	ts.tv_sec += 1;
+    }
+    while (sem_timedwait(&sync_sem, &ts) == -1) {
+	if (errno == EINTR) {
+	    continue;
+	}
+	if (errno == ETIMEDOUT) {
+	    gx_system::gx_print_warning("sem_timedwait", "timeout");
+	    return false;
+	}
+	gx_system::gx_print_error("sem_timedwait", "unknown error");
+	break;
+    }
+    return true;
+}
+
+void ProcessingChainBase::wait_latch() {
+    if (!latch) {
+	return;
+    }
+    if (!wait_rt_finished()) {
+	latch = false; // timeout; emergency measure for ladspa plugin...
+    }
+}
+
+void ProcessingChainBase::wait_ramp_down_finished() {
+    if (stopped) {
+	return;
+    }
+    while (ramp_mode == ramp_mode_down) {
+	if (!wait_rt_finished()) {
+	    break;
+	}
+    }
+}
+
 void ProcessingChainBase::start_ramp_up() {
+    RampMode rm = get_ramp_mode();
     if (!stopped) {
+	if (rm != ramp_mode_down_dead && rm != ramp_mode_down) {
+	    return;
+	}
 	set_ramp_value(0);
 	set_ramp_mode(ramp_mode_up_dead);
     }
@@ -76,7 +123,7 @@ void ProcessingChainBase::start_ramp_up() {
 
 void ProcessingChainBase::start_ramp_down() {
     RampMode rm = get_ramp_mode();
-    if (rm == ramp_mode_down_dead or rm == ramp_mode_down) {
+    if (rm == ramp_mode_down_dead || rm == ramp_mode_down) {
 	return;
     }
     int rv = min(steps_down,get_ramp_value());
@@ -90,12 +137,12 @@ void ProcessingChainBase::start_ramp_down() {
 
 void ProcessingChainBase::try_set_ramp_mode(RampMode oldmode, RampMode newmode, int oldrv, int newrv) {
     if (oldmode != newmode) {
-	if (!g_atomic_int_compare_and_exchange(&ramp_mode, oldmode, newmode)) {
+	if (!gx_system::atomic_compare_and_exchange(&ramp_mode, oldmode, newmode)) {
 	    return;
 	}
     }
     if (oldrv != newrv) {
-	if (!g_atomic_int_compare_and_exchange(&ramp_value, oldrv, newrv)) {
+	if (!gx_system::atomic_compare_and_exchange(&ramp_value, oldrv, newrv)) {
 	    return;
 	}
     }
@@ -187,15 +234,23 @@ void ProcessingChainBase::clear_module_states() {
 }
 
 void ProcessingChainBase::release() {
-    if (latch) {
-	wait_rt_finished();
-    }
+    wait_latch();
     for (list<Plugin*>::const_iterator p = to_release.begin(); p != to_release.end(); ++p) {
 	(*p)->pdef->activate_plugin(false, (*p)->pdef);
     }
     to_release.clear();
 }
 
+#ifndef NDEBUG
+void ProcessingChainBase::print_chain_state(const char *title) {
+    int val;
+    sem_getvalue(&sync_sem, &val);
+    printf("%s latch = %d, sync_sem = %d, stopped = %d, ramp_mode = %d\n",
+	   title, latch, val, stopped, ramp_mode);
+}
+#endif
+
+
 /****************************************************************
  ** MonoModuleChain, StereoModuleChain
  */
@@ -260,7 +315,7 @@ void MonoModuleChain::process(int count, float *input, float *output) {
     try_set_ramp_mode(rm, rm1, rv, rv1);
 }
 
-void StereoModuleChain::process(int count, float *input, float *output1, float *output2) {
+void StereoModuleChain::process(int count, float *input1, float *input2, float *output1, float *output2) {
     // run stereo rack
     RampMode rm = get_ramp_mode();
     if (rm == ramp_mode_down_dead) {
@@ -268,8 +323,8 @@ void StereoModuleChain::process(int count, float *input, float *output1, float *
 	memset(output2, 0, count*sizeof(float));
 	return;
     }
-    memcpy(output1, input, count*sizeof(float));
-    memcpy(output2, input, count*sizeof(float));
+    memcpy(output1, input1, count*sizeof(float));
+    memcpy(output2, input2, count*sizeof(float));
     for (stereochain_data *p = get_rt_chain(); p->func; ++p) {
 	(p->func)(count, output1, output2, output1, output2, p->plugin);
     }
@@ -333,8 +388,8 @@ void StereoModuleChain::process(int count, float *input, float *output1, float *
  */
 
 ModuleSelectorFromList::ModuleSelectorFromList(
-    ModuleSequencer& seq_, gx_ui::GxUI& ui, const char* id_, const char* name_,
-    PluginDef *plugins[], const char* select_id_,
+    EngineControl& seq_, gx_ui::GxUI& ui, const char* id_, const char* name_,
+    plugindef_creator plugins[], const char* select_id_,
     const char* select_name_, const char** groups_, int flags_)
     : ModuleSelector(seq_),
       PluginDef(),
@@ -348,9 +403,12 @@ ModuleSelectorFromList::ModuleSelectorFromList(
       plugin() {
     version = PLUGINDEF_VERSION;
     register_params = static_register;
-    modules = plugins;
-    PluginDef **p = plugins;
+    plugindef_creator *p = plugins;
     for (size = 0; *p; ++p, ++size);
+    modules = new PluginDef*[size];
+    for (unsigned int i = 0; i < size; ++i) {
+	modules[i] = plugins[i]();
+    }
     id = id_;
     name = name_;
     groups = groups_;
@@ -358,6 +416,10 @@ ModuleSelectorFromList::ModuleSelectorFromList(
     plugin = this;
 }
 
+ModuleSelectorFromList::~ModuleSelectorFromList() {
+    delete[] modules;
+}
+
 int ModuleSelectorFromList::register_parameter(const ParamReg &param) {
     value_pair *p = new value_pair[size+1];
     for (unsigned int i = 0; i < size; ++i) {
@@ -366,7 +428,7 @@ int ModuleSelectorFromList::register_parameter(const ParamReg &param) {
     }
     p[size].value_id = 0;
     p[size].value_label = 0;
-    param.registerUEnumVar(select_id, select_name, "S", "", p, &selector, 0, false);
+    param.registerUEnumVar(select_id, select_name, "S", "", p, &selector, 0);
     return 0;
 }
 
@@ -395,46 +457,135 @@ void ModuleSelectorFromList::set_module() {
     }
 }
 
+
+/****************************************************************
+ ** class EngineControl
+ */
+
+EngineControl::EngineControl()
+    : selectors(),
+      rack_changed(true),
+      ui(),
+      policy(),
+      priority(),
+      buffersize_change(),
+      samplerate_change(),
+      buffersize(0),
+      samplerate(0),
+      pluginlist(ui,*this) {
+}
+
+EngineControl::~EngineControl() {
+}
+
+void EngineControl::add_selector(ModuleSelector& sel) {
+    selectors.push_back(&sel);
+}
+
+void EngineControl::registerParameter(ParamMap& param, ParameterGroups& groups)
+{
+    pluginlist.registerParameter(param, groups);
+}
+
+void EngineControl::get_sched_priority(int &policy_, int &priority_, int prio_dim) { 
+    policy_ = policy;
+    priority_ = priority;
+    if (!prio_dim) {
+	return;
+    }
+    int min, max;
+    min = sched_get_priority_min(policy);
+    max = sched_get_priority_max(policy);
+    priority_ = priority - prio_dim;
+    if (priority_ > max) {
+	priority_ = max;
+    }
+    if (priority_ < min) {
+	priority_ = min;
+    }
+}
+
+void EngineControl::set_samplerate(unsigned int samplerate_) {
+    if (samplerate == samplerate_) {
+	return;
+    }
+    samplerate = samplerate_;
+    pluginlist.set_samplerate(samplerate);
+    samplerate_change(samplerate);
+}
+
+void EngineControl::set_buffersize(unsigned int buffersize_) {
+    if (buffersize == buffersize_) {
+	return;
+    }
+    buffersize = buffersize_;
+    buffersize_change(buffersize);
+}
+
+void EngineControl::init(unsigned int samplerate_, unsigned int buffersize_,
+			 int policy_, int priority_) {
+    if (policy_ != policy || priority_ != priority) {
+	policy = policy_;
+	priority = priority_;
+	set_buffersize(buffersize_);
+	set_samplerate(samplerate_);
+	return;
+    }
+    if (buffersize_ != buffersize) {
+	set_buffersize(buffersize_);
+    }
+    if (samplerate_ != samplerate) {
+	set_samplerate(samplerate_);
+    }
+}
+
+
 /****************************************************************
  ** ModuleSequencer
  */
 
-ModuleSequencer::ModuleSequencer():
-    selectors(),
-    rack_changed(true),
-    audio_mode(PGN_MODE_NORMAL),
-    policy(),
-    priority(),
-    ui(),
-    pluginlist(ui,*this),
-    mono_chain(),
-    stereo_chain(),
-    stateflags_mutex(),
-    stateflags(SF_INITIALIZING),
-    buffersize_change(),
-    samplerate_change() {
-}
-
-void ModuleSequencer::init(unsigned int samplerate, unsigned int buffersize,
-			   int policy_, int priority_) {
-    set_buffersize(buffersize);
-    set_samplerate(samplerate);
-    policy = policy_;
-    priority = priority_;
+ModuleSequencer::ModuleSequencer()
+    : EngineControl(),
+      audio_mode(PGN_MODE_NORMAL),
+      stateflags_mutex(),
+      stateflags(SF_INITIALIZING),
+      mono_chain(),
+      stereo_chain() {
 }
 
 ModuleSequencer::~ModuleSequencer() {
 }
 
-void ModuleSequencer::set_samplerate(unsigned int samplerate) {
-    pluginlist.set_samplerate(samplerate);
-    mono_chain.set_samplerate(samplerate);
-    stereo_chain.set_samplerate(samplerate);
-    samplerate_change(samplerate);
+void ModuleSequencer::start_ramp_up() {
+    mono_chain.start_ramp_up();
+    stereo_chain.start_ramp_up();
 }
 
-void ModuleSequencer::set_buffersize(unsigned int buffersize) {
-    buffersize_change(buffersize);
+void ModuleSequencer::start_ramp_down() {
+    mono_chain.start_ramp_down();
+    stereo_chain.start_ramp_down();
+}
+
+void ModuleSequencer::wait_ramp_down_finished() {
+    mono_chain.wait_ramp_down_finished();
+    stereo_chain.wait_ramp_down_finished();
+}
+
+bool ModuleSequencer::update_module_lists() {
+    if (!get_buffersize() || !get_samplerate()) {
+	return false;
+    }
+    if (prepare_module_lists()) {
+	commit_module_lists();
+	return true;
+    }
+    return false;
+}
+
+void ModuleSequencer::set_samplerate(unsigned int samplerate) {
+    mono_chain.set_samplerate(samplerate);
+    stereo_chain.set_samplerate(samplerate);
+    EngineControl::set_samplerate(samplerate);
 }
 
 bool ModuleSequencer::prepare_module_lists() {
@@ -479,10 +630,6 @@ void ModuleSequencer::commit_module_lists() {
     }
 }
 
-void ModuleSequencer::add_selector(ModuleSelector& sel) {
-    selectors.push_back(&sel);
-}
-
 void ModuleSequencer::set_stateflag(StateFlag flag) {
     if (stateflags & flag) {
 	return;
@@ -523,6 +670,14 @@ void ModuleSequencer::set_state(GxEngineState state) {
     set_rack_changed();
 }
 
+#ifndef NDEBUG
+void ModuleSequencer::print_engine_state() {
+    printf("stateflags = %d, audio_mode = %d\n", stateflags, audio_mode);
+    mono_chain.print_chain_state("mono  :");
+    stereo_chain.print_chain_state("stereo:");
+}
+#endif
+
 GxEngineState ModuleSequencer::get_state() {
     if (audio_mode & PGN_MODE_NORMAL) {
 	return kEngineOn;
@@ -536,225 +691,4 @@ GxEngineState ModuleSequencer::get_state() {
     }
 }
 
-void ModuleSequencer::registerParameter(gx_gui::ParameterGroups& groups)
-{
-    pluginlist.registerParameter(groups);
-}
-
-void ModuleSequencer::get_sched_priority(int &policy_, int &priority_, int prio_dim) { 
-    policy_ = policy;
-    priority_ = priority;
-    if (prio_dim) {
-	return;
-    }
-    int min, max;
-    min = sched_get_priority_min(policy);
-    max = sched_get_priority_max(policy);
-    priority_ = priority - prio_dim;
-    if (priority_ > max) {
-	priority_ = max;
-    }
-    if (priority_ < min) {
-	priority_ = min;
-    }
-}
-
-
-/****************************************************************
- ** class GxEngine
- */
-
-static PluginDef *builtin_crybaby_plugins[] = {
-    &gx_effects::crybaby::plugin,
-    &gx_effects::autowah::plugin,
-    0
-};
-
-static PluginDef *builtin_tonestack_plugins[] = {
-    &gx_tonestacks::tonestack_default::plugin,
-    &gx_tonestacks::tonestack_bassman::plugin,
-    &gx_tonestacks::tonestack_twin::plugin,
-    &gx_tonestacks::tonestack_princeton::plugin,
-    &gx_tonestacks::tonestack_jcm800::plugin,
-    &gx_tonestacks::tonestack_jcm2000::plugin,
-    &gx_tonestacks::tonestack_mlead::plugin,
-    &gx_tonestacks::tonestack_m2199::plugin,
-    &gx_tonestacks::tonestack_ac30::plugin,
-    &gx_tonestacks::tonestack_soldano::plugin,
-    &gx_tonestacks::tonestack_mesa::plugin,
-    &gx_tonestacks::tonestack_jtm45::plugin,
-    &gx_tonestacks::tonestack_ac15::plugin,
-    &gx_tonestacks::tonestack_peavey::plugin,
-    &gx_tonestacks::tonestack_ibanez::plugin,
-    &gx_tonestacks::tonestack_roland::plugin,
-    &gx_tonestacks::tonestack_ampeg::plugin,
-    &gx_tonestacks::tonestack_ampeg_rev::plugin,
-    &gx_tonestacks::tonestack_sovtek::plugin,
-    &gx_tonestacks::tonestack_bogner::plugin,
-    &gx_tonestacks::tonestack_groove::plugin,
-    &gx_tonestacks::tonestack_crunch::plugin,
-    &gx_tonestacks::tonestack_fender_blues::plugin,
-    &gx_tonestacks::tonestack_fender_default::plugin,
-    &gx_tonestacks::tonestack_fender_deville::plugin,
-    &gx_tonestacks::tonestack_gibsen::plugin,
-    0
-};
-
-static PluginDef *builtin_amp_plugins[] = {
-    &gx_amps::gxamp::plugin,
-    &gx_amps::gxamp3::plugin,
-    &gx_amps::gxamp14::plugin,
-    &gx_amps::gxamp10::plugin,
-
-    &gx_amps::gxamp2::plugin,
-
-    &gx_amps::gxamp9::plugin,
-    &gx_amps::gxamp11::plugin,
-    &gx_amps::gxamp17::plugin,
-    &gx_amps::gxamp13::plugin,
-
-    &gx_amps::gxamp5::plugin,
-    &gx_amps::gxamp4::plugin,
-    &gx_amps::gxamp15::plugin,
-    &gx_amps::gxamp12::plugin,
-
-    &gx_amps::gxamp7::plugin,
-    &gx_amps::gxamp8::plugin,
-    &gx_amps::gxamp16::plugin,
-    &gx_amps::gxamp6::plugin,
-    0
-};
-
-static const char* ampstack_groups[] = {
-    ".amp2.stage1",  N_("Tube1"),
-    ".amp2.stage2",  N_("Tube2"),
-    ".tube",         N_("Tube 1"),
-    ".gxdistortion", N_("Multi Band Distortion"),
-    0
-};
-
-GxEngine::GxEngine(const string& plugin_dir, gx_gui::ParameterGroups& groups)
-    : ModuleSequencer(),
-      resamp(),
-      // ModuleSelector's
-      crybaby(
-	  *this, ui, "crybaby", N_("Crybaby"), builtin_crybaby_plugins,
-	  "crybaby.autowah", _("select"), 0, PGN_POST_PRE),
-      tonestack(
-	  *this, ui, "amp.tonestack", N_("Tonestack"),
-	  builtin_tonestack_plugins, "amp.tonestack.select",
-	  _("select"), 0, PGN_POST_PRE),
-      ampstack(
-	  *this, ui, "ampstack", "?Tube", builtin_amp_plugins,
-	  "tube.select", _("select"), ampstack_groups),
-      // internal audio modules
-      noisegate(),
-      monomute(),
-      stereomute(),
-      tuner(*this),
-      midiaudiobuffer(tuner),
-      maxlevel(),
-      oscilloscope(&ui, *this),
-      convolver(*this),
-      cabinet(*this, resamp),
-      contrast(*this, resamp) {
-    tuner.set_dep_module(&midiaudiobuffer.plugin);
-
-    load_static_plugins();
-
-    // loaded from shared libs
-    pluginlist.load_from_path(plugin_dir, PLUGIN_POS_RACK);
-
-    // selector objects to switch "alternative" modules
-    add_selector(ampstack);
-    add_selector(crybaby);
-    add_selector(tonestack);
-    add_selector(tuner);
-
-    registerParameter(groups);
-
-#ifndef NDEBUG
-    pluginlist.printlist();
-#endif
-}
-
-GxEngine::~GxEngine() {
-}
-
-void GxEngine::load_static_plugins() {
-    PluginList& pl = pluginlist; // just a shortcut
-
-    // * mono amp input position *
-
-    pl.add(&tuner.plugin,                        PLUGIN_POS_START, PGN_PRE|PGN_MODE_NORMAL|PGN_MODE_BYPASS|PGN_MODE_MUTE);
-    pl.add(&midiaudiobuffer.plugin,              PLUGIN_POS_START, PGN_GUI|PGN_PRE|PGN_MODE_NORMAL|PGN_MODE_BYPASS);
-    pl.add(&noisegate.inputlevel,                PLUGIN_POS_START, PGN_GUI|PGN_PRE);
-    pl.add(&gx_effects::noise_shaper::plugin,    PLUGIN_POS_START, PGN_GUI|PGN_PRE);
-
-    // rack pre mono modules inserted here
-
-    pl.add(builtin_amp_plugins,                  PLUGIN_POS_START, PGN_ALTERNATIVE|PGN_POST);
-    pl.add(&ampstack.plugin,                     PLUGIN_POS_START, PGN_POST);
-    pl.add(&gx_effects::softclip::plugin,        PLUGIN_POS_START, PGN_GUI|PGN_POST);
-
-    // rack post mono modules inserted here
-
-    pl.add(&gx_effects::bassbooster::plugin,     PLUGIN_POS_END, PGN_GUI|PGN_POST);
-    pl.add(&gx_effects::gx_ampout::plugin,       PLUGIN_POS_END, PGN_GUI|PGN_POST);
-    pl.add(&contrast.plugin,                     PLUGIN_POS_END, PGN_GUI|PGN_POST);
-    pl.add(&noisegate.outputgate,                PLUGIN_POS_END, PGN_POST);
-    pl.add(&monomute,                            PLUGIN_POS_END, PGN_POST|PGN_MODE_MUTE);
-
-    // * amp insert position (stereo amp input) *
-
-    pl.add(&gx_effects::gxfeed::plugin,          PLUGIN_POS_START);
-
-    // rack stereo modules inserted here
-
-    pl.add(&gx_effects::gx_outputlevel::plugin,  PLUGIN_POS_END);
-    pl.add(&balance::plugin,                     PLUGIN_POS_END, PGN_MODE_BYPASS);
-    pl.add(&stereomute,                          PLUGIN_POS_END, PGN_MODE_MUTE);
-    pl.add(&maxlevel,                            PLUGIN_POS_END, PGN_MODE_NORMAL|PGN_MODE_BYPASS);
-
-    // * fx amp output *
-
-    // dynamic rack modules
-    // builtin 
-    pl.add(builtin_crybaby_plugins,              PLUGIN_POS_RACK, PGN_ALTERNATIVE);
-    pl.add(builtin_tonestack_plugins,            PLUGIN_POS_RACK, PGN_ALTERNATIVE);
-
-    // mono
-    pl.add(&gx_effects::low_high_pass::plugin,   PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::selecteq::plugin,        PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&crybaby.plugin,                      PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::gx_distortion::plugin,   PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::impulseresponse::plugin, PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::compressor::plugin,      PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::overdrive::plugin,       PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::echo::plugin,            PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::delay::plugin,           PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::freeverb::plugin,        PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&oscilloscope.plugin,                 PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::biquad::plugin,          PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::tremolo::plugin,         PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::phaser_mono::plugin,     PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::chorus_mono::plugin,     PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::flanger_mono::plugin,    PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::gx_feedback::plugin,     PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&tonestack.plugin,                    PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&cabinet.plugin,                      PLUGIN_POS_RACK, PGN_GUI);
-    // stereo
-    pl.add(&gx_effects::chorus::plugin,          PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::flanger::plugin,         PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::phaser::plugin,          PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::stereodelay::plugin,     PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::stereoecho::plugin,      PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::moog::plugin,            PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_amps::gx_ampmodul::plugin,        PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::tonecontroll::plugin,    PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&convolver.plugin,                    PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&gx_effects::stereoverb::plugin,      PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&pluginlib::zita_rev1::plugin,        PLUGIN_POS_RACK);
-}
-
 } // end namespace gx_engine
