@@ -29,68 +29,45 @@
 namespace gx_engine {
 
 void MidiVariables::register_parameter(const ParamReg& reg) {
-    gx_gui::registerNonMidiParam("midi_out.midistat",        &midistat, false);
-    gx_gui::registerNonMidiParam("midi_out.midistat1",       &midistat1, false);
-    gx_gui::registerNonMidiParam("midi_out.midistat2",       &midistat2, false);
-    gx_gui::registerNonMidiParam("midi_out.midistat3",       &midistat3, false);
+    reg.registerNonMidiVar("midi_out.midistat",  &midistat,  false);
+    reg.registerNonMidiVar("midi_out.midistat1", &midistat1, false);
+    reg.registerNonMidiVar("midi_out.midistat2", &midistat2, false);
+    reg.registerNonMidiVar("midi_out.midistat3", &midistat3, false);
 
-    gx_gui::registerParam("midi_out.channel_1.velocity", "velocity", &fslider26,
-                           64.f, 0.f, 127.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_1.volume", "volume", &fslider46,
-                           64.f, 0.f, 127.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_1.autogain", "autogain", &fautogain);
-    gx_gui::registerParam("midi_out.channel_1.channel", "channel 1", &fslider30,
-                           0.f, 0.f, 16.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_1.program", "program", &fslider31,
-                           0.f, 0.f, 248.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_1.oktave", "oktave", &fslider29,
-                           0.f, -2.f, 2.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_1.sensity", "sensity", &fslider27,
-                           20.f, 1.f, 500.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_1.auto_pitch", "auto pitch", &fpitch);
+    reg.registerVar("midi_out.channel_1.velocity", "velocity", "S", "", &fslider26, 64, 0, 127, 1);
+    reg.registerVar("midi_out.channel_1.volume", "volume", "S", "", &fslider46, 64, 0, 127, 1);
+    reg.registerBoolVar("midi_out.channel_1.autogain", "autogain", "B", "", &fautogain, false);
+    reg.registerVar("midi_out.channel_1.channel", "channel 1", "S", "", &fslider30, 0, 0, 16, 1);
+    reg.registerVar("midi_out.channel_1.program", "program", "S", "", &fslider31, 0, 0, 248, 1);
+    reg.registerVar("midi_out.channel_1.oktave", "oktave", "S", "", &fslider29, 0, -2, 2, 1);
+    reg.registerVar("midi_out.channel_1.sensity", "sensity", "S", "", &fslider27, 20, 1, 500, 1);
+    reg.registerBoolVar("midi_out.channel_1.auto_pitch", "auto pitch", "B", "", &fpitch, false);
 
-    gx_gui::registerParam("midi_out.channel_2.on_off", "on/off", &fcheckbox10, 0);
-    gx_gui::registerParam("midi_out.channel_2.velocity", "velocity", &fslider32,
-                           64.f, 0.f, 127.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_2.volume", "volume", &fslider47,
-                           64.f, 0.f, 127.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_2.autogain", "autogain", &fautogain1);
-    gx_gui::registerParam("midi_out.channel_2.channel", "channel 2", &fslider35,
-                           0.f, 0.f, 16.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_2.program", "program", &fslider36,
-                           0.f, 0.f, 248.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_2.oktave", "oktave", &fslider34,
-                           0.f, -2.f, 2.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_2.sensity", "sensity", &fslider33,
-                           20.f, 1.f, 500.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_2.auto_pitch", "auto pitch", &fpitch1);
+    reg.registerBoolVar("midi_out.channel_2.on_off", "on/off", "B", "", &fcheckbox10, false);
+    reg.registerVar("midi_out.channel_2.velocity", "velocity", "S", "", &fslider32, 64, 0, 127, 1);
+    reg.registerVar("midi_out.channel_2.volume", "volume", "S", "", &fslider47, 64, 0, 127, 1);
+    reg.registerBoolVar("midi_out.channel_2.autogain", "autogain", "B", "", &fautogain1, false);
+    reg.registerVar("midi_out.channel_2.channel", "channel 2", "S", "", &fslider35, 0, 0, 16, 1);
+    reg.registerVar("midi_out.channel_2.program", "program", "S", "", &fslider36, 0, 0, 248, 1);
+    reg.registerVar("midi_out.channel_2.oktave", "oktave", "S", "", &fslider34, 0, -2, 2, 1);
+    reg.registerVar("midi_out.channel_2.sensity", "sensity", "S", "", &fslider33, 20, 1, 500, 1);
+    reg.registerBoolVar("midi_out.channel_2.auto_pitch", "auto pitch", "B", "", &fpitch1, false);
 
-    gx_gui::registerParam("midi_out.channel_3.on_off", "on/off", &fcheckbox11, 0);
-    gx_gui::registerParam("midi_out.channel_3.velocity", "velocity", &fslider40,
-                           64.f, 0.f, 127.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_3.volume", "volume", &fslider48,
-                           64.f, 0.f, 127.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_3.autogain", "autogain", &fautogain2);
-    gx_gui::registerParam("midi_out.channel_3.channel", "channel 3", &fslider44,
-                           0.f, 0.f, 16.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_3.program", "program", &fslider43,
-                           0.f, 0.f, 248.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_3.oktave", "oktave", &fslider42,
-                           0.f, -2.f, 2.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_3.sensity", "sensity", &fslider41,
-                           20.f, 1.f, 500.f, 1.f);
-    gx_gui::registerParam("midi_out.channel_3.auto_pitch", "auto pitch", &fpitch2);
+    reg.registerBoolVar("midi_out.channel_3.on_off", "on/off", "B", "", &fcheckbox11, false);
+    reg.registerVar("midi_out.channel_3.velocity", "velocity", "S", "", &fslider40, 64, 0, 127, 1);
+    reg.registerVar("midi_out.channel_3.volume", "volume", "S", "", &fslider48, 64, 0, 127, 1);
+    reg.registerBoolVar("midi_out.channel_3.autogain", "autogain", "B", "", &fautogain2, false);
+    reg.registerVar("midi_out.channel_3.channel", "channel 3", "S", "", &fslider44, 0, 0, 16, 1);
+    reg.registerVar("midi_out.channel_3.program", "program", "S", "", &fslider43, 0, 0, 248, 1);
+    reg.registerVar("midi_out.channel_3.oktave", "oktave", "S", "", &fslider42, 0, -2, 2, 1);
+    reg.registerVar("midi_out.channel_3.sensity", "sensity", "S", "", &fslider41, 20, 1, 500, 1);
+    reg.registerBoolVar("midi_out.channel_3.auto_pitch", "auto pitch", "B", "", &fpitch2, false);
 
-    gx_gui::registerParam("beat_detector.stepper", "stepper", &fslider39,
-                           1.f, 1.f, 32.f, 1.f);
-    gx_gui::registerParam("beat_detector.note_off", "note off", &fslider37,
-                           2.f, 1.f, 127.f, 1.f);
-    gx_gui::registerParam("beat_detector.atack_gain", "atack gain", &fslider45,
-                           5.f, 1.f, 127.f, 1.f);
-    gx_gui::registerParam("beat_detector.beat_gain", "beat gain", &fslider38,
-                           1.f, 0.0f, 127.f, 1.f);
-    gx_gui::registerParam("beat_detector.midi_gain", "midi gain", &midi_gain,
-                           0.f, -40.f, 60.f, 1.f);
+    reg.registerVar("beat_detector.stepper", "stepper", "S", "", &fslider39, 1, 1, 32, 1);
+    reg.registerVar("beat_detector.note_off", "note off", "S", "", &fslider37, 2, 1, 127, 1);
+    reg.registerVar("beat_detector.atack_gain", "atack gain", "S", "", &fslider45, 5, 1, 127, 1);
+    reg.registerVar("beat_detector.beat_gain", "beat gain", "S", "", &fslider38, 1, 0.0f, 127, 1);
+    reg.registerVar("beat_detector.midi_gain", "midi gain", "S", "", &midi_gain, 0, -40, 60, 1);
 }
 
 void MidiVariables::init(int samplingFreq) {
@@ -197,7 +174,7 @@ void MidiVariables::process_midi(int len, float *audiodata, void *midi_port_buf,
 
 	    if (rms >= (Beat_is + fTemps38)) {
 		// Beat_is = rms;
-		Beat_is += static_cast<int>(rms*0.1);
+		Beat_is += rms*0.1;
 		send+=step;
 		if (fcheckbox10 ) send1+=step;
 		if (fcheckbox11 ) send2+=step;
@@ -222,7 +199,7 @@ void MidiVariables::process_midi(int len, float *audiodata, void *midi_port_buf,
 	    }
 
 	    if (send > iTemps27) {     // 20
-		if (static_cast<int>(fautogain) == 1) {
+		if (fautogain) {
 		    iTemps46 = static_cast<int>(beat0);
 		    if ( iTemps46 < 0) iTemps46 = 0;
 		    else if ( iTemps46 > 127) iTemps46 = 127;
@@ -244,7 +221,7 @@ void MidiVariables::process_midi(int len, float *audiodata, void *midi_port_buf,
 		}
 
 		noten = preNote + iTemps29;
-		send = 0;
+		send = 1;
 		midistat = true;
 		midistat1 = true;
 
@@ -305,7 +282,7 @@ void MidiVariables::process_midi(int len, float *audiodata, void *midi_port_buf,
 		}
 
 		if (send1 > iTemps33) {
-		    if (static_cast<int>(fautogain1) == 1) {
+		    if (fautogain1) {
 			iTemps47 = static_cast<int>(beat0);
 			if ( iTemps47 < 0) iTemps47 = 0;
 			else if ( iTemps47 > 127) iTemps47 = 127;
@@ -328,7 +305,7 @@ void MidiVariables::process_midi(int len, float *audiodata, void *midi_port_buf,
 		    }
 
 		    noten1 = preNote + iTemps34;
-		    send1 = 0;
+		    send1 = 1;
 		    midistat = true;
 		    midistat2 = true;
 		    if ((noten1 >= 0)&&(noten1 <= 127)) {
@@ -389,7 +366,7 @@ void MidiVariables::process_midi(int len, float *audiodata, void *midi_port_buf,
 		}
 
 		if (send2 > iTemps41) {   // 20
-		    if (static_cast<int>(fautogain2) == 1) {
+		    if (fautogain2) {
 			iTemps48 = static_cast<int>(beat0);
 			if ( iTemps48 < 0) iTemps48 = 0;
 			else if ( iTemps48 > 127) iTemps48 = 127;
@@ -424,7 +401,7 @@ void MidiVariables::process_midi(int len, float *audiodata, void *midi_port_buf,
 		    }
 
 		    noten2 = preNote + iTemps42;
-		    send2 = 0;
+		    send2 = 1;
 		    midistat = true;
 		    midistat3 = true;
 
@@ -461,48 +438,57 @@ void MidiVariables::process_midi(int len, float *audiodata, void *midi_port_buf,
 	    // end if playmidi = 1
 	} else {
 	    if (weg > iTemps37 || jcpu_load > 64.0) {
-		send = send1 = send2 = 0;
+        
 		Beat_is = static_cast<int>(fTemps45);
 		if (weg <  iTemps37a) {   // 5.0
-		    midistat = true;
-		    midi_send = jack_midi_event_reserve(midi_port_buf, i, 3);
+            if (send) {
+                send = 0;
+                midistat = true;
+                midi_send = jack_midi_event_reserve(midi_port_buf, i, 3);
 
-		    if (midi_send) {
-			// velocity
-			midi_send[2] = static_cast<int>(iTemps26);
-			// fix me all notes off
-			midi_send[1] = 123;
-			// controller
-			midi_send[0] = 0xB0 | static_cast<int>(iTemps30);
+                if (midi_send) {
+                // velocity
+                midi_send[2] = static_cast<int>(iTemps26);
+                // fix me all notes off
+                midi_send[1] = 123;
+                // controller
+                midi_send[0] = 0xB0 | static_cast<int>(iTemps30);
 		    }
+        }
 
 		    if (fcheckbox10) {
-			midistat = true;
-			midi_send1 = jack_midi_event_reserve(midi_port_buf, i, 3);
+                if (send1) {
+                    send1 = 0;
+                    midistat = true;
+                    midi_send1 = jack_midi_event_reserve(midi_port_buf, i, 3);
 
-			if (midi_send1) {
-			    // velocity
-			    midi_send1[2] = static_cast<int>(iTemps32);
-			    // fix me all notes off
-			    midi_send1[1] = 123;
-			    // controller
-			    midi_send1[0] = 0xB0 |  static_cast<int>(iTemps35);
-			}
-		    }
+                    if (midi_send1) {
+                        // velocity
+                        midi_send1[2] = static_cast<int>(iTemps32);
+                        // fix me all notes off
+                        midi_send1[1] = 123;
+                        // controller
+                        midi_send1[0] = 0xB0 |  static_cast<int>(iTemps35);
+                    }
+                }
+            }
 
 		    if (fcheckbox11) {
-			midistat = true;
-			midi_send2 = jack_midi_event_reserve(midi_port_buf, i, 3);
+                if (send2) {
+                    send2 = 0;
+                    midistat = true;
+                    midi_send2 = jack_midi_event_reserve(midi_port_buf, i, 3);
 
-			if (midi_send2) {
-			    // velocity
-			    midi_send2[2] = static_cast<int>(iTemps40);
-			    // fix me all notes off
-			    midi_send2[1] =  123;
-			    // controller
-			    midi_send2[0] = 0xB0 | static_cast<int>(iTemps44);
-			}
-		    }
+                    if (midi_send2) {
+                        // velocity
+                        midi_send2[2] = static_cast<int>(iTemps40);
+                        // fix me all notes off
+                        midi_send2[1] =  123;
+                        // controller
+                        midi_send2[0] = 0xB0 | static_cast<int>(iTemps44);
+                    }
+                }
+            }
 		    midistat = midistat1 = midistat2 = midistat3 = false;
 		}
 	    }
