@@ -32,6 +32,7 @@
 #include <iostream>        // NOLINT
 #include <gdk/gdkkeysyms.h>            // NOLINT
 #include <gtkmm/filechooserdialog.h>   // NOLINT
+#include <gxw/GxControlParameter.h>      // NOLINT
 
 
 namespace gx_preset {
@@ -472,37 +473,13 @@ void GxPreset::gx_save_preset(const char* presname, bool expand_menu) {
     gx_system::gx_print_info(_("Preset Saving"), string(_("saved preset ")) + string(presname));
 }
 
-///------factory presets--------///
-static bool gx_load_preset_from_factory(const char* presname, int i) {
-    GxSettings& gxs = GxSettings::get_instance();
-    gxs.load(GxSettings::factory,presname,gxs.get_factory(i));
-    return true;
-}
-
 // ----menu funktion load preset from factory
-void GxPreset::gx_load_factory_preset(Gtk::MenuItem *menuitem, int load_preset) {
+void GxPreset::gx_load_factory_preset(
+    const string& factory_name, const string& preset_name) {
+
     gx_gui::guivar.show_patch_info -= 9999;
-    // retrieve preset name
-    vector<Gtk::MenuItem*>::iterator it = gxpreset.fpm_list[load_preset].begin();
-    vector<string>::iterator its = gxpreset.fplist[load_preset].begin();
-    for (it = gxpreset.fpm_list[load_preset].begin(); it != gxpreset.fpm_list[load_preset].end(); ++it) {
-        if (menuitem == *it)
-            break;
-        ++its;
-    }
-
-    string preset_name = *its;
-    // recall preset by name
-    // Note: the UI does not know anything about gx_head's directory stuff
-    // Need to pass it on
-    bool preset_ok = gx_load_preset_from_factory(preset_name.c_str(), load_preset);
-
-    // check audio.result
-    if (!preset_ok) {
-        gx_system::gx_print_error(
-	    _("Preset Loading"), string(_("Could not load preset ")) + preset_name);
-        return;
-    }
+    GxSettings& gxs = GxSettings::get_instance();
+    gxs.load(GxSettings::factory, preset_name, factory_name);
     gx_gui::guivar.show_patch_info = -1;
     // print out info
     gx_system::gx_print_info(
@@ -510,31 +487,15 @@ void GxPreset::gx_load_factory_preset(Gtk::MenuItem *menuitem, int load_preset)
 }
 
 // load the factory preset file
-void GxPreset::gx_load_factory_file(int i) {
-    // initialize list
-    fplist[i].clear();
-    // initialize menu pointer list
-    fpm_list[i].clear();
-
-    GxSettings& gxs = GxSettings::get_instance();
-    gxs.fill_factory_preset_names(gxs.get_factory(i), fplist[i]);
-
-    vector<string>::iterator it;
-    for (it = fplist[i].begin() ; it != fplist[i].end(); ++it) {
-        // menu
-	gx_gui::GxMainInterface& gui = gx_gui::GxMainInterface::get_instance();
-	Gtk::Menu& menu = gui.mainmenu.factory_sub_menu[i];
-        vector<Gtk::MenuItem*>::iterator its;
-        // create item
-        string presna = *it;
-	Gtk::MenuItem *menuitem = manage(new Gtk::MenuItem(presna, true));
-	menuitem->signal_activate().connect(
-	    sigc::bind(sigc::ptr_fun(gx_load_factory_preset), menuitem, i));
-	menu.append(*menuitem);
-        its = fpm_list[i].end();
-        // add a pointer to the menuitem to the preset menu list
-        fpm_list[i].insert(its, menuitem);
+void GxPreset::gx_append_factory_file(const string& name, Gtk::Menu& menu) {
+    vector<string> fp;
+    GxSettings::get_instance().fill_factory_preset_names(name, fp);
+    for (vector<string>::iterator it = fp.begin() ; it != fp.end(); ++it) {
+	Gtk::MenuItem *menuitem = new Gtk::MenuItem(*it, true);
         menuitem->show();
+	menuitem->signal_activate().connect(
+	    sigc::bind(sigc::ptr_fun(gx_load_factory_preset), name, *it));
+	menu.append(*manage(menuitem));
     }
 }
 
@@ -794,23 +755,29 @@ namespace gx_gui {
 
 /* ----- Menu check item signaled from parameter ------ */
 
+MenuCheckItem::MenuCheckItem(const char *label, gx_engine::ParamMap& pmap, const char *id, bool sv)
+  : Gtk::CheckMenuItem(label, true),
+    param() {
+    set_parameter(pmap.reg_switch(id, false, sv));
+}
+
 void MenuCheckItem::on_my_activate() {
     param->set(get_active());
 }
 
-SwitchParameter* MenuCheckItem::get_parameter() {
+gx_engine::SwitchParameter* MenuCheckItem::get_parameter() {
     return param;
 }
 
-void MenuCheckItem::set_parameter(SwitchParameter *p) {
+void MenuCheckItem::set_parameter(gx_engine::SwitchParameter *p) {
     param = p;
-    parameter_map.insert(p);
     p->signal_changed().connect(sigc::mem_fun(*this, &MenuCheckItem::set_active));
+    set_active(p->get());
     signal_activate().connect(
         sigc::mem_fun(*this, &MenuCheckItem::on_my_activate));
 }
 
-void MenuCheckItem::add_parameter(SwitchParameter *p) {
+void MenuCheckItem::add_parameter(gx_engine::SwitchParameter *p) {
     param = p;
     p->signal_changed().connect(sigc::mem_fun(*this, &MenuCheckItem::set_active));
     signal_activate().connect(
@@ -821,13 +788,12 @@ void RadioCheckItem::on_my_toggled() {
     param->set(get_active());
 }
 
-SwitchParameter* RadioCheckItem::get_parameter() {
+gx_engine::SwitchParameter* RadioCheckItem::get_parameter() {
     return param;
 }
 
-void RadioCheckItem::set_parameter(SwitchParameter *p) {
+void RadioCheckItem::set_parameter(gx_engine::SwitchParameter *p) {
     param = p;
-    parameter_map.insert(p);
     p->signal_changed().connect(sigc::mem_fun(*this, &RadioCheckItem::set_active));
     signal_activate().connect(
         sigc::mem_fun(*this, &RadioCheckItem::on_my_toggled));
@@ -895,7 +861,7 @@ bool gx_start_jack_dialog() {
 
     default:
     case GTK_RESPONSE_YES:
-        retstat = gx_system::gx_start_jack();
+        retstat = true; //gx_system::gx_start_jack();
         break;
     }
 
@@ -906,7 +872,7 @@ bool gx_start_jack_dialog() {
 /* -------------- for thread that checks jackd liveliness -------------- */
 bool GxMainInterface::survive_jack_shutdown() {
     // return if jack is not down
-    if (gx_system::gx_system_call("pgrep", "jackd", true) == SYSTEM_OK) {
+    if (gx_system::gx_system_call("pgrep jackd", true) == SYSTEM_OK) {
         if (jack.is_jack_down()) {
         sleep(2);
 	    jack.set_jack_down(false);
@@ -1028,6 +994,19 @@ void gx_patch() {
     gtk_widget_show_all(gw.patch_info);
 }
 
+void gx_show_help() {
+    GError *error = NULL;
+    gtk_show_uri(gdk_screen_get_default(), "http://sourceforge.net/apps/mediawiki/guitarix/index.php?title=Main_Page",
+    gtk_get_current_event_time(), &error);
+    if (error)
+    {
+        gx_system::gx_print_error("guitarix help",
+				  _("failed to load online help   "));
+        g_error_free(error);
+    } 
+
+}
+
 // ----menu funktion about
 void gx_show_about() {
     static string about;
@@ -1058,22 +1037,11 @@ void gx_show_about() {
 }
 
 
-void gx_reset_effects(GtkWidget *widget, gpointer data ) {
-    string pos(".position");
-    for (ParamMap::iterator i = parameter_map.begin(); i != parameter_map.end(); ++i) {
-        string id = i->first;
-        if (id.size() > pos.size() &&
-            id.compare(id.size()-pos.size(), pos.size(), pos) == 0) {
-            i->second->set_std_value();
-        }
-    }
-}
-
 // reset the extended sliders to default settings
-void gx_reset_units(Glib::ustring group_id) {
+void gx_reset_units(gx_engine::ParamMap& pmap, Glib::ustring group_id) {
     group_id += ".";
     string on_off = group_id + "on_off";
-    for (ParamMap::iterator i = parameter_map.begin(); i != parameter_map.end(); ++i) {
+    for (gx_engine::ParamMap::iterator i = pmap.begin(); i != pmap.end(); ++i) {
         if (i->first.compare(0, group_id.size(), group_id) == 0) {
             if (i->second->isControllable()) {
                 string id = i->first;
@@ -1086,20 +1054,29 @@ void gx_reset_units(Glib::ustring group_id) {
 }
 
 gboolean gx_set_resizeable(gpointer data) {
-    if (!gtk_window_get_resizable(GTK_WINDOW(data)))
+    if (!gtk_window_get_resizable(GTK_WINDOW(data)) &&
+      GDK_IS_WINDOW (GTK_WIDGET(data)->window)) {
         gtk_window_set_resizable(GTK_WINDOW(data), TRUE);
-    return false;
+        return false;
+    } else {
+        return true;
+    }
 }
 
 gboolean gx_set_sresizeable(gpointer data) {
-    if (!gtk_window_get_resizable(GTK_WINDOW(data)))
+    if (!gtk_window_get_resizable(GTK_WINDOW(data)) &&
+      GDK_IS_WINDOW (GTK_WIDGET(data)->window))
         gtk_window_set_resizable(GTK_WINDOW(data), FALSE);
     return false;
 }
 
 gboolean gx_set_default(gpointer data) {
-    gtk_widget_set_size_request(GTK_WIDGET(data), -1, -1);
-    return false;
+    if(GDK_IS_WINDOW (GTK_WIDGET(data)->window)) {
+        gtk_widget_set_size_request(GTK_WIDGET(data), -1, -1);
+        return false;
+    } else {
+        return true;
+    }
 }
 
 gboolean gx_set_default_size(gpointer data) {
@@ -1112,13 +1089,16 @@ gboolean gx_set_default_ssize(gpointer data) {
     return false;
 }
 
+
 // ----- show extendend settings slider
 void gx_show_extended_settings(GtkWidget *widget, gpointer data) {
     gx_gui::GxMainInterface& gui = gx_gui::GxMainInterface::get_instance();
+    GtkAllocation alloc;
+    gtk_widget_get_allocation (GTK_WIDGET(gui.RBox), &alloc);
     if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget)) == TRUE) {
 
         GtkWidget *plug = gtk_widget_get_parent(GTK_WIDGET(data));
-        gtk_widget_show_all(GTK_WIDGET(plug));
+        gtk_widget_show(GTK_WIDGET(plug));
         GtkWidget *vbox = gtk_widget_get_parent(GTK_WIDGET(plug));
         GtkWidget *box1 = gtk_widget_get_parent(GTK_WIDGET(vbox));
         vbox = gtk_widget_get_parent(GTK_WIDGET(box1));
@@ -1132,17 +1112,20 @@ void gx_show_extended_settings(GtkWidget *widget, gpointer data) {
             if (gui.fWindow.get_resizable())
                 gui.fWindow.set_resizable(false);
             gtk_widget_show(GTK_WIDGET(vbox));
-
-            gtk_widget_set_size_request(GTK_WIDGET(gui.RBox), -1, 460);
-                if (guivar.g_threads[7] == 0 || g_main_context_find_source_by_id
-                                        (NULL, guivar.g_threads[7]) == NULL)
-                    guivar.g_threads[7] = g_timeout_add_full(
-			G_PRIORITY_HIGH_IDLE + 10, 40, gx_gui::gx_set_resizeable,
-			gpointer(gui.fWindow.gobj()), NULL);
-                if (guivar.g_threads[6] == 0 || g_main_context_find_source_by_id
-                                        (NULL, guivar.g_threads[6]) == NULL)
-                    guivar.g_threads[6] = g_timeout_add_full(G_PRIORITY_HIGH_IDLE + 10, 50,
-                                   gx_gui::gx_set_default, gpointer(gui.RBox), NULL);
+            if (GDK_IS_WINDOW (gui.RBox->window)) {
+                gtk_widget_set_size_request(GTK_WIDGET(gui.RBox), -1, alloc.height);
+            } else {
+                gtk_widget_set_size_request(GTK_WIDGET(gui.RBox), -1, 460);
+            }
+            if (guivar.g_threads[7] == 0 || g_main_context_find_source_by_id
+                                    (NULL, guivar.g_threads[7]) == NULL)
+                guivar.g_threads[7] = g_timeout_add_full(
+                    G_PRIORITY_HIGH_IDLE + 10, 40, gx_gui::gx_set_resizeable,
+                    gpointer(gui.fWindow.gobj()), NULL);
+            if (guivar.g_threads[6] == 0 || g_main_context_find_source_by_id
+                                    (NULL, guivar.g_threads[6]) == NULL)
+                guivar.g_threads[6] = g_timeout_add_full(G_PRIORITY_HIGH_IDLE + 10, 50,
+                               gx_gui::gx_set_default, gpointer(gui.RBox), NULL);
         }
     } else {
         GtkWidget *plug = gtk_widget_get_parent(GTK_WIDGET(data));
@@ -1161,16 +1144,20 @@ void gx_show_extended_settings(GtkWidget *widget, gpointer data) {
                 gui.fWindow.set_resizable(false);
             gtk_widget_show(GTK_WIDGET(vbox));
 
-            gtk_widget_set_size_request(GTK_WIDGET(gui.RBox), -1, 460);
-                if (guivar.g_threads[7] == 0 || g_main_context_find_source_by_id
+            if (GDK_IS_WINDOW (gui.RBox->window)) {
+                gtk_widget_set_size_request(GTK_WIDGET(gui.RBox), -1, alloc.height);
+            } else {
+                gtk_widget_set_size_request(GTK_WIDGET(gui.RBox), -1, 460);
+            }    
+            if (guivar.g_threads[7] == 0 || g_main_context_find_source_by_id
                                         (NULL, guivar.g_threads[7]) == NULL)
-                    guivar.g_threads[7] = g_timeout_add_full(
-			G_PRIORITY_HIGH_IDLE + 10, 40, gx_gui::gx_set_resizeable,
-			gpointer(gui.fWindow.gobj()), NULL);
-                if (guivar.g_threads[6] == 0 || g_main_context_find_source_by_id
-                                        (NULL, guivar.g_threads[6]) == NULL)
-                    guivar.g_threads[6] = g_timeout_add_full(G_PRIORITY_HIGH_IDLE + 10, 50,
-                                   gx_gui::gx_set_default, gpointer(gui.RBox), NULL);
+            guivar.g_threads[7] = g_timeout_add_full(
+                G_PRIORITY_HIGH_IDLE + 10, 40, gx_gui::gx_set_resizeable,
+                gpointer(gui.fWindow.gobj()), NULL);
+            if (guivar.g_threads[6] == 0 || g_main_context_find_source_by_id
+                                    (NULL, guivar.g_threads[6]) == NULL)
+                guivar.g_threads[6] = g_timeout_add_full(G_PRIORITY_HIGH_IDLE + 10, 50,
+                               gx_gui::gx_set_default, gpointer(gui.RBox), NULL);
         }
     }
 }
@@ -1282,61 +1269,36 @@ void GxMainInterface::on_toolbar_activate() {
         if (fWindow.get_resizable())
             fWindow.set_resizable(false);
         gtk_widget_hide(gw.rack_tool_bar);
-    }
-    if (guivar.g_threads[7] == 0 || g_main_context_find_source_by_id(NULL, guivar.g_threads[7]) == NULL)
-            guivar.g_threads[7] = g_timeout_add_full(
-		G_PRIORITY_HIGH_IDLE + 10, 40, gx_set_resizeable,
-		gpointer(fWindow.gobj()), NULL);
-    if (guivar.g_threads[6] == 0 || g_main_context_find_source_by_id(NULL, guivar.g_threads[6]) == NULL)
+    
+        if (guivar.g_threads[7] == 0 || g_main_context_find_source_by_id(NULL, guivar.g_threads[7]) == NULL)
+                guivar.g_threads[7] = g_timeout_add_full(
+            G_PRIORITY_HIGH_IDLE + 10, 40, gx_set_resizeable,
+            gpointer(fWindow.gobj()), NULL);
+    
+        if (guivar.g_threads[6] == 0 || g_main_context_find_source_by_id(NULL, guivar.g_threads[6]) == NULL)
             guivar.g_threads[6] = g_timeout_add_full(G_PRIORITY_HIGH_IDLE + 10, 50,
                            gx_set_default, gpointer(RBox), NULL);
-}
-
-// ----menu function gx_tuner
-void GxMainInterface::on_tuner_activate() {
-    bool tuner_on = mainmenu.fShowTuner.get_active();
-    engine.tuner.used_for_display(tuner_on);
-    engine.set_rack_changed();
-    if (tuner_on) {
-        gtk_widget_show_all(gw.tuner_widget);
-        fTuner.show();
-    } else {
-        GtkAllocation my_size;
-        gtk_widget_get_allocation(GTK_WIDGET(RBox), &my_size);
-        gtk_widget_set_size_request(GTK_WIDGET(RBox), -1, my_size.height);
-        if (fWindow.get_resizable())
-            fWindow.set_resizable(false);
-        fTuner.hide();
-        gtk_widget_hide(gw.tuner_widget);
     }
-    if (guivar.g_threads[7] == 0 || g_main_context_find_source_by_id(NULL, guivar.g_threads[7]) == NULL)
-            guivar.g_threads[7] = g_timeout_add_full(
-		G_PRIORITY_HIGH_IDLE + 10, 40, gx_set_resizeable,
-		gpointer(fWindow.gobj()), NULL);
-    if (guivar.g_threads[6] == 0 || g_main_context_find_source_by_id(NULL, guivar.g_threads[6]) == NULL)
-            guivar.g_threads[6] = g_timeout_add_full(G_PRIORITY_HIGH_IDLE + 10, 50,
-                           gx_set_default, gpointer(RBox), NULL);
 }
 
 // ---- menu function gx_midi_out
-void gx_midi_out(Gtk::CheckMenuItem& menuitem) {
+void GxMainInterface::gx_midi_out() {
     GList*   child_list =  gtk_container_get_children(GTK_CONTAINER(gw.midibox));
     GtkWidget *child = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 0));
     g_list_free(child_list);
-    gx_show_extended_settings(GTK_WIDGET(menuitem.gobj()), (gpointer) child);
+    gx_show_extended_settings(GTK_WIDGET(mainmenu.fShowMidiOut.gobj()), (gpointer) child);
     static bool first = true;
-    if (menuitem.get_active()) {
+    if (mainmenu.fShowMidiOut.get_active()) {
         if (first) {
 	    guivar.refresh_size = 0;
 	}
         //gx_engine::turnOnMidi();//FIXME
     } else {
         //gx_engine::turnOffMidi();//FIXME
-        string group = "midi_out.on_off";
-        parameter_map[group].set_std_value();
+        pmap["midi_out.on_off"].set_std_value();
     }
     first =false;
-    GxMainInterface::get_instance().engine.set_rack_changed();
+    engine.set_rack_changed();
 }
 
 // ----- hide the extendend settings slider
@@ -1358,6 +1320,29 @@ void GxMainInterface::gx_systray_menu(guint button, guint32 activate_time) {
     mainmenu.engine_menu.popup(2, gtk_get_current_event_time());
 }
 
+// ----- hide / show values in controller
+void GxMainInterface::on_value_activate() {
+    if (mainmenu.fShowValue.get_active()) {
+        gtk_rc_parse_string(
+        "style \"ShowValue\"\n"
+         "{\n"
+         "  GxRegler::show-value = 0\n"
+         "}\n"
+         "class \"*GxRegler*\" style:highest \"ShowValue\"");
+    
+        gtk_rc_reset_styles(gtk_settings_get_default()) ;
+    } else {
+        gtk_rc_parse_string(
+        "style \"ShowValue\"\n"
+         "{\n"
+         "  GxRegler::show-value = 1\n"
+         "}\n"
+         "class \"*GxRegler*\" style:highest \"ShowValue\"");
+    
+        gtk_rc_reset_styles(gtk_settings_get_default()) ; 
+    }
+}
+
 // ---- choice dialog without text entry
 gint gx_nchoice_dialog_without_entry(
     const char* window_title,
@@ -1466,7 +1451,7 @@ gint gx_choice_dialog_with_text_entry(
 
     g_signal_connect_swapped(button1, "clicked",  G_CALLBACK(func), gtk_entry);
     g_signal_connect(GTK_DIALOG(dialog)->vbox, "expose-event",
-                     G_CALLBACK(gx_cairo::info_box_expose), NULL);
+                     G_CALLBACK(gx_cairo::rectangle_skin_color_expose), NULL);
 
     gtk_dialog_set_has_separator(GTK_DIALOG(dialog), TRUE);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), default_response);
@@ -1632,5 +1617,246 @@ gboolean gx_delete_event(GtkWidget *widget, gpointer   data ) {
     gtk_range_set_value(GTK_RANGE(widget), 0);
     return TRUE;
 }
+
+
+/****************************************************************
+ ** class GxBuilder
+ */
+
+// GList(GObject*) helper class must be defined in other namespace
+} // end namespace gx_gui
+namespace Glib { namespace Container_Helpers {
+template <>
+struct TypeTraits<GObject*> {
+    typedef GObject *CppType;
+    typedef GObject *CType;
+    typedef GObject *CTypeNonConst;
+
+    static CType to_c_type(CppType item) { return item; }
+    static CppType to_cpp_type(CType item) { return item; }
+    static void release_c_type(CType) {}
+};
+}} namespace gx_gui {
+
+//static
+Glib::RefPtr<GxBuilder> GxBuilder::create_from_file(
+    const std::string& filename, gx_ui::GxUI* ui, const char* object_id) {
+    Glib::RefPtr<GxBuilder> builder = GxBuilder::create();
+    try {
+	if (object_id) {
+	    builder->add_from_file(filename, object_id);
+	} else {
+	    builder->add_from_file(filename);
+	}
+    } catch(const Glib::FileError& ex) {
+        gx_system::gx_print_fatal("FileError", ex.what());
+    } catch(const Gtk::BuilderError& ex) {
+        gx_system::gx_print_fatal("Builder Error", ex.what());
+    }
+    if (ui) {
+	builder->fixup_controlparameters(*ui);
+    }
+    return builder;
+}
+
+//static
+Glib::RefPtr<GxBuilder> GxBuilder::create_from_file(
+    const std::string& filename, gx_ui::GxUI* ui, const Glib::StringArrayHandle& object_ids) {
+    Glib::RefPtr<GxBuilder> builder = GxBuilder::create();
+    try {
+	builder->add_from_file(filename, object_ids);
+    } catch(const Glib::FileError& ex) {
+        gx_system::gx_print_fatal("FileError", ex.what());
+    } catch(const Gtk::BuilderError& ex) {
+        gx_system::gx_print_fatal("Builder Error", ex.what());
+    }
+    if (ui) {
+	builder->fixup_controlparameters(*ui);
+    }
+    return builder;
+}
+
+//static
+Glib::RefPtr<GxBuilder> GxBuilder::create_from_string(
+    const Glib::ustring& buffer, gx_ui::GxUI* ui, const char* object_id) {
+    Glib::RefPtr<GxBuilder> builder = GxBuilder::create();
+    try {
+	if (object_id) {
+	    builder->add_from_string(buffer, object_id);
+	} else {
+	    builder->add_from_string(buffer);
+	}
+    } catch(const Gtk::BuilderError& ex) {
+        gx_system::gx_print_fatal("Builder Error", ex.what());
+    }
+    if (ui) {
+	builder->fixup_controlparameters(*ui);
+    }
+    return builder;
+}
+
+//static
+Glib::RefPtr<GxBuilder> GxBuilder::create_from_string(
+    const Glib::ustring& buffer, gx_ui::GxUI* ui, const Glib::StringArrayHandle& object_ids) {
+    Glib::RefPtr<GxBuilder> builder = GxBuilder::create();
+    try {
+	builder->add_from_string(buffer, object_ids);
+    } catch(const Gtk::BuilderError& ex) {
+        gx_system::gx_print_fatal("Builder Error", ex.what());
+    }
+    if (ui) {
+	builder->fixup_controlparameters(*ui);
+    }
+    return builder;
+}
+
+Gtk::Window *GxBuilder::get_first_window() {
+    Glib::SListHandle<GObject*> objs = Glib::SListHandle<GObject*>(
+	gtk_builder_get_objects(gobj()), Glib::OWNERSHIP_DEEP);
+    for (Glib::SListHandle<GObject*>::iterator i = objs.begin(); i != objs.end(); ++i) {
+	if (g_type_is_a(G_OBJECT_TYPE(*i), GTK_TYPE_WINDOW)) {
+	    return Glib::wrap(GTK_WINDOW(*i), false);
+	}
+    }
+    assert(false);
+    return 0;
+}
+
+Gtk::Widget* GxBuilder::get_widget_checked(const Glib::ustring& name, GType type, bool take_ref) {
+    GtkWidget *cobject = get_cwidget(name);
+    if(!cobject) {
+	g_critical("gtkmm: GxBuilder: widget `%s' was not found in the GtkBuilder file, or the specified part of it.", 
+		   name.c_str());
+	return 0;
+    }
+    if(!g_type_is_a(G_OBJECT_TYPE(cobject), type)) {
+	g_critical("gtkmm: widget `%s' (in GtkBuilder file) is of type `%s' but `%s' was expected",
+		   name.c_str(), G_OBJECT_TYPE_NAME(cobject), g_type_name(type));
+	return 0;
+    }
+    return Glib::wrap (GTK_WIDGET (cobject), take_ref);
+}
+
+/*
+ ** GxBuilder::fixup_controlparameters + helper classes
+ */
+
+template<class T>
+class uiToggle: public gx_ui::GxUiItemV<T> {
+protected:
+    Glib::RefPtr<Gtk::ToggleButton> button;
+    void on_button_toggled();
+    virtual void reflectZone();
+public:
+    uiToggle(gx_ui::GxUI& ui, Glib::RefPtr<Gtk::ToggleButton>& b, T *zone);
+};
+
+template<class T>
+uiToggle<T>::uiToggle(gx_ui::GxUI& ui, Glib::RefPtr<Gtk::ToggleButton>& b, T *zone)
+    : gx_ui::GxUiItemV<T>(&ui, zone), button(b) {
+    button->signal_toggled().connect(sigc::mem_fun(*this, &uiToggle<T>::on_button_toggled));
+}
+
+template<class T>
+void uiToggle<T>::on_button_toggled() {
+    gx_ui::GxUiItemV<T>::modifyZone(button->get_active());
+}
+
+template<class T>
+void uiToggle<T>::reflectZone() {
+    T v = *gx_ui::GxUiItemV<T>::fZone;
+    gx_ui::GxUiItemV<T>::fCache = v;
+    button->set_active(v);
+}
+
+void GxBuilder::fixup_controlparameters(gx_ui::GxUI& ui) {
+    Glib::SListHandle<GObject*> objs = Glib::SListHandle<GObject*>(
+        gtk_builder_get_objects(gobj()), Glib::OWNERSHIP_DEEP);
+    for (Glib::SListHandle<GObject*>::iterator i = objs.begin(); i != objs.end(); ++i) {
+	if (g_type_is_a(G_OBJECT_TYPE(*i), GTK_TYPE_WIDGET)) {
+	    const char *id = gtk_buildable_get_name(GTK_BUILDABLE(*i));
+	    const char *p = g_strstr_len(id, -1, ":");
+	    if (p) {
+		gtk_widget_set_name(GTK_WIDGET(*i), p+1);
+	    }
+	}
+        if (!g_type_is_a(G_OBJECT_TYPE(*i), GX_TYPE_CONTROL_PARAMETER)) {
+            continue;
+        }
+        Glib::RefPtr<Gxw::ControlParameter> w = Glib::wrap(GX_CONTROL_PARAMETER(*i), true);
+        Glib::ustring v = w->cp_get_var();
+        if (v.empty()) {
+            continue;
+        }
+        if (!gx_engine::parameter_map.hasId(v)) {
+	    Glib::RefPtr<Gtk::Widget> wd = Glib::RefPtr<Gtk::Widget>::cast_dynamic(w);
+	    wd->set_sensitive(0);
+            wd->set_tooltip_text(v);
+            gx_system::gx_print_warning("load dialog",
+                (boost::format("Parameter variable %1% not found") % v).str());
+            continue;
+        }
+        gx_engine::Parameter& p = gx_engine::parameter_map[v];
+        if (!p.desc().empty()) {
+            Glib::RefPtr<Gtk::Widget>::cast_dynamic(w)->set_tooltip_text(
+		gettext(p.desc().c_str()));
+        }
+        if (p.isFloat()) {
+            gx_engine::FloatParameter &fp = p.getFloat();
+            w->cp_configure(p.l_group(), p.l_name(), fp.lower, fp.upper, fp.step);
+            w->cp_set_value(fp.get_value());
+            Glib::RefPtr<Gtk::Range> r = Glib::RefPtr<Gtk::Range>::cast_dynamic(w);
+            if (r) {
+                Gtk::Adjustment *adj = r->get_adjustment();
+                gx_gui::uiAdjustment* c = new gx_gui::uiAdjustment(&ui, &fp.get_value(), adj->gobj());
+                adj->signal_value_changed().connect(
+                    sigc::bind<GtkAdjustment*>(
+                        sigc::bind<gpointer>(
+                            sigc::ptr_fun(gx_gui::uiAdjustment::changed),
+                                         (gpointer)c), adj->gobj()));
+            } else {
+                Glib::RefPtr<Gtk::ToggleButton> t =
+                    Glib::RefPtr<Gtk::ToggleButton>::cast_dynamic(w);
+                if (t) {
+                    new uiToggle<float>(ui, t, &fp.get_value());
+                }
+            }
+            if (fp.isControllable()) {
+                gx_gui::connect_midi_controller(GTK_WIDGET(w->gobj()), &fp.get_value());
+            }
+        } else if (p.isBool()) {
+            gx_engine::BoolParameter &fp = p.getBool();
+            w->cp_configure(p.l_group(), p.l_name(), 0, 0, 0);
+            w->cp_set_value(fp.get_value());
+	    Glib::RefPtr<Gtk::ToggleButton> t =
+		Glib::RefPtr<Gtk::ToggleButton>::cast_dynamic(w);
+	    if (t) {
+		new uiToggle<bool>(ui, t, &fp.get_value());
+	    }
+            if (fp.isControllable()) {
+                gx_gui::connect_midi_controller(GTK_WIDGET(w->gobj()), &fp.get_value());
+            }
+	} else if (p.isInt()) {
+            gx_engine::IntParameter &fp = p.getInt();
+	    Glib::RefPtr<Gxw::Selector> t =
+		Glib::RefPtr<Gxw::Selector>::cast_dynamic(w);
+	    if (t) {
+		Gtk::TreeModelColumn<Glib::ustring> label;
+		Gtk::TreeModelColumnRecord rec;
+		rec.add(label);
+		Glib::RefPtr<Gtk::ListStore> ls = Gtk::ListStore::create(rec);
+		for (const value_pair *p = fp.getValueNames(); p->value_id; ++p) {
+		    ls->append()->set_value(0, Glib::ustring(fp.value_label(*p)));
+		}
+		t->set_model(ls);
+		t->cp_set_value(fp.get_value());
+	    }
+        } else {
+            gx_system::gx_print_warning("load dialog",
+                      (boost::format("Parameter variable %1%: type not handled") % v).str());
+        }
+    }
+}
+
 } /* end of gx_gui namespace */
 
