@@ -34,9 +34,9 @@ namespace gx_gui {
 
 #ifndef NDEBUG
 // debug_check
-inline void all_midi_params_assigned() {
-    for (ParamMap::iterator i = parameter_map.begin(); i != parameter_map.end(); ++i) {
-        if (i->second->isControllable() && !i->second->isUsed() && !i->second->isExperimental())
+inline void all_midi_params_assigned(gx_engine::ParamMap& pmap) {
+    for (gx_engine::ParamMap::iterator i = pmap.begin(); i != pmap.end(); ++i) {
+        if (i->second->isControllable() && !i->second->isUsed())
             gx_system::gx_print_error("Debug Check",
                                       "midi-parameter not assigned in ui: " + i->first);
     }
@@ -58,10 +58,9 @@ void GxMainInterface::setup() {
 
                 openToolBar(_("Plugins"));
                 {
-		    create_ptoggle_button("presets");
+                    create_ptoggle_button("presets");
                 }
                 closeBox();
-                addNumDisplay();
             }
             closeBox();
 
@@ -309,15 +308,38 @@ void GxMainInterface::setup() {
     // add a Patch Info widget
     openPatchInfoBox(&gx_gui::guivar.show_patch_info);
 
-    UiBuilder builder(this);
+    UiBuilderImpl builder(this);
     engine.pluginlist.append_rack(builder);
 
-    debug_check(all_midi_params_assigned);
-    if (!connect_jack(true)) {
-	gx_settings.load(gx_system::GxSettingsBase::state);
+#ifndef NDEBUG
+    debug_check(all_midi_params_assigned, pmap);
+    const char *dbgfmt = getenv("GUITARIX_DEBUG_PARAMETER");
+    if (dbgfmt) {
+	pmap.set_init_values();
+	pmap.dump(dbgfmt);
     }
-    for (int i = 0; i < GX_NUM_OF_FACTORY_PRESET; i++) {
-	gx_preset::gxpreset.gx_load_factory_file(i);
+#endif
+
+    gx_settings.loadstate();
+    if (!options.get_rcset().empty()) {
+	gx_gui::gx_actualize_skin_index(options.skin, options.get_rcset());
+    }
+    gx_gui::gx_update_skin_menu_item(gx_engine::audio.fskin);
+    gx_gui::gx_update_skin(gx_engine::audio.fskin, "");
+    connect_jack(true);
+    vector<string> l;
+    gx_settings.fill_factory_names(l);
+    for (vector<string>::iterator i = l.begin(); i != l.end(); ++i) {
+	Gtk::MenuItem *mi = new Gtk::MenuItem(*i);
+	Gtk::Menu *menu = new Gtk::Menu();
+	gx_preset::gxpreset.gx_append_factory_file(*i, *menu);
+	mi->set_submenu(*manage(menu));
+	mainmenu.preset_factory_settings_menu.append(*manage(mi));
+    }
+    if (l.empty()) {
+	mainmenu.preset_factory_settings_label.set_sensitive(false);
+    } else {
+	mainmenu.preset_factory_settings_menu.show_all();
     }
     gx_preset::gxpreset.gx_refresh_preset_menus();
     gx_jconv::gx_load_jcgui();
