@@ -30,177 +30,6 @@
 
 namespace gx_engine {
 
-/****************************************************************
- ** MonoMute, StereoMute, MaxLevel
- */
-
-MonoMute::MonoMute()
-    : PluginDef() {
-    version = PLUGINDEF_VERSION;
-    id = "monomute";
-    name = "?monomute";
-    mono_audio = process;
-}
-
-void MonoMute::process(int count, float *input, float *output, PluginDef*) {
-    (void)memset(output, 0, count*sizeof(float));
-}
-
-StereoMute::StereoMute()
-    : PluginDef() {
-    version = PLUGINDEF_VERSION;
-    id = "stereomute";
-    name = "?stereomute";
-    stereo_audio = process;
-}
-
-void StereoMute::process(int count, float *input0, float *input1,
-			 float *output0, float *output1, PluginDef*) {
-    (void)memset(output0, 0, count*sizeof(float));
-    (void)memset(output1, 0, count*sizeof(float));
-}
-
-MaxLevel::MaxLevel()
-    : PluginDef() {
-    version = PLUGINDEF_VERSION;
-    flags = PGN_SNOOP;
-    id = "maxlevel";
-    name = "?maxlevel";
-    stereo_audio = process;
-    activate_plugin = activate;
-}
-
-float MaxLevel::maxlevel[2] = {0};
-
-void MaxLevel::process(int count, float *input1, float *input2, float*, float*, PluginDef*) {
-    const float *data[2] = {input1, input2};
-    for (int c = 0; c < 2; c++) {
-        float level = 0;
-        for (int i = 0; i < count; i++) {
-            float t = abs(data[c][i]);
-            if (level < t) {
-                level = t;
-            }
-        }
-        maxlevel[c] = max(maxlevel[c], level);
-    }
-}
-
-int MaxLevel::activate(bool start, PluginDef *plugin) {
-    if (!start) {
-	reset();
-    }
-    return 0;
-}
-
-
-/****************************************************************
- ** class TunerAdapter
- */
-
-TunerAdapter::TunerAdapter(ModuleSequencer& engine_)
-    : ModuleSelector(engine_),
-      PluginDef(),
-      pitch_tracker(),
-      state(),
-      engine(engine_),
-      dep_plugin(),
-      plugin() {
-    version = PLUGINDEF_VERSION;
-    flags = PGN_SNOOP;
-    id = "tuner";
-    name = "?tuner";
-    mono_audio = feed_tuner;
-    set_samplerate = init;
-    register_params = regparam;
-    plugin.pdef = this;
-}
-
-void TunerAdapter::init(unsigned int samplingFreq, PluginDef *plugin) {
-    TunerAdapter& self = *static_cast<TunerAdapter*>(plugin);
-    int priority, policy;
-    // zita-convoler uses 5 levels, so substract 6
-    self.engine.get_sched_priority(policy, priority, 6);
-    self.pitch_tracker.init(policy, priority, samplingFreq);
-}
-
-void TunerAdapter::set_and_check(int use, bool on) {
-    if (on) {
-	state |= use;
-    } else {
-	state &= ~use;
-    }
-    plugin.on_off = state;
-}
-
-void TunerAdapter::feed_tuner(int count, float* input, float*, PluginDef* plugin) {
-    static_cast<TunerAdapter*>(plugin)->pitch_tracker.add(count, input);
-}
-
-int TunerAdapter::regparam(const ParamReg& reg) {
-    static_cast<TunerAdapter*>(reg.plugin)->plugin.on_off = false;
-    return 0;
-}
-
-void TunerAdapter::set_module() {
-    used_by_midi(dep_plugin->on_off);
-}
-
-
-/****************************************************************
- ** class MidiAudioBuffer
- */
-
-static const char *midi_out_groups[] = {
-	"channel_1", N_("Midi Out 1"), 
-	"channel_2", N_("Midi Out 2"),
-	"channel_3", N_("Midi Out 3"),
-	".beat_detector",      N_("Beat Detector"),
-	0
-    };
-
-MidiAudioBuffer::MidiAudioBuffer(TunerAdapter& t)
-    : PluginDef(),
-      midi(),
-      tuner(t),
-      jack(0),
-      plugin() {
-    version = PLUGINDEF_VERSION;
-    flags = PGN_SNOOP;
-    id = "midi_out";
-    name = N_("Midi Out");
-    groups = midi_out_groups;
-    mono_audio = fill_buffer;
-    set_samplerate = init;
-    register_params = regparam;
-    plugin.pdef = this;
-}
-
-void MidiAudioBuffer::fill_buffer(int count, float *input, float*,
-				  PluginDef *plugin) {
-    MidiAudioBuffer& self = *static_cast<MidiAudioBuffer*>(plugin);
-    if (!self.jack) {
-	return;
-    }
-    void *buf = self.jack->get_midi_buffer(count);
-    if (buf) {
-	self.midi.process_midi(count, input, buf, self.jack->get_jcpu_load(),
-			       self.tuner.get_freq(), self.tuner.get_note());
-    }
-}
-
-void MidiAudioBuffer::init(unsigned int samplingFreq, PluginDef *plugin) {
-    MidiAudioBuffer& self = *static_cast<MidiAudioBuffer*>(plugin);
-    self.midi.init(samplingFreq);
-}
-
-int MidiAudioBuffer::regparam(const ParamReg& reg) {
-    MidiAudioBuffer& self = *static_cast<MidiAudioBuffer*>(reg.plugin);
-    self.midi.register_parameter(reg);
-    return 0;
-}
-
-
 /****************************************************************
  **  class NoiseGate
  */
@@ -270,60 +99,6 @@ int NoiseGate::outputgate_activate(bool start, PluginDef *pdef) {
 }
 
 
-/****************************************************************
- ** class OscilloscopeAdapter
- */
-
-OscilloscopeAdapter::OscilloscopeAdapter(
-    gx_ui::GxUI *ui, ModuleSequencer& engine)
-    : PluginDef(),
-      plugin(),
-      activation(),
-      size_change(),
-      post_pre_signal(ui, &plugin.effect_post_pre),
-      visible(ui, &plugin.box_visible)
-{
-    assert(buffer == 0);
-    version = PLUGINDEF_VERSION;
-    flags = PGN_SNOOP;
-    id = "oscilloscope";
-    name = N_("Oscilloscope");
-    mono_audio = fill_buffer;
-    activate_plugin = activate;
-    plugin.pdef = this;
-    engine.buffersize_change.connect(
-	sigc::mem_fun(*this, &OscilloscopeAdapter::change_buffersize));
-}
-
-void OscilloscopeAdapter::change_buffersize(unsigned int size_) {
-    //FIXME waveview display needs mutex
-    size_change(0);
-    float *b = buffer;
-    buffer = new float[size_];
-    size = size_;
-    clear_buffer();
-    size_change(size_);
-    delete b;
-}
-
-float* OscilloscopeAdapter::buffer = 0;
-unsigned int OscilloscopeAdapter::size = 0;
-
-// rt process function
-void OscilloscopeAdapter::fill_buffer(int count, float *input0, float *output0, PluginDef*) {
-    assert(count == static_cast<int>(size));
-    (void)memcpy(buffer, output0, sizeof(float)*count);
-}
-
-int OscilloscopeAdapter::activate(bool start, PluginDef *plugin) {
-    return static_cast<OscilloscopeAdapter*>(plugin)->activation(start);
-}
-
-void OscilloscopeAdapter::clear_buffer() {
-    memset(buffer, 0, size*sizeof(float));
-}
-
-
 /****************************************************************
  ** class GxJConvSettings
  */
@@ -492,12 +267,15 @@ void GxJConvSettings::readJSON(gx_system::JsonParser& jp,
  ** class ConvolverAdapter
  */
 
-ConvolverAdapter::ConvolverAdapter(ModuleSequencer& engine_)
+#include "faust/jconv_post.cc"
+
+ConvolverAdapter::ConvolverAdapter(EngineControl& engine_)
     : PluginDef(),
       conv(),
       activate_mutex(),
       engine(engine_),
       activated(false),
+      jc_post(),
       plugin(),
       jcset() {
     version = PLUGINDEF_VERSION;
@@ -509,11 +287,14 @@ ConvolverAdapter::ConvolverAdapter(ModuleSequencer& engine_)
     register_params = convolver_register;
     //FIXME: add clear_state
     plugin = this;
-    engine.buffersize_change.connect(
+    engine.signal_buffersize_change().connect(
 	sigc::mem_fun(*this, &ConvolverAdapter::change_buffersize));
     GxJConvSettings::checkbutton7 = &plugin.on_off;
 }
 
+ConvolverAdapter::~ConvolverAdapter() {
+}
+
 void ConvolverAdapter::change_buffersize(unsigned int size) {
     boost::mutex::scoped_lock lock(activate_mutex);
     if (activated) {
@@ -522,19 +303,16 @@ void ConvolverAdapter::change_buffersize(unsigned int size) {
 	    conv.checkstate();
 	}
 	conv.set_buffersize(size);
-	conv_start();
+	if (size) {
+	    conv_start();
+	}
     } else {
 	conv.set_buffersize(size);
     }
 }
 
-static gboolean conv_error_message(gpointer data) {
-    gx_system::gx_print_error("Convolver", "overload");
-    return false;
-}
-
 void ConvolverAdapter::restart() {
-    if (!*GxJConvSettings::checkbutton7) {
+    if (!plugin.on_off) {
         return;
     }
     conv.stop();
@@ -553,15 +331,18 @@ void ConvolverAdapter::restart() {
     int policy, priority;
     engine.get_sched_priority(policy, priority);
     if (!rc || !conv.start(policy, priority)) {
-        *GxJConvSettings::checkbutton7 = 0;
+        plugin.on_off = false;
     }
 }
 
 bool ConvolverAdapter::conv_start() {
+    if (!conv.get_buffersize() || !conv.get_samplerate()) {
+	return false;
+    }
     string path = jcset.getFullIRPath();
     if (path.empty()) {
         gx_system::gx_print_warning(_("convolver"), _("no impulseresponse file"));
-        *GxJConvSettings::checkbutton7 = 0;
+        plugin.on_off = false;
         return false;
     }
     while (!conv.checkstate());
@@ -584,9 +365,6 @@ bool ConvolverAdapter::conv_start() {
     return conv.start(policy, priority);
 }
 
-
-#include "faust/jconv_post.cc"
-
 void ConvolverAdapter::convolver(int count, float *input0, float *input1,
 				 float *output0, float *output1, PluginDef* plugin) {
     ConvolverAdapter& self = *static_cast<ConvolverAdapter*>(plugin);
@@ -594,18 +372,17 @@ void ConvolverAdapter::convolver(int count, float *input0, float *input1,
         float conv_out0[count];
         float conv_out1[count];
         if (!self.conv.compute(count, output0, output1, conv_out0, conv_out1)) {
-            *GxJConvSettings::checkbutton7 = 0;
-            g_idle_add(conv_error_message, gpointer(NULL));
+            self.plugin.on_off = false;
+	    gx_system::gx_print_error("Convolver", "overload");
         } else {
-            jconv_post::compute(count, output0, output1,
-				conv_out0, conv_out1, output0, output1);
+            self.jc_post.compute(count, output0, output1,
+			    conv_out0, conv_out1, output0, output1);
         }
     }
 }
 
 int ConvolverAdapter::convolver_register(const ParamReg& reg) {
-    jconv_post::register_params(reg);
-    return 0;
+    return static_cast<ConvolverAdapter*>(reg.plugin)->jc_post.register_par(reg);
 }
 
 void ConvolverAdapter::convolver_init(unsigned int samplingFreq, PluginDef *p) {
@@ -614,31 +391,42 @@ void ConvolverAdapter::convolver_init(unsigned int samplingFreq, PluginDef *p) {
     if (self.activated) {
 	self.conv.stop();
 	self.conv.set_samplerate(samplingFreq);
-	jconv_post::init(samplingFreq, p);
+	self.jc_post.init(samplingFreq);
 	while (self.conv.is_runnable()) {
 	    self.conv.checkstate();
 	}
 	self.conv_start();
     } else {
 	self.conv.set_samplerate(samplingFreq);
-	jconv_post::init(samplingFreq, p);
+	self.jc_post.init(samplingFreq);
     }
 }
 
 int ConvolverAdapter::activate(bool start, PluginDef *p) {
     ConvolverAdapter& self = *static_cast<ConvolverAdapter*>(p);
     boost::mutex::scoped_lock lock(self.activate_mutex);
-    if (start == self.activated) {
-	return 0;
-    }
     if (start) {
+	if (self.activated && self.conv.is_runnable()) {
+	    return 0;
+	}
+    } else {
+	if (!self.activated) {
+	    return 0;
+	}
+    }
+    self.activated = start;
+    if (start) {
+	if (self.jc_post.activate(true) != 0) {
+	    gx_system::gx_print_error(_("convolver"), "jconv post activate error?!");
+	    return -1;
+	}
 	if (!self.conv_start()) {
 	    return -1;
 	}
     } else {
 	self.conv.stop();
+	self.jc_post.activate(false);
     }
-    self.activated = start;
     return 0;
 }
 
@@ -648,7 +436,7 @@ int ConvolverAdapter::activate(bool start, PluginDef *p) {
  */
 
 
-BaseConvolver::BaseConvolver(ModuleSequencer& engine_, gx_resample::BufferResampler& resamp)
+BaseConvolver::BaseConvolver(EngineControl& engine_, gx_resample::BufferResampler& resamp)
     : PluginDef(),
       conv(resamp),
       activate_mutex(),
@@ -659,7 +447,7 @@ BaseConvolver::BaseConvolver(ModuleSequencer& engine_, gx_resample::BufferResamp
     set_samplerate = init;
     activate_plugin = activate;
     plugin = this;
-    engine.buffersize_change.connect(
+    engine.signal_buffersize_change().connect(
 	sigc::mem_fun(*this, &BaseConvolver::change_buffersize));
 }
 
@@ -670,7 +458,11 @@ void BaseConvolver::change_buffersize(unsigned int bufsize) {
     boost::mutex::scoped_lock lock(activate_mutex);
     conv.set_buffersize(bufsize);
     if (activated) {
-	start(true);
+	if (!bufsize) {
+	    conv_stop();
+	} else {
+	    start(true);
+	}
     }
 }
 
@@ -686,6 +478,11 @@ void BaseConvolver::init(unsigned int samplingFreq, PluginDef *p) {
 int BaseConvolver::activate(bool start, PluginDef *p) {
     BaseConvolver& self = *static_cast<BaseConvolver*>(p);
     boost::mutex::scoped_lock lock(self.activate_mutex);
+    if (start) {
+	if (!self.conv.get_buffersize()) {
+	    start = false;
+	}
+    }
     if (start == self.activated) {
 	return 0;
     }
@@ -758,7 +555,9 @@ static CabEntry& getCabEntry(unsigned int n) {
 
 static const float no_sum = 1e10;
 
-CabinetConvolver::CabinetConvolver(ModuleSequencer& engine, gx_resample::BufferResampler& resamp):
+#include "faust/cabinet_impulse_former.cc"
+
+CabinetConvolver::CabinetConvolver(EngineControl& engine, gx_resample::BufferResampler& resamp):
     BaseConvolver(engine, resamp),
     current_cab(-1),
     level(0),
@@ -766,7 +565,8 @@ CabinetConvolver::CabinetConvolver(ModuleSequencer& engine, gx_resample::BufferR
     bass(0),
     treble(0),
     sum(no_sum),
-    cab_names(new value_pair[cab_table_size+1]) {
+    cab_names(new value_pair[cab_table_size+1]),
+    impf() {
     for (unsigned int i = 0; i < cab_table_size; ++i) {
 	CabEntry& cab = getCabEntry(i);
 	cab_names[i].value_id = cab.value_id;
@@ -793,17 +593,15 @@ bool CabinetConvolver::conv_update() {
     return ret;
 }
 
-#include "faust/cabinet_impulse_former.cc"
-
 bool CabinetConvolver::start(bool force) {
     CabDesc& cab = *getCabEntry(cabinet).data;
     if (cabinet_changed() || sum_changed() || force) {
 	conv.stop();
         update_cabinet();
 	update_sum();
-	cabinet_impulse_former::init(cab.ir_sr);
+	impf.init(cab.ir_sr);
 	float cab_irdata_c[cab.ir_count];
-	cabinet_impulse_former::compute(cab.ir_count,cab.ir_data,cab_irdata_c);
+	impf.compute(cab.ir_count,cab.ir_data,cab_irdata_c);
 	while (!conv.checkstate());
 	if (!conv.configure(cab.ir_count, cab_irdata_c, cab.ir_sr)) {
 	    return false;
@@ -821,44 +619,39 @@ bool CabinetConvolver::start(bool force) {
 bool CabinetConvolver::update() {
     CabDesc& cab = *getCabEntry(cabinet).data;
     float cab_irdata_c[cab.ir_count];
-    cabinet_impulse_former::compute(cab.ir_count,cab.ir_data,cab_irdata_c);
+    impf.compute(cab.ir_count,cab.ir_data,cab_irdata_c);
     return conv.update(cab.ir_count, cab_irdata_c, cab.ir_sr);
 }
 
 // reduce gain to compensate the increased gain by the cabinet
 inline void CabinetConvolver::compensate_cab(int count, float *input0, float *output0) {
-    double fSlow0 = (0.0010000000000000009 * pow(10, (0.05 * (-level*2.0))));
+    double fSlow0 = 0.001 * pow(10, -0.1 * level);
     static double fRec0[2] = {0, 0};
     for (int i = 0; i < count; i++) {
         fRec0[0] = (fSlow0 + (0.999 * fRec0[1]));
-        output0[i] = (FAUSTFLOAT)(static_cast<double>(input0[i]) * fRec0[0]);
+        output0[i] = input0[i] * fRec0[0];
         // post processing
         fRec0[1] = fRec0[0];
     }
 }
 
-static gboolean cab_error_message(gpointer data) {
-    gx_system::gx_print_error("Convolver", "cabinet overload");
-    return false;
-}
-
 // wraper for the rack order function pointers
 void CabinetConvolver::run_cab_conf(int count, float *input0, float *output0, PluginDef *p) {
     CabinetConvolver& self = *static_cast<CabinetConvolver*>(p);
     self.compensate_cab(count, output0, output0);
     if (!self.conv.compute(count, output0)) {
-        g_idle_add(cab_error_message, gpointer(NULL));
+	gx_system::gx_print_error("Convolver", "cabinet overload");
         self.current_cab = -1;
     }
 }
 
 int CabinetConvolver::register_cab(const ParamReg& reg) {
     CabinetConvolver& cab = *static_cast<CabinetConvolver*>(reg.plugin);
-    gx_gui::registerEnumParam("cab.select", "select", cab.cab_names, &cab.cabinet, 0);
+    reg.registerIEnumVar("cab.select", "select", "B", "", cab.cab_names, &cab.cabinet, 0);
     reg.registerVar("cab.Level", "",  "S", "", &cab.level,  1.0, 0.5, 5.0, 0.5);
     reg.registerVar("cab.bass", "",   "S", "", &cab.bass,   0.0, -10.0, 10.0, 0.5);
     reg.registerVar("cab.treble", "", "S", "", &cab.treble, 0.0, -10.0, 10.0, 0.5);
-    cabinet_impulse_former::register_params(reg);
+    cab.impf.register_par(reg);
     return 0;
 }
 
@@ -867,25 +660,29 @@ int CabinetConvolver::register_cab(const ParamReg& reg) {
  ** class ContrastConvolver
  */
 
-ContrastConvolver::ContrastConvolver(ModuleSequencer& engine, gx_resample::BufferResampler& resamp):
+#include "faust/presence_level.cc"
+
+ContrastConvolver::ContrastConvolver(EngineControl& engine, gx_resample::BufferResampler& resamp):
     BaseConvolver(engine, resamp),
     level(0),
-    sum(no_sum) {
+    sum(no_sum),
+    presl() {
     id = "con";
     name = N_("Contrast convolver");
     mono_audio = run_contrast;
     register_params = register_con;
 }
 
-#include "faust/presence_level.cc"
+ContrastConvolver::~ContrastConvolver() {
+}
 
 bool ContrastConvolver::start(bool force) {
     if (sum_changed() || force) {
 	conv.stop();
 	update_sum();
-	presence_level::init(contrast_ir_desc.ir_sr);
+	presl.init(contrast_ir_desc.ir_sr);
 	float contrast_irdata_c[contrast_ir_desc.ir_count];
-	presence_level::compute(contrast_ir_desc.ir_count,contrast_ir_desc.ir_data,contrast_irdata_c);
+	presl.compute(contrast_ir_desc.ir_count,contrast_ir_desc.ir_data,contrast_irdata_c);
 	while (!conv.checkstate());
 	if (!conv.configure(contrast_ir_desc.ir_count, contrast_irdata_c, contrast_ir_desc.ir_sr)) {
 	    return false;
@@ -915,21 +712,16 @@ inline void ContrastConvolver::compensate_con(int count, float *input0, float *o
 int ContrastConvolver::register_con(const ParamReg& reg) {
     ContrastConvolver& self = *static_cast<ContrastConvolver*>(reg.plugin);
     reg.registerVar("con.Level", "",  "S", "", &self.level,  1.0, 0.5, 5.0, 0.5);
-    presence_level::register_params(reg);
+    self.presl.register_par(reg);
     return 0;
 }
 
-static gboolean contrast_error_message(gpointer data) {
-    gx_system::gx_print_error("Convolver", "presence overload");
-    return false;
-}
-
 // wraper for the presence function
 void ContrastConvolver::run_contrast(int count, float *input0, float *output0, PluginDef *p) {
     ContrastConvolver& self = *static_cast<ContrastConvolver*>(p);
     self.compensate_con(count, output0, output0);
     if (!self.conv.compute(count, output0)) {
-        g_idle_add(contrast_error_message, gpointer(NULL));
+	gx_system::gx_print_error("Convolver", "presence overload");
         self.sum = no_sum;
     }
 }
