@@ -168,9 +168,9 @@ void GxJack::write_connections(gx_system::JsonWriter& w) {
  */
 
 // ----- pop up a dialog for starting jack
-bool GxJack::gx_jack_init() {
+bool GxJack::gx_jack_init(bool startserver) {
     AVOIDDENORMALS;
-    int jackopt = JackNoStartServer;
+    int jackopt = (startserver ? JackNullOption : JackNoStartServer);
     gx_system::CmdlineOptions& opt = gx_system::get_options();
     client_instance = opt.get_jack_instancename();
     if (client_instance.empty()) {
@@ -306,12 +306,12 @@ void GxJack::gx_jack_cleanup() {
 }
 
 // ---- Jack server connection / disconnection
-bool GxJack::gx_jack_connection(bool connect) {
+bool GxJack::gx_jack_connection(bool connect, bool startserver) {
     if (connect) {
 	if (client) {
 	    return true;
 	}
-	if (!gx_jack_init()) {
+	if (!gx_jack_init(startserver)) {
 	    return false;
 	}
 	engine.clear_stateflag(gx_engine::GxEngine::SF_INITIALIZING);
@@ -509,7 +509,7 @@ int GxJack::gx_jack_process(jack_nframes_t nframes, void *arg) {
 
         // midi input processing
 	if (self.ports.midi_input.port) {
-	    gx_gui::controller_map.compute_midi_in(
+	    gx_engine::controller_map.compute_midi_in(
 		jack_port_get_buffer(self.ports.midi_input.port, nframes));
 	}
     }
@@ -526,9 +526,9 @@ int GxJack::gx_jack_insert_process(jack_nframes_t nframes, void *arg) {
     if (!self.is_jack_exit()) {
         AVOIDDENORMALS;
         // gx_head DSP computing
+	float *ibuf = get_float_buf(self.ports.insert_in.port, nframes);
 	self.engine.stereo_chain.process(
-	    nframes,
-	    get_float_buf(self.ports.insert_in.port, nframes),
+	    nframes, ibuf, ibuf,
 	    get_float_buf(self.ports.output1.port, nframes),
 	    get_float_buf(self.ports.output2.port, nframes));
     }
@@ -659,7 +659,7 @@ int GxJack::gx_jack_srate_callback(jack_nframes_t samplerate, void* arg) {
     }
     self.engine.set_stateflag(gx_engine::GxEngine::SF_JACK_RECONFIG);
     self.jack_sr = samplerate;
-    self.engine.samplerate_change(samplerate);
+    self.engine.set_samplerate(samplerate);
     self.engine.clear_stateflag(gx_engine::GxEngine::SF_JACK_RECONFIG);
     return 0;
 }
@@ -673,7 +673,7 @@ int GxJack::gx_jack_buffersize_callback(jack_nframes_t nframes, void* arg) {
     }
     self.engine.set_stateflag(gx_engine::GxEngine::SF_JACK_RECONFIG);
     self.jack_bs = nframes;
-    self.engine.buffersize_change(nframes);
+    self.engine.set_buffersize(nframes);
     self.engine.clear_stateflag(gx_engine::GxEngine::SF_JACK_RECONFIG);
     self.buffersize_change();
     return 0;
@@ -738,7 +738,7 @@ int GxJack::return_last_session_event() {
 	session_callback_seen += 1;
 	jack_session_reply(client, event);
 	jack_session_event_free(event);
-	g_atomic_pointer_set(&session_event, 0);
+	gx_system::atomic_set_0(&session_event);
     }
     return session_callback_seen;
 }
@@ -749,7 +749,7 @@ int GxJack::return_last_session_event_ins() {
 	session_callback_seen -= 1;
 	jack_session_reply(client_insert, event);
 	jack_session_event_free(event);
-	g_atomic_pointer_set(&session_event_ins, 0);
+	gx_system::atomic_set_0(&session_event_ins);
     }
     return session_callback_seen;
 }
@@ -774,8 +774,8 @@ string GxJack::get_uuid_insert() {
 
 void GxJack::gx_jack_session_callback(jack_session_event_t *event, void *arg) {
     GxJack& self = *static_cast<GxJack*>(arg);
-    if (!g_atomic_pointer_compare_and_exchange(
-	    reinterpret_cast<void* volatile*>(&self.session_event), 0, event)) {
+    jack_session_event_t *np = 0;
+    if (!gx_system::atomic_compare_and_exchange(&self.session_event, np, event)) {
 	gx_system::gx_print_error("jack","last session not cleared");
 	return;
     }
@@ -784,8 +784,8 @@ void GxJack::gx_jack_session_callback(jack_session_event_t *event, void *arg) {
 
 void GxJack::gx_jack_session_callback_ins(jack_session_event_t *event, void *arg) {
     GxJack& self = *static_cast<GxJack*>(arg);
-    if (!g_atomic_pointer_compare_and_exchange(
-	    reinterpret_cast<void* volatile*>(&self.session_event_ins), 0, event)) {
+    jack_session_event_t *np = 0;
+    if (!gx_system::atomic_compare_and_exchange(&self.session_event_ins, np, event)) {
 	gx_system::gx_print_error("jack","last session not cleared");
 	return;
     }
