@@ -25,10 +25,10 @@
 
 #include "engine.h"               // NOLINT
 
+#include <sys/stat.h>
+
 namespace gx_system {
 
-
-
 /****************************************************************
  ** JsonWriter
  */
@@ -105,7 +105,7 @@ void JsonWriter::write(unsigned int i, bool nl) {
     snl(nl);
 }
 
-void JsonWriter::write_lit(string s, bool nl) {
+void JsonWriter::write_lit(const string& s, bool nl) {
     komma();
     *os << s;
     snl(nl);
@@ -167,7 +167,7 @@ void JsonWriter::write_key(const char* p, bool nl) {
     first = true;
 }
 
-void JsonWriter::write_key(string p, bool nl) {
+void JsonWriter::write_key(const string& p, bool nl) {
     write(p, nl);
     *os << ": ";
     first = true;
@@ -496,6 +496,12 @@ JsonParser *StateFile::create_reader() {
     if (is) {
 	is->seekg(0);
     } else {
+	struct stat st;
+	if (stat(filename.c_str(), &st) == 0) {
+	    mtime = st.st_mtime;
+	} else {
+	    mtime = 0;
+	}
 	is = new ifstream(filename.c_str());
     }
     JsonReader *jp = new JsonReader(is);
@@ -514,6 +520,23 @@ JsonParser *StateFile::create_reader() {
     return jp;
 }
 
+void StateFile::ensure_is_current() {
+    if (filename.empty() || !mtime) {
+	return;
+    }
+    struct stat st;
+    if (stat(filename.c_str(), &st) == 0) {
+	if (st.st_mtime == mtime) {
+	    return;
+	}
+    } else {
+	mtime = 0;
+    }
+    delete is;
+    is = 0;
+}
+
+
 class ModifyState: public JsonWriter {
 private:
     string filename;
@@ -619,16 +642,24 @@ JsonWriter *StateFile::create_writer(bool *preserve_preset) {
 PresetFile::PresetFile()
     : filename(),
       is(0),
+      mtime(),
       header(),
       entries() {
 }
 
 void PresetFile::open() {
     delete is;
+    is = 0;
     entries.clear();
     if (filename.empty()) {
 	return;
     }
+    struct stat st;
+    if (stat(filename.c_str(), &st) == 0) {
+	mtime = st.st_mtime;
+    } else {
+	mtime = 0;
+    }
     is = new ifstream(filename.c_str());
     JsonParser jp(is);
     jp.next(JsonParser::begin_array);
@@ -653,6 +684,22 @@ void PresetFile::open() {
     jp.next(JsonParser::end_token);
 }
 
+void PresetFile::ensure_is_current() {
+    if (filename.empty() || !mtime) {
+	return;
+    }
+    struct stat st;
+    if (stat(filename.c_str(), &st) == 0) {
+	if (st.st_mtime == mtime) {
+	    return;
+	}
+    } else {
+	mtime = 0;
+    }
+    delete is;
+    is = 0;
+}
+
 void PresetFile::open(const string& fname) {
     filename = fname;
     open();
@@ -840,7 +887,8 @@ bool PresetFile::rename(const string& name, string newname) {
 AbstractStateIO::~AbstractStateIO() {}
 AbstractPresetIO::~AbstractPresetIO() {}
 
-GxSettingsBase::GxSettingsBase(gx_engine::ModuleSequencer& seq_)
+// seq_ may not yet be initialized, only use address!
+GxSettingsBase::GxSettingsBase(gx_engine::EngineControl& seq_)
     : state_io(),
       preset_io(),
       statefile(),
