@@ -42,11 +42,11 @@ void ToggleCheckButton::on_my_toggled() {
     param->set(get_active());
 }
 
-SwitchParameter* ToggleCheckButton::get_parameter() {
+gx_engine::SwitchParameter* ToggleCheckButton::get_parameter() {
     return param;
 }
 
-void ToggleCheckButton::set_parameter(SwitchParameter *p) {
+void ToggleCheckButton::set_parameter(gx_engine::SwitchParameter *p) {
     param = p;
     p->signal_changed().connect(sigc::mem_fun(*this, &ToggleCheckButton::set_active));
     signal_toggled().connect(
@@ -123,7 +123,7 @@ GxPaintBox::~GxPaintBox() {}
 GxPaintBox::GxPaintBox(gx_ui::GxUI& ui, const char *expose_funk)
     : m_box(false, 0) {
     m_paintbox.property_paint_func() = expose_funk;
-    m_paintbox.add(m_box);
+    m_paintbox.pack_start(m_box);
 }
 /****************************************************************/
 
@@ -163,7 +163,7 @@ GxMidiBox::GxMidiBox(gx_ui::GxUI& ui, const char *expose_funk)
     m_bbox.pack_end(*unit_on_off, false, false);
     m_paintbox.property_paint_func() = expose_funk;
     m_eventbox.add(m_paintbox);
-    m_paintbox.add(m_box);
+    m_paintbox.pack_start(m_box);
     m_eventbox.set_border_width(0);
     m_eventbox.set_name("black_box");
     m_box.set_homogeneous(false);
@@ -181,10 +181,156 @@ GxNotebookBox::GxNotebookBox(const gx_ui::GxUI& ui) {
 }
 /****************************************************************/
 
+GxVHideBox::~GxVHideBox() {}
+
+void GxVHideBox::on_hide_button_pressed() {
+    GList*   child_list =  gtk_container_get_children(GTK_CONTAINER(m_box.gobj()));
+    GtkWidget* child = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 0));
+    g_list_free(child_list);
+    
+    GxMainInterface& gui = GxMainInterface::get_instance();
+    GtkAllocation alloc;
+    gtk_widget_get_allocation (GTK_WIDGET(gui.RBox), &alloc);
+    GtkWidget *plug = gtk_widget_get_parent(GTK_WIDGET(m_box.gobj()));
+    GtkWidget *vbox = gtk_widget_get_parent(GTK_WIDGET(plug));
+    GtkWidget *box1 = gtk_widget_get_parent(GTK_WIDGET(vbox));
+    vbox = gtk_widget_get_parent(GTK_WIDGET(box1));
+    box1 = gtk_widget_get_parent(GTK_WIDGET(vbox));
+    vbox = gtk_widget_get_parent(GTK_WIDGET(box1));
+    
+    if(m_tcb.get_active()) {
+        gtk_widget_hide(GTK_WIDGET(child));
+    } else {
+        gtk_widget_show(GTK_WIDGET(child));
+    }
+
+    gtk_widget_hide(GTK_WIDGET(vbox));
+    if (gui.fWindow.get_resizable())
+        gui.fWindow.set_resizable(false);
+    gtk_widget_show(GTK_WIDGET(vbox));
+
+    if (GDK_IS_WINDOW (gui.RBox->window)) {
+        gtk_widget_set_size_request(GTK_WIDGET(gui.RBox), -1, alloc.height);
+    } else {
+        gtk_widget_set_size_request(GTK_WIDGET(gui.RBox), -1, 460);
+    }
+    if (guivar.g_threads[7] == 0 || g_main_context_find_source_by_id
+                            (NULL, guivar.g_threads[7]) == NULL)
+        guivar.g_threads[7] = g_timeout_add_full(
+            G_PRIORITY_HIGH_IDLE + 10, 40, gx_set_resizeable,
+            gpointer(gui.fWindow.gobj()), NULL);
+    if (guivar.g_threads[6] == 0 || g_main_context_find_source_by_id
+                            (NULL, guivar.g_threads[6]) == NULL)
+        guivar.g_threads[6] = g_timeout_add_full(G_PRIORITY_HIGH_IDLE + 10, 50,
+                       gx_set_default, gpointer(gui.RBox), NULL);
+}
+
+GxVHideBox::GxVHideBox(gx_ui::GxUI& ui, gx_engine::Parameter& param_switch) 
+    : m_tcb(&ui, &param_switch.getBool().get_value()) {
+    m_box.property_paint_func() = "rectangle_skin_color_expose";
+    m_fixed.set_size_request(25, 25);
+    m_fixed.put(m_tcb, 2, 4);
+    m_tcb.m_label.set_text("¤");
+    m_tcb.set_size_request(20, 15);
+    m_box.pack_end(m_fixed, false, false, 0);
+    
+    m_tcb.signal_toggled().connect(
+        sigc::mem_fun(*this, &GxVHideBox::on_hide_button_pressed));
+}
+/****************************************************************/
+
 GxMoveBox::~GxMoveBox() {
 }
 
-GxMoveBox::GxMoveBox(const gx_ui::GxUI& ui) {
+void GxMoveBox::on_hide_button_pressed() {
+    GList*   child_list =  gtk_container_get_children(GTK_CONTAINER(m_paintbox.gobj()));
+    GtkWidget* child2 = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 0));
+    GtkWidget* parent = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 1));
+    GtkWidget* parent1 = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 2));
+    g_list_free(child_list);
+    child_list =  gtk_container_get_children(GTK_CONTAINER(parent));
+    GtkWidget* childname = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 0));
+    GtkWidget* child = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 1));
+    g_list_free(child_list);
+    child_list =  gtk_container_get_children(GTK_CONTAINER(parent1));
+    GtkWidget* child1 = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 0));
+    g_list_free(child_list);
+    child_list =  gtk_container_get_children(GTK_CONTAINER(child1));
+    GtkWidget* child4 = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 0));
+    child1 = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 1));
+    GtkWidget* child5 = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 2));
+    
+    g_list_free(child_list);
+    child_list =  gtk_container_get_children(GTK_CONTAINER(child1));
+    GtkWidget* child3 = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 0));
+    child1 = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 1));
+    g_list_free(child_list);
+    
+    if(!GTK_IS_WIDGET(child1)) {
+        child_list =  gtk_container_get_children(GTK_CONTAINER(child3));
+        child3 = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 0));
+        child1 = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 1));
+        g_list_free(child_list);
+    }
+    
+    GxMainInterface& gui = GxMainInterface::get_instance();
+    GtkAllocation alloc;
+    gtk_widget_get_allocation (GTK_WIDGET(gui.RBox), &alloc);
+    GtkWidget *plug = gtk_widget_get_parent(GTK_WIDGET(m_paintbox.gobj()));
+    GtkWidget *vbox = gtk_widget_get_parent(GTK_WIDGET(plug));
+    GtkWidget *box1 = gtk_widget_get_parent(GTK_WIDGET(vbox));
+    vbox = gtk_widget_get_parent(GTK_WIDGET(box1));
+    box1 = gtk_widget_get_parent(GTK_WIDGET(vbox));
+    vbox = gtk_widget_get_parent(GTK_WIDGET(box1));
+
+    if(m_button2.get_active()) {
+        gtk_widget_hide(GTK_WIDGET(child));  // switch
+        gtk_widget_show(GTK_WIDGET(child2)); // sw_minitoggle
+        gtk_widget_show(GTK_WIDGET(child3)); // master controller
+        gtk_widget_hide(GTK_WIDGET(child1)); // paintbox
+        gtk_widget_hide(GTK_WIDGET(child4)); // resetbutton
+        gtk_widget_hide(GTK_WIDGET(child5)); // resetbutton
+        gtk_widget_set_size_request(GTK_WIDGET(childname), 80, -1);
+        m_fixed.set_size_request(65, 25);
+        gtk_fixed_move(GTK_FIXED(m_fixed.gobj()),GTK_WIDGET(m_button1.gobj()),2, 4);
+        gtk_fixed_move(GTK_FIXED(m_fixed.gobj()),GTK_WIDGET(m_button2.gobj()),22, 4);
+        gtk_fixed_move(GTK_FIXED(m_fixed.gobj()),GTK_WIDGET(m_button.gobj()),37, 4);
+    } else {
+        gtk_widget_show(GTK_WIDGET(child));
+        gtk_widget_hide(GTK_WIDGET(child2));
+        gtk_widget_hide(GTK_WIDGET(child3));
+        gtk_widget_show(GTK_WIDGET(child1));
+        gtk_widget_show(GTK_WIDGET(child4));
+        gtk_widget_show(GTK_WIDGET(child5));
+        gtk_widget_set_size_request(GTK_WIDGET(childname), -1, -1);
+        m_fixed.set_size_request(25, -1);
+        gtk_fixed_move(GTK_FIXED(m_fixed.gobj()),GTK_WIDGET(m_button1.gobj()),2, 5);
+        gtk_fixed_move(GTK_FIXED(m_fixed.gobj()),GTK_WIDGET(m_button2.gobj()),2, 20);
+        gtk_fixed_move(GTK_FIXED(m_fixed.gobj()),GTK_WIDGET(m_button.gobj()),2, 35);
+    }
+    gtk_widget_hide(GTK_WIDGET(vbox));
+    if (gui.fWindow.get_resizable())
+        gui.fWindow.set_resizable(false);
+    gtk_widget_show(GTK_WIDGET(vbox));
+
+    if (GDK_IS_WINDOW (gui.RBox->window)) {
+        gtk_widget_set_size_request(GTK_WIDGET(gui.RBox), -1, alloc.height);
+    } else {
+        gtk_widget_set_size_request(GTK_WIDGET(gui.RBox), -1, 460);
+    }
+    if (guivar.g_threads[7] == 0 || g_main_context_find_source_by_id
+                            (NULL, guivar.g_threads[7]) == NULL)
+        guivar.g_threads[7] = g_timeout_add_full(
+            G_PRIORITY_HIGH_IDLE + 10, 40, gx_set_resizeable,
+            gpointer(gui.fWindow.gobj()), NULL);
+    if (guivar.g_threads[6] == 0 || g_main_context_find_source_by_id
+                            (NULL, guivar.g_threads[6]) == NULL)
+        guivar.g_threads[6] = g_timeout_add_full(G_PRIORITY_HIGH_IDLE + 10, 50,
+                       gx_set_default, gpointer(gui.RBox), NULL);
+}
+
+GxMoveBox::GxMoveBox(gx_ui::GxUI& ui, gx_engine::Parameter& param_switch)
+    :m_button2(&ui, &param_switch.getBool().get_value()) {
     m_paintbox.property_paint_func() = "rectangle_skin_color_expose";
     m_label.set_text("▼");
     m_label.set_name("rack_slider");
@@ -196,6 +342,9 @@ GxMoveBox::GxMoveBox(const gx_ui::GxUI& ui) {
     m_button1.add(m_label1);
     m_button1.set_size_request(20, 15);
     m_button1.set_name("effect_reset");
+    m_button2.m_label.set_text("¤");
+    m_button2.set_size_request(20, 15);
+    //m_button2.set_name("effect_reset");
     GtkStyle *style = gtk_widget_get_style(GTK_WIDGET(m_label.gobj()));
     pango_font_description_set_size(style->font_desc, 6*PANGO_SCALE);
     pango_font_description_set_weight(style->font_desc, PANGO_WEIGHT_BOLD);
@@ -203,18 +352,21 @@ GxMoveBox::GxMoveBox(const gx_ui::GxUI& ui) {
     gtk_widget_modify_font(GTK_WIDGET(m_label1.gobj()), style->font_desc);
     m_fixed.set_size_request(25, -1);
     m_fixed.put(m_button1, 2, 5);
-    m_fixed.put(m_button, 2, 20);
+    m_fixed.put(m_button, 2, 35);
+    m_fixed.put(m_button2, 2, 20);
     m_paintbox.pack_end(m_fixed, false, false, 0 );
     m_fixed.show_all();
     m_paintbox.set_border_width(4);
     m_paintbox.property_spacing() = 0;
     m_paintbox.property_homogeneous() = false;
+    m_button2.signal_toggled().connect(
+        sigc::mem_fun(*this, &GxMoveBox::on_hide_button_pressed));
 }
 
 /****************************************************************/
 
 
-GxDialogButtonBox::GxDialogButtonBox(gx_ui::GxUI& ui, Parameter& param_dialog)
+GxDialogButtonBox::GxDialogButtonBox(gx_ui::GxUI& ui, gx_engine::Parameter& param_dialog)
     : box(false, 0),
     show_dialog(ui, sw_button, param_dialog.getBool()) {
     box.set_border_width(0);
@@ -230,12 +382,12 @@ void GxDialogWindowBox::on_dialog_menu_activate() {
     if (!menuitem.get_active()) {
         string group = group_id;
         group += ".on_off";
-        parameter_map[group].set_std_value();
+        gx_engine::parameter_map[group].set_std_value();
     }
 }
 
 void GxDialogWindowBox::on_reset_button_pressed() {
-    gx_reset_units(group_id);
+    gx_reset_units(gx_engine::parameter_map, group_id);
 }
 
 GxDialogWindowBox::~GxDialogWindowBox() {
@@ -243,13 +395,13 @@ GxDialogWindowBox::~GxDialogWindowBox() {
 }
 
 GxDialogWindowBox::GxDialogWindowBox(gx_ui::GxUI& ui, const char *expose_funk,
-                                     Parameter& param_dialog,
-                                     Parameter& param_switch,
+                                     gx_engine::Parameter& param_dialog,
+                                     gx_engine::Parameter& param_switch,
                                      Gtk::ToggleButton& button, GtkWidget * Caller)
     : box(false, 0),
       unit_on_off(UiSwitch::new_switch(ui, sw_led, param_switch)),
-      menuitem(&ui, &param_dialog.getBool().value),
-      m_tcb(&ui, &param_dialog.getBool().value),
+      menuitem(&ui, &param_dialog.getBool().get_value()),
+      m_tcb(&ui, &param_dialog.getBool().get_value()),
       m_regler_tooltip_window(Gtk::WINDOW_POPUP)  {
     group_id = param_switch.id().substr(0, param_switch.id().find_last_of("."));
     Glib::ustring title = param_switch.l_group();
@@ -276,7 +428,7 @@ GxDialogWindowBox::GxDialogWindowBox(gx_ui::GxUI& ui, const char *expose_funk,
     box4.pack_start(box6, false, false, 0);
     box4.pack_start(box, true, true, 0);
     box4.pack_end(box5, false, false, 0);
-    paintbox.add(box4);
+    paintbox.pack_start(box4);
     paintbox.set_tooltip_text(title.c_str());
     m_tcb.m_label.set_text(title.c_str());
     menuitem.signal_activate().connect(
@@ -285,6 +437,82 @@ GxDialogWindowBox::GxDialogWindowBox(gx_ui::GxUI& ui, const char *expose_funk,
 
 /****************************************************************/
 
+void GxTunerRackBox::on_dialog_menu_activate() {
+    gx_show_extended_settings(GTK_WIDGET(menuitem.gobj()), (gpointer)paintbox.gobj());
+   // GxMainInterface& gui = GxMainInterface::get_instance();
+   // bool tuner_on = menuitem.get_active();
+   // gui.engine.tuner.used_for_display(tuner_on);
+   // gui.engine.set_rack_changed();
+    if (!menuitem.get_active()) {
+        string group = group_id;
+        group += ".on_off";
+        gx_engine::parameter_map[group].set_std_value();
+    }
+}
+
+void GxTunerRackBox::on_toggled() {
+    //gx_show_extended_settings(GTK_WIDGET(menuitem.gobj()), (gpointer)paintbox.gobj());
+    GxMainInterface& gui = GxMainInterface::get_instance();
+    bool tuner_on = unit_on_off->get_active();
+    gui.engine.tuner.used_for_display(tuner_on);
+    gui.engine.set_rack_changed();
+    gui.mainmenu.fShowTuner.set_active(tuner_on);
+   // menuitem.set_active(tuner_on);
+}
+
+void GxTunerRackBox::on_reset_button_pressed() {
+    gx_reset_units(gx_engine::parameter_map, group_id);
+}
+
+GxTunerRackBox::~GxTunerRackBox() {
+    delete unit_on_off;
+}
+
+GxTunerRackBox::GxTunerRackBox(gx_ui::GxUI& ui, const char *expose_funk,
+                                     gx_engine::Parameter& param_dialog,
+                                     gx_engine::Parameter& param_switch,
+                                     Gtk::ToggleButton& button, GtkWidget * Caller)
+    : box(false, 0),
+      unit_on_off(UiSwitch::new_switch(ui, sw_led, param_switch)),
+      menuitem(&ui, &param_dialog.getBool().get_value()),
+      m_tcb(&ui, &param_dialog.getBool().get_value()),
+      m_regler_tooltip_window(Gtk::WINDOW_POPUP)  {
+    group_id = param_switch.id().substr(0, param_switch.id().find_last_of("."));
+    Glib::ustring title = param_switch.l_group();
+    box1.pack_end(*unit_on_off, false, false);
+    box.set_border_width(2);
+    box4.set_spacing(2);
+    box4.set_border_width(2);
+    box5.set_border_width(4);
+    box6.set_border_width(4);
+    paintbox.property_paint_func() = expose_funk;
+    paintbox.set_name(title);
+    reset_button.set_name("effect_reset");
+    reset_button1.set_name("effect_reset");
+    box5.add(reset_button);
+    box6.add(reset_button1);
+    box5.set_size_request(15, -1);
+    box6.set_size_request(15, -1);
+    reset_button.signal_pressed().connect(
+        sigc::mem_fun(*this, &GxTunerRackBox::on_reset_button_pressed));
+    reset_button1.signal_pressed().connect(
+        sigc::mem_fun(*this, &GxTunerRackBox::on_reset_button_pressed));
+    reset_button.set_tooltip_text(_("Reset Button, press to reset settings"));
+    reset_button1.set_tooltip_text(_("Reset Button, press to reset settings"));
+    box4.pack_start(box6, false, false, 0);
+    box4.pack_start(box, true, true, 0);
+    box4.pack_end(box5, false, false, 0);
+    paintbox.pack_start(box4);
+    paintbox.set_tooltip_text(title.c_str());
+    m_tcb.m_label.set_text(title.c_str());
+    unit_on_off->signal_toggled().connect(
+        sigc::mem_fun(*this, &GxTunerRackBox::on_toggled));
+    menuitem.signal_activate().connect(
+        sigc::mem_fun(*this, &GxTunerRackBox::on_dialog_menu_activate));
+}
+
+/****************************************************************/
+
 bool GxWindowBox::on_window_delete_event(GdkEventAny*, gpointer d) {
     gtk_check_menu_item_set_active(
                 GTK_CHECK_MENU_ITEM(GTK_WIDGET(d)), FALSE
@@ -361,9 +589,7 @@ void GxScrollBox::on_rack_reorder_horizontal() {
 
             if (strcmp(gtk_widget_get_name(parent), "gtkmm__GtkVBox") == 0) {
                 gtk_widget_ref(gx_gui::gw.rack_tool_bar);
-                gtk_widget_ref(gx_gui::gw.tuner_widget);
                 gtk_container_remove(GTK_CONTAINER(parent), gx_gui::gw.rack_tool_bar);
-                gtk_container_remove(GTK_CONTAINER(parent), gx_gui::gw.tuner_widget);
                 parent = gtk_widget_get_parent(GTK_WIDGET(parent));
                 GList*   child_list =  gtk_container_get_children(GTK_CONTAINER(parent));
                 parent = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 1));
@@ -371,11 +597,9 @@ void GxScrollBox::on_rack_reorder_horizontal() {
                 parent = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 0));
                 child_list =  gtk_container_get_children(GTK_CONTAINER(parent));
                 parent = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 2));
-                gtk_container_add(GTK_CONTAINER(parent), gx_gui::gw.tuner_widget);
                 parent = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 0));
                 gtk_container_add(GTK_CONTAINER(parent), gx_gui::gw.rack_tool_bar);
                 gtk_widget_unref(gx_gui::gw.rack_tool_bar);
-                gtk_widget_unref(gx_gui::gw.tuner_widget);
                 g_list_free(child_list);
             }
 
@@ -410,19 +634,14 @@ void GxScrollBox::on_rack_reorder_vertical() {
 
             if (strcmp(gtk_widget_get_name(parent), "gtkmm__GtkHBox") == 0) {
                 gtk_widget_ref(gx_gui::gw.rack_tool_bar);
-                gtk_widget_ref(gx_gui::gw.tuner_widget);
                 gtk_container_remove(GTK_CONTAINER(parent), gx_gui::gw.rack_tool_bar);
-                parent = gtk_widget_get_parent(GTK_WIDGET(gx_gui::gw.tuner_widget));
-                gtk_container_remove(GTK_CONTAINER(parent), gx_gui::gw.tuner_widget);
                 parent = gtk_widget_get_parent(GTK_WIDGET(parent));
                 parent = gtk_widget_get_parent(GTK_WIDGET(parent));
                 parent = gtk_widget_get_parent(GTK_WIDGET(parent));
                 GList*   child_list =  gtk_container_get_children(GTK_CONTAINER(parent));
                 parent = reinterpret_cast<GtkWidget *>(g_list_nth_data(child_list, 0));
                 gtk_container_add(GTK_CONTAINER(parent), gx_gui::gw.rack_tool_bar);
-                gtk_container_add(GTK_CONTAINER(parent), gx_gui::gw.tuner_widget);
                 gtk_widget_unref(gx_gui::gw.rack_tool_bar);
-                gtk_widget_unref(gx_gui::gw.tuner_widget);
                 g_list_free(child_list);
             }
 
@@ -438,8 +657,8 @@ void GxScrollBox::on_rack_reorder_vertical() {
     }
 }
 
-GxScrollBox::GxScrollBox(gx_ui::GxUI& ui, const char *pb_2, Glib::ustring titl, GtkWidget * d,
-			 RadioCheckItem& fOrdervRack_, RadioCheckItem& fOrderhRack_)
+GxScrollBox::GxScrollBox(gx_ui::GxUI& ui, const char *pb_2, Glib::ustring titl, gx_engine::ParamMap& pmap,
+			 GtkWidget *d, RadioCheckItem& fOrdervRack_, RadioCheckItem& fOrderhRack_)
     : window(Gtk::WINDOW_TOPLEVEL),
       rbox(false, 4),
       fOrdervRack(fOrdervRack_),
@@ -457,8 +676,8 @@ GxScrollBox::GxScrollBox(gx_ui::GxUI& ui, const char *pb_2, Glib::ustring titl,
     paintbox1.property_paint_func() = pb_2;
     window.signal_delete_event().connect(
          sigc::bind<gpointer>(sigc::mem_fun(*this, &GxScrollBox::on_window_delete_event), d));
-    paintbox1.add(m_scrolled_window);
-    paintbox1.add(m_scrolled_window2);
+    paintbox1.pack_start(m_scrolled_window);
+    paintbox1.pack_start(m_scrolled_window2);
     m_scrolled_window.add(box);
     m_scrolled_window2.add(vbox);
     window.add(paintbox1);
@@ -473,14 +692,14 @@ GxScrollBox::GxScrollBox(gx_ui::GxUI& ui, const char *pb_2, Glib::ustring titl,
     fOrdervRack.set_label(mtitle);
     gx_gui::GxMainInterface& gui = gx_gui::GxMainInterface::get_instance();
     gui.mainmenu.plugin_menu.append(fOrdervRack);
-    fOrdervRack.set_parameter(new SwitchParameter("system.order_rack_v", false, false));
+    fOrdervRack.set_parameter(pmap.reg_switch("system.order_rack_v", false, false));
     fOrdervRack.show();
 
     mtitle = _("Order Rack Horizontally");
     fOrderhRack.set_label(mtitle);
     gui.mainmenu.plugin_menu.append(fOrderhRack);
     fOrderhRack.set_active(false);
-    fOrderhRack.set_parameter(new SwitchParameter("system.order_rack_h", false, false));
+    fOrderhRack.set_parameter(pmap.reg_switch("system.order_rack_h", false, false));
     fOrderhRack.show();
 
     paintbox1.show();
@@ -528,9 +747,9 @@ GxToolBox::GxToolBox(gx_ui::GxUI& ui,
     window.signal_delete_event().connect(
          sigc::bind<gpointer>(sigc::mem_fun(*this, &GxToolBox::on_window_delete_event), d));
     box.add(rbox);
-    paintbox1.add(m_scrolled_window);
+    paintbox1.pack_start(m_scrolled_window);
     m_scrolled_window.add(paintbox);
-    paintbox.add(box);
+    paintbox.pack_start(box);
     window.add(paintbox1);
     window.signal_button_press_event().connect(
         sigc::mem_fun(*this, &GxToolBox::on_button_pressed));
@@ -539,6 +758,7 @@ GxToolBox::GxToolBox(gx_ui::GxUI& ui,
     m_scrolled_window.show();
     rbox.show();
 }
+
 /****************************************************************/
 }
 
