@@ -28,7 +28,7 @@
 
 #include "engine.h"               // NOLINT
 
-namespace gx_gui {
+namespace gx_engine {
 
 /****************************************************************
  ** Global Variables
@@ -38,9 +38,6 @@ namespace gx_gui {
 MidiStandardControllers midi_std_ctr; // map ctrl num -> standard name
 MidiControllerList controller_map; // map ctrl num -> controlled parameters
 
-/* Parameters */
-ParamMap parameter_map; // map id -> parameter, zone -> parameter
-
 
 /****************************************************************
  ** class MidiStandardControllers
@@ -114,7 +111,7 @@ MidiStandardControllers::MidiStandardControllers() {
     }
 }
 
-void MidiStandardControllers::replace(int ctr, string name) {
+void MidiStandardControllers::replace(int ctr, const string& name) {
     map<int, modstring>::iterator i = m.find(ctr);
     if (name.empty()) {
         if (i != m.end()) {
@@ -173,38 +170,38 @@ void MidiStandardControllers::readJSON(gx_system::JsonParser& jp) {
 
 void MidiController::writeJSON(gx_system::JsonWriter& jw) const {
     jw.begin_array();
-    jw.write(param.id());
-    if (param.getControlType() == Parameter::Continuous ||
-        param.getControlType() == Parameter::Enum) {
+    jw.write(param->id());
+    if (param->getControlType() == Parameter::Continuous ||
+        param->getControlType() == Parameter::Enum) {
         jw.write(_lower);
         jw.write(_upper);
     } else {
-        assert(param.getControlType() == Parameter::Switch);
+        assert(param->getControlType() == Parameter::Switch);
     }
     jw.end_array();
 }
 
-MidiController *MidiController::readJSON(gx_system::JsonParser& jp) {
+MidiController *MidiController::readJSON(gx_system::JsonParser& jp, ParamMap& pmap) {
     jp.next(gx_system::JsonParser::begin_array);
     jp.next(gx_system::JsonParser::value_string);
     string id = jp.current_value();
-    if (!parameter_map.hasId(id)) {
+    if (!pmap.hasId(id)) {
         gx_system::gx_print_warning(_("Midi controller settings"),
                                     _("unknown parameter: ") + id);
         while (jp.next() != gx_system::JsonParser::end_array);
         return 0;
     }
-    Parameter& param = parameter_map[id];
+    Parameter& pm = pmap[id];
     float lower = 0, upper = 0;
     bool bad = false;
     bool chg = false;
-    if (param.getControlType() == Parameter::Continuous ||
-        param.getControlType() == Parameter::Enum) {
+    if (pm.getControlType() == Parameter::Continuous ||
+        pm.getControlType() == Parameter::Enum) {
         if (jp.peek() != gx_system::JsonParser::end_array) {
             float pmin, pmax;
-            if (param.hasRange()) {
-                pmin = param.getLowerAsFloat();
-                pmax = param.getUpperAsFloat();
+            if (pm.hasRange()) {
+                pmin = pm.getLowerAsFloat();
+                pmax = pm.getUpperAsFloat();
             } else {
                 bad = true;
                 pmin = pmax = 0;
@@ -231,7 +228,7 @@ MidiController *MidiController::readJSON(gx_system::JsonParser& jp) {
             bad = true;
         }
     } else {
-        if (param.getControlType() != Parameter::Switch) {
+        if (pm.getControlType() != Parameter::Switch) {
             bad = true;
         }
     }
@@ -247,7 +244,7 @@ MidiController *MidiController::readJSON(gx_system::JsonParser& jp) {
             _("recall MIDI state"),
             _("Parameter range outside bounds, changed: ") + id);
     }
-    return new MidiController(param, lower, upper);
+    return new MidiController(pm, lower, upper);
 }
 
 
@@ -271,8 +268,8 @@ MidiControllerList::MidiControllerList()
 void MidiControllerList::on_pgm_chg() {
     int pgm;
     do {
-	pgm = g_atomic_int_get(&program_change);
-    } while (!g_atomic_int_compare_and_exchange(&program_change, pgm, -1));
+	pgm = gx_system::atomic_get(program_change);
+    } while (!gx_system::atomic_compare_and_exchange(&program_change, pgm, -1));
     new_program(pgm);
 }
 
@@ -360,14 +357,14 @@ void MidiControllerList::writeJSON(gx_system::JsonWriter& w) {
     w.end_array(true);
 }
 
-void MidiControllerList::readJSON(gx_system::JsonParser& jp, controller_array& m) {
+void MidiControllerList::readJSON(gx_system::JsonParser& jp, ParamMap& param, controller_array& m) {
     jp.next(gx_system::JsonParser::begin_array);
     while (jp.peek() != gx_system::JsonParser::end_array) {
         jp.next(gx_system::JsonParser::value_number);
         midi_controller_list& l = m[jp.current_value_int()];
         jp.next(gx_system::JsonParser::begin_array);
         while (jp.peek() != gx_system::JsonParser::end_array) {
-            MidiController *p = MidiController::readJSON(jp);
+            MidiController *p = MidiController::readJSON(jp, param);
             if (p) {
                 l.push_front(*p);
             }
@@ -425,7 +422,7 @@ void MidiControllerList::compute_midi_in(void* midi_input_port_buf) {
     for (i = 0; i < event_count; i++) {
         jack_midi_event_get(&in_event, midi_input_port_buf, i);
         if ((in_event.buffer[0] & 0xf0) == 0xc0) {  // program change on any midi channel
-            g_atomic_int_set(&program_change, in_event.buffer[1]);
+            gx_system::atomic_set(&program_change, in_event.buffer[1]);
             pgm_chg();
         } else if ((in_event.buffer[0] & 0xf0) == 0xb0) {   // controller
             set_ctr_val(in_event.buffer[1], in_event.buffer[2]);
@@ -453,7 +450,7 @@ ParameterGroups::~ParameterGroups() {
 }
 
 #ifndef NDEBUG
-void ParameterGroups::group_exists(string id) {
+void ParameterGroups::group_exists(const string& id) {
     if (groups.find(id) == groups.end()) {
 	gx_system::gx_print_error("Debug Check", "Group does not exist: " + id);
     } else {
@@ -461,7 +458,7 @@ void ParameterGroups::group_exists(string id) {
     }
 }
 
-void ParameterGroups::group_is_new(string id) {
+void ParameterGroups::group_is_new(const string& id) {
     if (groups.find(id) != groups.end()) {
 	gx_system::gx_print_error("Debug Check", "Group already exists: " + id);
     }
@@ -480,7 +477,7 @@ ParameterGroups& get_group_table() {
     return groups;
 }
 
-string param_group(string id, bool nowarn) {
+string param_group(const string& id, bool nowarn) {
     static ParameterGroups& groups = get_group_table();
     const string& group_id = id.substr(0, id.find_last_of("."));
     if (nowarn) {
@@ -548,9 +545,6 @@ void compare_parameter(const char *title, Parameter* p1, Parameter* p2, bool all
     if (p1->used != p2->used) {
 	printf("%s[%s]: used different: %d / %d\n", title, p1->_id.c_str(), p1->used, p2->used);
     }
-    if (p1->experimental != p2->experimental) {
-	printf("%s[%s]: experimental different: %d / %d\n", title, p1->_id.c_str(), p1->experimental, p2->experimental);
-    }
     if (p1->c_type != p2->c_type) {
 	printf("%s[%s]: c_type different: %d / %d\n", title, p1->_id.c_str(), p1->c_type, p2->c_type);
     }
@@ -561,8 +555,8 @@ void compare_parameter(const char *title, Parameter* p1, Parameter* p2, bool all
     if (p1->isFloat()) {
 	FloatParameter& f1 = p1->getFloat();
 	FloatParameter& f2 = p2->getFloat();
-	if (&f1.value != &f2.value) {
-	    printf("%s[%s]: value address different: %p / %p\n", title, p1->_id.c_str(), &f1.value, &f2.value);
+	if (f1.value != f2.value) {
+	    printf("%s[%s]: value address different: %p / %p\n", title, p1->_id.c_str(), f1.value, f2.value);
 	}
 	if (f1.lower != f2.lower) {
 	    printf("%s[%s]: float lower different: %g / %g\n", title, p1->_id.c_str(), f1.lower, f2.lower);
@@ -578,7 +572,7 @@ void compare_parameter(const char *title, Parameter* p1, Parameter* p2, bool all
 	}
 	if (all) {
 	    if (f1.value != f2.value) {
-		printf("%s[%s]: float value different: %g / %g\n", title, p1->_id.c_str(), f1.value, f2.value);
+		printf("%s[%s]: float value different: %g / %g\n", title, p1->_id.c_str(), *f1.value, *f2.value);
 	    }
 	    if (f1.json_value != f2.json_value) {
 		printf("%s[%s]: float json value different: %g / %g\n", title, p1->_id.c_str(), f1.json_value, f2.json_value);
@@ -606,26 +600,33 @@ void compare_parameter(const char *title, Parameter* p1, Parameter* p2, bool all
 	assert(false);
 	return;
     }
+    assert(false);
 }
 #endif
 
 /* FloatParameter */
 
-void *FloatParameter::zone() {
-    return &value;
+FloatParameter::~ParameterV() {
+    if (own_var) {
+	delete value;
+    }
 }
 
-void FloatParameter::set(int n, int high, float llimit, float ulimit) {
+void *FloatParameter::zone() {
+    return value;
+}
+
+void FloatParameter::set(float n, float high, float llimit, float ulimit) {
     switch (c_type) {
     case Continuous:
         assert(n >= 0 && n <= high);
-        value = llimit + (static_cast<float>(n) / (high)) * (ulimit - llimit);
+        *value = llimit + (n / high) * (ulimit - llimit);
         break;
     case Switch:
-        value = (2*n > high ? 1.0 : 0.0);
+        *value = (2*n > high ? 1.0 : 0.0);
         break;
     case Enum:
-        value = lower + min(static_cast<float>(n), upper-lower);
+        *value = lower + min(n, upper-lower);
         break;
     default:
         assert(false);
@@ -634,12 +635,12 @@ void FloatParameter::set(int n, int high, float llimit, float ulimit) {
 }
 
 void FloatParameter::set_std_value() {
-    value = std_value;
+    *value = std_value;
 }
 
 void FloatParameter::writeJSON(gx_system::JsonWriter& jw) {
     jw.write_key(_id.c_str());
-    jw.write(value);
+    jw.write(*value);
 }
 
 void FloatParameter::readJSON_value(gx_system::JsonParser& jp) {
@@ -674,9 +675,9 @@ float FloatParameter::getStepAsFloat() const {
 
 /* FloatEnumParameter */
 
-FloatEnumParameter::FloatEnumParameter(string id, string name, const value_pair* vn, bool preset,
-                                       float &v, int sv, bool ctrl, bool exp):
-    FloatParameter(id, name, Enum, preset, v, sv, 0, get_upper(vn), 1, ctrl, exp),
+FloatEnumParameter::FloatEnumParameter(const string& id, const string& name, const value_pair* vn, bool preset,
+                                       float *v, int sv, bool ctrl):
+    FloatParameter(id, name, Enum, preset, v, sv, 0, get_upper(vn), 1, ctrl),
     value_names(vn) {}
 
 const value_pair *FloatEnumParameter::getValueNames() const {
@@ -685,7 +686,7 @@ const value_pair *FloatEnumParameter::getValueNames() const {
 
 void FloatEnumParameter::writeJSON(gx_system::JsonWriter& jw) {
     jw.write_key(_id.c_str());
-    jw.write(value_names[static_cast<int>(round(value))].value_id);
+    jw.write(value_names[static_cast<int>(round(*value))].value_id);
 }
 
 void FloatEnumParameter::readJSON_value(gx_system::JsonParser& jp) {
@@ -714,20 +715,31 @@ void FloatEnumParameter::readJSON_value(gx_system::JsonParser& jp) {
 
 /* IntParameter */
 
-void *IntParameter::zone() {
-    return &value;
+IntParameter::~ParameterV() {
+    if (own_var) {
+	delete value;
+    }
 }
 
-void IntParameter::set(int n, int high, float llimit, float ulimit) {
+void *IntParameter::zone() {
+    return value;
+}
+
+int IntParameter::idx_from_id(string v_id) {
+    assert(false);
+    return 0;
+}
+
+void IntParameter::set(float n, float high, float llimit, float ulimit) {
     switch (c_type) {
     case Continuous:
         assert(false); // not implemented
         break;
     case Switch:
-        value = (2*n > high ? 1065353216 : 0); // FIXME float/int problem
+	assert(false); // not implemented
         break;
     case Enum:
-        assert(false); // not implemented
+        *value = lower + min(static_cast<int>(n), upper-lower);
         break;
     default:
         assert(false);
@@ -736,12 +748,12 @@ void IntParameter::set(int n, int high, float llimit, float ulimit) {
 }
 
 void IntParameter::set_std_value() {
-    value = std_value;
+    *value = std_value;
 }
 
 void IntParameter::writeJSON(gx_system::JsonWriter& jw) {
     jw.write_key(_id.c_str());
-    jw.write(value);
+    jw.write(*value);
 }
 
 void IntParameter::readJSON_value(gx_system::JsonParser& jp) {
@@ -767,18 +779,28 @@ float IntParameter::getUpperAsFloat() const {
 
 /* EnumParameter */
 
-EnumParameter::EnumParameter(string id, string name, const value_pair* vn, bool preset,
-                             int &v, int sv, bool ctrl, bool exp):
-    IntParameter(id, name, Enum, preset, v, sv, 0, get_upper(vn), ctrl, exp),
+EnumParameter::EnumParameter(const string& id, const string& name, const value_pair* vn, bool preset,
+                             int *v, int sv, bool ctrl):
+    IntParameter(id, name, Enum, preset, v, sv, 0, get_upper(vn), ctrl),
     value_names(vn) {}
 
 const value_pair *EnumParameter::getValueNames() const {
     return value_names;
 }
 
+int EnumParameter::idx_from_id(string v_id) {
+    int n = 0;
+    for (; n <= upper; n++) {
+        if (v_id == value_names[n].value_id) {
+            return n;
+        }
+    }
+    return -1;
+}
+
 void EnumParameter::writeJSON(gx_system::JsonWriter& jw) {
     jw.write_key(_id.c_str());
-    jw.write(value_names[value].value_id);
+    jw.write(value_names[*value].value_id);
 }
 
 void EnumParameter::readJSON_value(gx_system::JsonParser& jp) {
@@ -789,13 +811,8 @@ void EnumParameter::readJSON_value(gx_system::JsonParser& jp) {
         return;
     }
     jp.check_expect(gx_system::JsonParser::value_string);
-    int n = 0;
-    for (; n <= upper; n++) {
-        if (jp.current_value() == value_names[n].value_id) {
-            break;
-        }
-    }
-    if (n > upper) {
+    int n = idx_from_id(jp.current_value());
+    if (n < 0) {
         gx_system::gx_print_warning(
             _("read parameter"), (boost::format(_("parameter %1%: unknown enum value: %2%"))
                                % _id % jp.current_value()).str());
@@ -806,20 +823,26 @@ void EnumParameter::readJSON_value(gx_system::JsonParser& jp) {
 
 /* unsigned IntParameter */
 
-void *UIntParameter::zone() {
-    return &value;
+UIntParameter::~ParameterV() {
+    if (own_var) {
+	delete value;
+    }
 }
 
-void UIntParameter::set(int n, int high, float llimit, float ulimit) {
+void *UIntParameter::zone() {
+    return value;
+}
+
+void UIntParameter::set(float n, float high, float llimit, float ulimit) {
     switch (c_type) {
     case Continuous:
         assert(false); // not implemented
         break;
     case Switch:
-        value = (2*n > high ? 65535 : 0); // FIXME float/int problem
+	assert(false); // not implemented
         break;
     case Enum:
-        assert(false); // not implemented
+        *value = lower + min(static_cast<unsigned int>(n), upper-lower);
         break;
     default:
         assert(false);
@@ -828,12 +851,12 @@ void UIntParameter::set(int n, int high, float llimit, float ulimit) {
 }
 
 void UIntParameter::set_std_value() {
-    value = std_value;
+    *value = std_value;
 }
 
 void UIntParameter::writeJSON(gx_system::JsonWriter& jw) {
     jw.write_key(_id.c_str());
-    jw.write(value);
+    jw.write(*value);
 }
 
 void UIntParameter::readJSON_value(gx_system::JsonParser& jp) {
@@ -859,9 +882,9 @@ float UIntParameter::getUpperAsFloat() const {
 
 /* UEnumParameter */
 
-UEnumParameter::UEnumParameter(string id, string name, const value_pair* vn, bool preset,
-                             unsigned int &v, unsigned int sv, bool ctrl, bool exp):
-    UIntParameter(id, name, Enum, preset, v, sv, 0, get_upper(vn), ctrl, exp),
+UEnumParameter::UEnumParameter(const string& id, const string& name, const value_pair* vn, bool preset,
+                             unsigned int *v, unsigned int sv, bool ctrl):
+    UIntParameter(id, name, Enum, preset, v, sv, 0, get_upper(vn), ctrl),
     value_names(vn) {}
 
 const value_pair *UEnumParameter::getValueNames() const {
@@ -870,7 +893,7 @@ const value_pair *UEnumParameter::getValueNames() const {
 
 void UEnumParameter::writeJSON(gx_system::JsonWriter& jw) {
     jw.write_key(_id.c_str());
-    jw.write(value_names[value].value_id);
+    jw.write(value_names[*value].value_id);
 }
 
 void UEnumParameter::readJSON_value(gx_system::JsonParser& jp) {
@@ -899,14 +922,20 @@ void UEnumParameter::readJSON_value(gx_system::JsonParser& jp) {
 
 /* BoolParameter */
 
-void *BoolParameter::zone() {
-    return &value;
+BoolParameter::~ParameterV() {
+    if (own_var) {
+	delete value;
+    }
 }
 
-void BoolParameter::set(int n, int high, float llimit, float ulimit) {
+void *BoolParameter::zone() {
+    return value;
+}
+
+void BoolParameter::set(float n, float high, float llimit, float ulimit) {
     switch (c_type) {
     case Switch:
-        value = (2*n > high);
+        *value = (2*n > high);
     default:
         assert(false);
         break;
@@ -914,12 +943,12 @@ void BoolParameter::set(int n, int high, float llimit, float ulimit) {
 }
 
 void BoolParameter::set_std_value() {
-    value = std_value;
+    *value = std_value;
 }
 
 void BoolParameter::writeJSON(gx_system::JsonWriter& jw) {
     jw.write_key(_id.c_str());
-    jw.write(value);
+    jw.write(*value);
 }
 
 void BoolParameter::readJSON_value(gx_system::JsonParser& jp) {
@@ -949,7 +978,7 @@ void SwitchParameter::set_std_value() {
     set(std_value);
 }
 
-void SwitchParameter::set(int n, int high, float llimit, float ulimit) {
+void SwitchParameter::set(float n, float high, float llimit, float ulimit) {
     assert(false);
 }
 
@@ -1007,7 +1036,7 @@ void *FileParameter::zone() {
     return &value;
 }
 
-void FileParameter::set(int n, int high, float llimit, float ulimit) {
+void FileParameter::set(float n, float high, float llimit, float ulimit) {
     assert(false);
 }
 
@@ -1067,21 +1096,27 @@ void FileParameter::copy(const string& destination) {
 
 /* StringParameter */
 
-void *StringParameter::zone() {
-    return &value;
+StringParameter::~ParameterV() {
+    if (own_var) {
+	delete value;
+    }
 }
 
-void StringParameter::set(int n, int high, float llimit, float ulimit) {
+void *StringParameter::zone() {
+    return value;
+}
+
+void StringParameter::set(float n, float high, float llimit, float ulimit) {
     assert(false);
 }
 
 void StringParameter::set_std_value() {
-    value = std_value;
+    *value = std_value;
 }
 
 void StringParameter::writeJSON(gx_system::JsonWriter& jw) {
     jw.write_key(_id.c_str());
-    jw.write(value);
+    jw.write(*value);
 }
 
 void StringParameter::readJSON_value(gx_system::JsonParser& jp) {
@@ -1104,6 +1139,9 @@ ParamMap::ParamMap()
 }
 
 ParamMap::~ParamMap() {
+    for (iterator i = id_map.begin(); i != id_map.end(); ++i) {
+	delete i->second;
+    }
 }
 
 #ifndef NDEBUG
@@ -1124,7 +1162,7 @@ void ParamMap::check_addr(const void *p) {
         cerr << "zone not found: " << p << endl;
     }
 }
-void ParamMap::check_id(string id) {
+void ParamMap::check_id(const string& id) {
     if (!hasId(id)) {
         cerr << "string-id not found: " << id << endl;
     }
@@ -1136,12 +1174,66 @@ void ParamMap::check_p(const char *p) {
     }
 }
 
-void ParamMap::dump() {
-    printf("parameter map dump\n");
+void ParamMap::dump(const string& fmt) {
+    gx_system::JsonWriter *p = 0;
+    gx_system::JsonWriter jw;
+    if (fmt == "json") {
+	jw.set_stream(&cout);
+	p = &jw;
+	jw.begin_array();
+	jw.newline();
+    } else {
+	printf("parameter map dump\n");
+    }
     for (iterator i = id_map.begin(); i != id_map.end(); ++i) {
-        printf("P: %s\n", i->second->id().c_str());
+	i->second->dump(p);
+    }
+    if (p) {
+	jw.end_array();
+	jw.close();
+    } else {
+	printf("---------------------\n");
+    }
+}
+
+void Parameter::dump(gx_system::JsonWriter *jw) {
+    if (jw) {
+	jw->begin_array();
+	jw->write(id());
+	jw->write(l_group());
+	jw->write(l_name());
+	switch (v_type) {
+	case tp_float:  jw->write("f"); jw->write(getFloat().get_value());     break;
+	case tp_int:    jw->write("i"); jw->write(getInt().get_value());       break;
+	case tp_uint:   jw->write("u"); jw->write(getUInt().get_value());      break;
+	case tp_bool:   jw->write("b"); jw->write(getBool().get_value());      break;
+	case tp_switch: jw->write("s"); jw->write(getSwitch().get());          break;
+	case tp_file:   jw->write("F"); jw->write(getFile().get_parse_name()); break;
+	case tp_string: jw->write("S"); jw->write(getString().get_value());    break;
+	default: assert(false);
+	}
+	jw->write(getLowerAsFloat());
+	jw->write(getUpperAsFloat());
+	jw->write(getStepAsFloat());
+	const value_pair *vn = getValueNames();
+	if (vn) {
+	    jw->begin_array();
+	    for (int n = 0; ; ++n) {
+		if (!vn[n].value_id) {
+		    break;
+		}
+		jw->begin_array();
+		jw->write(vn[n].value_id);
+		jw->write(vn[n].value_label ? vn[n].value_label : vn[n].value_id);
+		jw->end_array();
+	    }
+	    jw->end_array();
+	}
+	jw->end_array();
+	jw->newline();
+    } else {
+	printf("P: %s vt=%d ct=%d c=%d\n", _id.c_str(), v_type, c_type, controllable);
     }
-    printf("---------------------\n");
 }
 #endif
 
