@@ -81,7 +81,7 @@ void MeasureThreadsafe::print(bool verbose) {
     } else {
         pn = m;
     }
-    g_atomic_pointer_set(&pmeasure, pn);
+    atomic_set(&pmeasure, pn);
     p->print(verbose);
     p->reset();
 }
@@ -197,12 +197,13 @@ CmdlineOptions::CmdlineOptions()
       jack_uuid(),
       jack_uuid2(),
       load_file(shellvar("GUITARIX_LOAD_FILE")),
-      builder_dir(GX_BUILDER_DIR1),
-      style_dir(GX_STYLE_DIR1),
+      builder_dir(GX_BUILDER_DIR),
+      style_dir(GX_STYLE_DIR),
+      factory_dir(GX_FACTORY_DIR),
       pixmap_dir(GX_PIXMAPS_DIR),
       user_dir(),
       plugin_dir(),
-      sys_IR_dir(GX_SOUND_DIR1),
+      sys_IR_dir(GX_SOUND_DIR),
       IR_pathlist(),
       rcset(shellvar("GUITARIX_RC_STYLE")),
       lterminal(false),
@@ -356,7 +357,7 @@ string CmdlineOptions::get_opskin() {
     return opskin;
 }
 
-static void log_terminal(const string& msg, GxMsgType tp) {
+static void log_terminal(const string& msg, GxMsgType tp, bool plugged) {
     const char *t;
     switch (tp) {
     case kInfo:    t = "I"; break;
@@ -364,7 +365,9 @@ static void log_terminal(const string& msg, GxMsgType tp) {
     case kError:   t = "E"; break;
     default:       t = "?"; break;
     }
-    cerr << t << " " << msg << endl;
+    if (!plugged) {
+	cerr << t << " " << msg << endl;
+    }
 }
 
 void CmdlineOptions::make_ending_slash(string& dirpath) {
@@ -386,10 +389,12 @@ void CmdlineOptions::process(int argc, char** argv) {
              << endl;
         exit(0);
     }
+#ifdef NDEBUG
     if (argc > 1) {
 	throw gx_system::GxFatalError(
 	    string("unknown argument on command line: ")+argv[1]);
     }
+#endif
     if (clear && !rcset.empty()) {
 	throw Glib::OptionError(
 	    Glib::OptionError::BAD_VALUE,
@@ -402,6 +407,7 @@ void CmdlineOptions::process(int argc, char** argv) {
 
     make_ending_slash(builder_dir);
     make_ending_slash(style_dir);
+    make_ending_slash(factory_dir);
     make_ending_slash(pixmap_dir);
     make_ending_slash(user_dir);
     make_ending_slash(plugin_dir);
@@ -435,7 +441,8 @@ Logger::Logger()
       msgmutex(),
       got_new_msg(),
       ui_thread(),
-      handlers() {
+      handlers(),
+      queue_all_msgs(true) {
 }
 
 Logger& Logger::get_logger() {
@@ -462,6 +469,14 @@ Logger::msg_signal& Logger::signal_message() {
     return handlers;
 }
 
+void Logger::unplug_queue() {
+    if (!queue_all_msgs) {
+	return;
+    }
+    queue_all_msgs = false;
+    write_queued();
+}
+
 void Logger::write_queued() {
     if (handlers.empty()) {
 	return;
@@ -475,7 +490,7 @@ void Logger::write_queued() {
 
     // feed throught the handler(s)
     for (list<logmsg>::iterator i = l.begin(); i != l.end(); ++i) {
-	handlers(i->msg, i->msgtype);
+	handlers(i->msg, i->msgtype, i->plugged);
     }
 }
 
@@ -498,13 +513,16 @@ void Logger::print(const char* func, const string& msg, GxMsgType msgtype) {
     if (handlers.empty() || !(pthread_equal(pthread_self(), ui_thread))) {
 	boost::mutex::scoped_lock lock(msgmutex);
 	// defer output
-        msglist.push_back(logmsg(m, msgtype));
+        msglist.push_back(logmsg(m, msgtype, false));
 	if (!handlers.empty() && msglist.size() == 1) {
 	    (*got_new_msg)();
 	}
     } else {
 	write_queued();
-	handlers(m, msgtype);
+	handlers(m, msgtype, false);
+	if (queue_all_msgs) {
+	    msglist.push_back(logmsg(m, msgtype, true));
+	}
     }
 }
 
@@ -575,12 +593,12 @@ GxExit& GxExit::get_instance() {
 // ----start jack if possible
 bool gx_start_jack() {
     // first, let's try via qjackctl
-    if (gx_system::gx_system_call("which", "qjackctl", true) == SYSTEM_OK) {
-        if (gx_system::gx_system_call("qjackctl", "--start", true, true) == SYSTEM_OK) {
+    if (gx_system::gx_system_call("which qjackctl", true) == SYSTEM_OK) {
+        if (gx_system::gx_system_call("qjackctl --start", true, true) == SYSTEM_OK) {
             sleep(5);
 
             // let's check it is really running
-            if (gx_system::gx_system_call("pgrep", "jackd", true) == SYSTEM_OK) {
+            if (gx_system::gx_system_call("pgrep jackd", true) == SYSTEM_OK) {
                 return true;
             }
         }
@@ -588,7 +606,7 @@ bool gx_start_jack() {
 
     // qjackctl not found or not started, let's try .jackdrc
     string jackdrc = "$HOME/.jackdrc";
-    if (gx_system::gx_system_call("ls", jackdrc.c_str(), true, false) == SYSTEM_OK) {
+    if (gx_system::gx_system_call("ls " + jackdrc, true, false) == SYSTEM_OK) {
         // open it
         jackdrc = string(getenv("HOME")) + string("/") + ".jackdrc";
         string cmdline = "";
@@ -602,12 +620,12 @@ bool gx_start_jack() {
 
         // launch jackd
         if (!cmdline.empty())
-            if (gx_system::gx_system_call(cmdline.c_str(), "", true, true) == SYSTEM_OK) {
+            if (gx_system::gx_system_call(cmdline, true, true) == SYSTEM_OK) {
 
                 sleep(2);
 
                 // let's check it is really running
-                if (gx_system::gx_system_call("pgrep", "jackd", true) == SYSTEM_OK) {
+                if (gx_system::gx_system_call("pgrep jackd", true) == SYSTEM_OK) {
                     return true;
                 }
             }
@@ -617,13 +635,10 @@ bool gx_start_jack() {
 }
 
 // ---- gx_head system function
-int gx_system_call(const char* name1,
-                   const char* name2,
+int gx_system_call(const string& cmd,
                    const bool  devnull,
                    const bool  escape) {
-    string str(name1);
-    str.append(" ");
-    str.append(name2);
+    string str = cmd;
 
     if (devnull)
         str.append(" 1>/dev/null 2>&1");
@@ -642,20 +657,4 @@ int gx_system_call(const char* name1,
     return rc;
 }
 
-// polymorph1
-int gx_system_call(const char*   name1,
-                   const string& name2,
-                   const bool  devnull,
-                   const bool  escape) {
-    return gx_system_call(name1, name2.c_str(), devnull, escape);
-}
-
-// polymorph3
-int gx_system_call(const string& name1,
-                   const char*   name2,
-                   const bool  devnull,
-                   const bool  escape) {
-    return gx_system_call(name1.c_str(), name2, devnull, escape);
-}
-
 } /* end of gx_system namespace */
