@@ -41,7 +41,7 @@ public:
     //FIXME:
     float filebutton;
 
-    void register_parameter();
+    void register_parameter(ParamMap& pmap);
 };
 
 extern AudioVariables audio;
@@ -65,9 +65,10 @@ private:
 public:
     Plugin plugin;
     ModuleSelectorFromList(
-	ModuleSequencer& seq, gx_ui::GxUI& ui, const char* id, const char* name,
-	PluginDef **module_ids,	const char* select_id,
+	EngineControl& seq, gx_ui::GxUI& ui, const char* id, const char* name,
+	plugindef_creator module_ids[],	const char* select_id,
 	const char* select_name, const char** groups = 0, int flags = 0);
+    ~ModuleSelectorFromList();
     void set_module();
 };
 
@@ -97,7 +98,7 @@ public:
     CabinetConvolver cabinet;
     ContrastConvolver contrast;
 public:
-    GxEngine(const string& plugin_dir, gx_gui::ParameterGroups& groups);
+    GxEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups);
     ~GxEngine();
     void set_jack(gx_jack::GxJack *jack) { midiaudiobuffer.set_jack(jack); }
 };
