@@ -45,6 +45,7 @@
 #include <gxwmm/simplevaluedisplay.h>
 #include <gxwmm/fastmeter.h>
 #include <gxwmm/tuner.h>
+#include <gxwmm/racktuner.h>
 #include <gxwmm/waveview.h>
 #include <gtkmm/box.h>
 #include <gtkmm/alignment.h>
@@ -59,6 +60,24 @@
 
 namespace gx_gui {
 
+class GxMainInterface;
+
+class UiBuilderImpl: public gx_engine::UiBuilderBase {
+private:
+    static GxMainInterface *intf;
+    static void openVerticalBox_(const char* label);
+    static void openHorizontalBox_(const char* label);
+    static void openHorizontalhideBox_(const char* label);
+    static void create_small_rackknob_(const char *id, const char *label);
+    static void create_master_slider_(const char *id, const char *label);
+    static void create_selector_(const char *id);
+    static void closeBox_();
+    static void load_glade_(const char *data);
+    void load(gx_engine::Plugin *p);
+public:
+    UiBuilderImpl(GxMainInterface *i);
+};
+
 /****************************************************************
  **
  ** ---------------- the main GUI class ----------------
@@ -86,7 +105,7 @@ class GuiVariables {
     int                 main_yorg;
     string              skin;
 
-    void register_gui_parameter();
+    void register_gui_parameter(gx_engine::ParamMap& pmap);
 };
 
 extern GuiVariables guivar;
@@ -99,7 +118,6 @@ class GlobalWidgets {
     /* global GUI widgets */
     GtkWidget*          midibox;
     GtkWidget*          patch_info;
-    GtkWidget*          tuner_widget;
     GtkWidget*          rack_widget;
     GtkWidget*          srack_widget;
     GtkWidget*          rack_tool_bar;
@@ -113,70 +131,6 @@ void gx_start_stop_jconv(GtkWidget*, gpointer);
 
 /****************************************************************/
 
-/* ---- linking menu items and parameter ---- */
-class MenuCheckItem: public Gtk::CheckMenuItem {
- private:
-    SwitchParameter* param;
-    void on_my_activate();
- public:
-    // FIXME not gtk-2.12: MenuCheckItem() { set_use_underline(); }
-    MenuCheckItem(): Gtk::CheckMenuItem("", true), param() {}
-    MenuCheckItem(const char *label): Gtk::CheckMenuItem(label, true), param() {}
-    void set_parameter(SwitchParameter *p);
-    void add_parameter(SwitchParameter *p);
-    SwitchParameter * get_parameter();
-};
-
-class MenuCheckItemUiBool: public Gtk::CheckMenuItem, gx_ui::GxUiItemBool {
- private:
-    virtual void reflectZone();
-    void on_my_activate();
- public:
-    // FIXME not gtk-2.12: MenuCheckItem() { set_use_underline(); }
-    MenuCheckItemUiBool(gx_ui::GxUI* ui, bool* zone);
-};
-
-/****************************************************************/
-
-/* ---- linking menu items and parameter ---- */
-class RadioCheckItem: public Gtk::RadioMenuItem {
- private:
-    SwitchParameter* param;
-    void on_my_toggled();
- public:
-    // FIXME not gtk-2.12: MenuCheckItem() { set_use_underline(); }
-    RadioCheckItem(Gtk::RadioMenuItem::Group& group, const char *label=""):
-	Gtk::RadioMenuItem(group, label, true), param() {}
-    void set_parameter(SwitchParameter *p);
-    SwitchParameter * get_parameter();
-};
-
-/****************************************************************/
-
-class ToggleCheckButton: public Gtk::ToggleButton {
- private:
-    SwitchParameter* param;
-    void on_my_toggled();
- public:
-    Gtk::Label m_label;
-    void set_parameter(SwitchParameter *p);
-    SwitchParameter * get_parameter();
-    ToggleCheckButton();
-    ~ToggleCheckButton();
-};
-
-class ToggleCheckButtonUiBool: public Gtk::ToggleButton, gx_ui::GxUiItemBool {
- private:
-    virtual void reflectZone();
-    void on_my_toggled();
- public:
-    Gtk::Label m_label;
-    ToggleCheckButtonUiBool(gx_ui::GxUI* ui, bool* zone);
-    ~ToggleCheckButtonUiBool();
-};
-
-/****************************************************************/
-
 class UiRegler: gx_ui::GxUiItemFloat, protected Gtk::Adjustment {
  protected:
     Gxw::Regler *m_regler;
@@ -184,41 +138,30 @@ class UiRegler: gx_ui::GxUiItemFloat, protected Gtk::Adjustment {
     virtual void reflectZone();
  public:
     static GtkWidget* create(gx_ui::GxUI& ui, Gxw::Regler *regler, string id, bool show_value);
-    UiRegler(gx_ui::GxUI &ui, FloatParameter &param, Gxw::Regler *regler, bool show_value);
+    UiRegler(gx_ui::GxUI &ui, gx_engine::FloatParameter &param, Gxw::Regler *regler, bool show_value);
     virtual ~UiRegler();
     GtkWidget *get_widget() { return GTK_WIDGET(m_regler->gobj()); }
 };
 
 /****************************************************************/
 
-class UiSelector {
- protected:
+class UiSelectorBase {
+protected:
     Gxw::Selector m_selector;
-    void init(Parameter& param);
- public:
-    UiSelector();
-    static GtkWidget* create(gx_ui::GxUI& ui, string id, const char *widget_name);
+public:
+    UiSelectorBase(gx_engine::Parameter& param);
+    void set_name(Glib::ustring n) { m_selector.set_name(n); }
     GtkWidget *get_widget() { return GTK_WIDGET(m_selector.gobj()); }
 };
 
-/****************************************************************/
-
-class UiSelectorFloat: public UiSelector, gx_ui::GxUiItemFloat, protected Gtk::Adjustment {
- protected:
+template <class T>
+class UiSelector: public UiSelectorBase, private gx_ui::GxUiItemV<T>, protected Gtk::Adjustment {
+private:
     virtual void reflectZone();
     void on_value_changed();
- public:
-    UiSelectorFloat(gx_ui::GxUI& ui, FloatParameter &param);
-};
-
-/****************************************************************/
-
-class UiSelectorInt: public UiSelector, gx_ui::GxUiItemInt, protected Gtk::Adjustment {
- protected:
-    virtual void reflectZone();
-    void on_value_changed();
- public:
-    UiSelectorInt(gx_ui::GxUI& ui, IntParameter &param);
+public:
+    UiSelector(gx_ui::GxUI& ui, gx_engine::ParameterV<T> &param);
+    ~UiSelector();
 };
 
 /****************************************************************/
@@ -231,7 +174,7 @@ class UiReglerWithCaption: public UiRegler {
     static GtkWidget* create(gx_ui::GxUI& ui, Gxw::Regler *regler, string id, bool show_value);
     static GtkWidget* create(gx_ui::GxUI& ui, Gxw::Regler *regler, string id,
                              Glib::ustring label, bool show_value);
-    UiReglerWithCaption(gx_ui::GxUI &ui, FloatParameter &param, Gxw::Regler *regler,
+    UiReglerWithCaption(gx_ui::GxUI &ui, gx_engine::FloatParameter &param, Gxw::Regler *regler,
                         Glib::ustring label, bool show_value);
     GtkWidget *get_widget() { return GTK_WIDGET(m_box.gobj());}
 };
@@ -246,7 +189,21 @@ class UiRackReglerWithCaption: public UiRegler {
     static GtkWidget* create(gx_ui::GxUI& ui, Gxw::Regler *regler, string id);
     static GtkWidget* create(gx_ui::GxUI& ui, Gxw::Regler *regler, string id,
                              Glib::ustring label);
-    UiRackReglerWithCaption(gx_ui::GxUI &ui, FloatParameter &param, Gxw::Regler *regler,
+    UiRackReglerWithCaption(gx_ui::GxUI &ui, gx_engine::FloatParameter &param, Gxw::Regler *regler,
+                            Glib::ustring label);
+    GtkWidget *get_widget() { return GTK_WIDGET(m_box.gobj());}
+};
+/****************************************************************/
+
+class UiRackMasterRegler: public UiRegler {
+ private:
+    Gtk::Label m_label;
+    Gtk::HBox m_box;
+ public:
+    static GtkWidget* create(gx_ui::GxUI& ui, Gxw::Regler *regler, string id);
+    static GtkWidget* create(gx_ui::GxUI& ui, Gxw::Regler *regler, string id,
+                             Glib::ustring label);
+    UiRackMasterRegler(gx_ui::GxUI &ui, gx_engine::FloatParameter &param, Gxw::Regler *regler,
                             Glib::ustring label);
     GtkWidget *get_widget() { return GTK_WIDGET(m_box.gobj());}
 };
@@ -261,106 +218,13 @@ class UiRackRegler: public UiRegler {
     static GtkWidget* create(gx_ui::GxUI& ui, Gxw::Regler *regler, string id);
     static GtkWidget* create(gx_ui::GxUI& ui, Gxw::Regler *regler, string id,
                              Glib::ustring label);
-    UiRackRegler(gx_ui::GxUI &ui, FloatParameter &param, Gxw::Regler *regler,
+    UiRackRegler(gx_ui::GxUI &ui, gx_engine::FloatParameter &param, Gxw::Regler *regler,
                  Glib::ustring label);
     GtkWidget *get_widget() { return GTK_WIDGET(m_box.gobj());}
 };
 
 /****************************************************************/
 
-class UiSwitch: public Gxw::Switch {
- public:
-    explicit UiSwitch(const char *sw_type);
-    GtkWidget *get_widget() { return GTK_WIDGET(gobj());}
-    static UiSwitch *new_switch(gx_ui::GxUI& ui, const char *sw_type, Parameter &param);
-    static UiSwitch *new_switch(gx_ui::GxUI& ui, const char *sw_type, string id) {
-        if (!parameter_map.hasId(id)) return 0;
-        return new_switch(ui, sw_type, parameter_map[id]);
-    }
-    static GtkWidget *create(gx_ui::GxUI& ui, const char *sw_type, string id) {
-        return new_switch(ui, sw_type, id)->get_widget();}
-};
-
-/****************************************************************/
-
-class UiSwitchFloat: public UiSwitch, gx_ui::GxUiItemFloat {
- protected:
-    void on_toggled();
-    virtual void reflectZone();
- public:
-    UiSwitchFloat(gx_ui::GxUI& ui, const char *sw_type, FloatParameter &param);
-};
-
-/****************************************************************/
-
-class UiSwitchBool: public UiSwitch, gx_ui::GxUiItemBool {
- protected:
-    void on_toggled();
-    virtual void reflectZone();
- public:
-    UiSwitchBool(gx_ui::GxUI& ui, const char *sw_type, BoolParameter &param);
-};
-
-/****************************************************************/
-
-class UiSwitchWithCaption {
- private:
-    Gtk::Label m_label;
-    Gtk::Box *m_box;
- protected:
-    UiSwitch *m_switch;
- public:
-    static GtkWidget* create(gx_ui::GxUI& ui, const char *sw_type, string id,
-                             Gtk::PositionType pos);
-    static GtkWidget* create(gx_ui::GxUI& ui, const char *sw_type, string id,
-                             Glib::ustring label, Gtk::PositionType pos);
-    UiSwitchWithCaption(gx_ui::GxUI &ui, const char *sw_type, Parameter &param,
-                        Glib::ustring label, Gtk::PositionType pos);
-    ~UiSwitchWithCaption();
-    GtkWidget *get_widget() { return GTK_WIDGET(m_box->gobj()); }
-};
-
-/****************************************************************/
-
-class UiCabSwitch: public UiSwitchWithCaption {
- private:
-    void on_switch_toggled();
- public:
-    static GtkWidget* create(gx_ui::GxUI& ui, string id, Glib::ustring label);
-    UiCabSwitch(gx_ui::GxUI &ui, Parameter &param, Glib::ustring label);
-};
-
-/****************************************************************/
-
-class UiContrastSwitch: public UiSwitchWithCaption {
- private:
-    void on_switch_toggled();
- public:
-    static GtkWidget* create(gx_ui::GxUI& ui, string id, Glib::ustring label);
-    UiContrastSwitch(gx_ui::GxUI &ui, Parameter &param, Glib::ustring label);
-};
-
-/****************************************************************/
-
-struct uiTuner : public Gtk::Alignment, private gx_ui::GxUiItemFloat {
- private:
-    Gxw::Tuner fTuner;
-    Gtk::Fixed fBox;
-    Gtk::EventBox eBox;
-    Gxw::Wheel wheel;
-    float refpitch;
-    gx_gui::FloatParameter refpitch_param;
-    Gtk::Adjustment adjust;
-    gx_engine::TunerAdapter& adapt;
-    void freq_changed();
-    void reflectZone();
-    void on_value_changed();
- public:
-    uiTuner(gx_engine::TunerAdapter& a, gx_ui::GxUI& ui);
-};
-
-/****************************************************************/
-
 class PToggleButton {
 private:
     Gtk::Label m_label;
@@ -411,15 +275,45 @@ class GxUiRadioMenu: public gx_ui::GxUiItemUInt {
 private:
     vector<Gtk::RadioMenuItem*> items;
     Gtk::RadioButtonGroup group;
-    UIntParameter& param;
+    gx_engine::UIntParameter& param;
     virtual void reflectZone();
     void on_activate(int i);
 public:
-    GxUiRadioMenu(gx_ui::GxUI* ui, UIntParameter& param);
+    GxUiRadioMenu(gx_ui::GxUI* ui, gx_engine::UIntParameter& param);
     virtual ~GxUiRadioMenu();
     void setup(Gtk::MenuShell& menucont, Glib::RefPtr<Gtk::AccelGroup>& ag);
 };
 
+
+/****************************************************************
+ ** class SelectJackControlPgm
+ */
+
+class SelectJackControlPgm: public Gtk::Window {
+private:
+    Gtk::Label  *description;
+    Gtk::Entry  *customstarter;
+    Gtk::ComboBox *startercombo;
+    Gtk::CheckButton *dontask;
+    gx_engine::IntParameter& starter;
+    gx_engine::StringParameter& starter_cmd;
+    gx_engine::SwitchParameter& ask;
+    sigc::signal<void> close;
+    void on_starter_changed();
+    void on_ok_button();
+    void on_cancel_button();
+    bool on_delete_event(GdkEventAny* event);
+    static SelectJackControlPgm* create_from_builder(BaseObjectType* cobject, Glib::RefPtr<GxBuilder> bld, gx_engine::ParamMap& pmap) {
+	return new SelectJackControlPgm(cobject, bld, pmap);
+    }
+    SelectJackControlPgm(BaseObjectType* cobject, Glib::RefPtr<GxBuilder> bld, gx_engine::ParamMap& pmap);
+public:
+    ~SelectJackControlPgm();
+    static SelectJackControlPgm* create(gx_ui::GxUI *ui, gx_system::CmdlineOptions& opt, gx_engine::ParamMap& pmap);
+    sigc::signal<void>& signal_close() { return close; }
+};
+
+
 /****************************************************************
  ** class TextLoggingBox
  */
@@ -440,7 +334,7 @@ private:
     void set_color();
     bool on_delete_event();
     void set_expander_color(const char *color);
-    void show_msg(string msgbuf, gx_system::GxMsgType msgtype);
+    void show_msg(string msgbuf, gx_system::GxMsgType msgtype, bool plugged);
 public:
     TextLoggingBox(const char* label);
     ~TextLoggingBox();
@@ -494,12 +388,6 @@ public:
     Gtk::Menu          preset_submenu[GX_NUM_OF_PRESET_LISTS];
     Gtk::MenuItem      preset_factory_settings_label;
     Gtk::Menu          preset_factory_settings_menu;
-    Gtk::MenuItem      factory_funkmuscle_label;
-    Gtk::MenuItem      factory_zettberlin_label;
-    Gtk::MenuItem      factory_autoandimat_label;
-    Gtk::MenuItem      factory_StudioDave_label;
-    Gtk::MenuItem      factory_JP_label;
-    Gtk::Menu          factory_sub_menu[5];
 
     // preset more menu
     Gtk::MenuItem      preset_patch_info_item;
@@ -542,10 +430,11 @@ public:
     Gtk::Menu          options_menu;
     Gtk::CheckMenuItem options_meterbridge;
     MenuCheckItem      fShowTuner;
+    MenuCheckItem      fShowValue;
     Gtk::MenuItem      skin_menu_label;
     Gtk::Menu          skin_menu;
     Gtk::RadioMenuItem::Group skingroup;
-    MenuCheckItem      fSetMouse;
+    Gtk::MenuItem      select_jack_cmd;
     MenuCheckItem      fShowLogger;
     MenuCheckItem      fShowTooltips;
     MenuCheckItem      fMidiInPreset;
@@ -568,7 +457,7 @@ public:
 
     void addJackServerMenu(GxMainInterface& intf);
 
-    MainMenu(gx_ui::GxUI& ui, const gx_system::CmdlineOptions& options);
+    MainMenu(gx_ui::GxUI& ui, const gx_system::CmdlineOptions& options, gx_engine::ParamMap& pmap);
     void setup(GxMainInterface& intf);
 };
 
@@ -579,12 +468,14 @@ private:
 public://FIXME
     Gtk::Window           fWindow;
     gx_system::CmdlineOptions& options;
+    gx_engine::ParamMap&  pmap;
     // for key acclerators
     Glib::RefPtr<Gtk::AccelGroup> fAccelGroup;
     void portmap_connection_changed(string port1, string port2, bool conn);
 private:
     bool                  in_session;
     gx_portmap::PortMapWindow* portmap_window;
+    SelectJackControlPgm *select_jack_control;
     void                  on_portmap_activate();
     void                  on_portmap_response(int);
     /* engine status and switch */
@@ -609,15 +500,14 @@ private:
 
     void                  do_program_change(int);
     bool                  on_meter_button_release(GdkEventButton* ev);
-    void                  on_tuner_activate();
     void                  gx_show_menu_settings(GtkWidget *widget, gpointer data);
+    void                  on_value_activate();
     void                  on_log_activate();
     void                  on_rack_activate();
     void                  on_rrack_activate();
     void                  on_srack_activate();
     void                  on_toolbar_activate();
     void                  on_tube_activate();
-    void                  set_mouse_mode();
     void                  on_show_oscilloscope(bool);
     bool                  on_refresh_oscilloscope();
     void                  on_oscilloscope_post_pre(int post_pre);
@@ -629,12 +519,16 @@ private:
     void                  jack_connection_change();
     void                  save_window_position();
     void                  refresh_latency_menu();
-    bool                  connect_jack(bool v);
+    bool                  start_jack();
+    void                  connect_jack(bool v);
     void                  gx_jack_connection();
     bool                  survive_jack_shutdown();
     void                  set_jack_buffer_size(jack_nframes_t buf_size);
     void                  set_convolver_filename();
     void                  set_in_session();
+    void                  gx_midi_out();
+    void                  on_select_jack_control();
+    void                  delete_select_jack_control();
     Gtk::Label            convolver_filename_label; //FIXME
     int                   fTop;
     GtkWidget*            fBox[stackSize];
@@ -649,7 +543,6 @@ private:
     bool                  on_logger_delete_event(GdkEventAny*);
 
     Gxw::FastMeter        fLevelMeters[2];
-    uiTuner               fTuner;
     Gxw::WaveView         fWaveView;
 
     GtkWidget*            fSignalLevelBar;
@@ -682,7 +575,7 @@ public:
     static const gboolean homogene = FALSE;
 
 public:
-    explicit GxMainInterface(gx_engine::GxEngine&, gx_system::CmdlineOptions& options);
+    explicit GxMainInterface(gx_engine::GxEngine&, gx_system::CmdlineOptions& options, gx_engine::ParamMap& pmap);
     ~GxMainInterface();
     static GxMainInterface& get_instance() { assert(instance); return *instance; }
     bool is_in_session() { return in_session; }
@@ -708,6 +601,7 @@ public:
     void openHorizontalhideBox(const char* label = "");
     void openHorizontalhideBox1(const char* label = "");
     void openVerticalBox(const char* label = "");
+    void openVerticalHideBox(const char* label = "");
     void openVerticalBox1(const char* label = "");
     void openFlipLabelBox(const char* = "");
     void openSetLabelBox();
@@ -717,6 +611,7 @@ public:
     void opensDialogBox(const char *id_dialog, const char *id_switch);
     void openDialogBox(const char *id_dialog, const char *id_switch, const char *expose_funk, GtkWidget *box);
     void opensDialogBox(const char *id_dialog, const char *id_switch, const char *expose_funk, GtkWidget *box);
+    void openRackTunerBox(const char *id_dialog, const char *id_switch, const char *expose_funk, GtkWidget *box);
     void openPatchInfoBox(float* zone);
     void openWarningBox(const char* label, float* zone);
     void openEventBox(const char* label = "");
@@ -740,19 +635,21 @@ public:
     void closeStereoRackBox();
     void openMonoRackBox(const char* label, int* posit, const char *id_on_off, const char *id_pre_post, const char *id_dialog);
     void closeMonoRackBox();
+    void openTunerRackBox(const char* label, int* posit, const char *id_on_off, const char *id_pre_post, const char *id_dialog);
+    void closeTunerRackBox();
     void loadRackFromGladeData(const char *xmldesc);
     void closeBox();
 
     // -- active widgets
     void addJConvButton(const char* label, float* zone);
     void addJConvFavButton(const char* label);
+    void addSmallJConvFavButton(const char* label);
     void addToggleButton(const char* label, float* zone);
     void addJToggleButton(const char* label, bool* zone);
     void addPToggleButton(const char* label);
     void addMToggleButton(const char* label, bool* zone);
-    void addCheckButton(const char* label, float* zone);
+    void addCheckButton(const char* label, bool* zone);
     void addNumEntry(const char* label, float* zone, float init, float min, float max, float step);
-    void addNumDisplay();
     void addLiveWaveDisplay(const char* label);
     void addStatusDisplay(const char* label, bool* zone );
     // void addselector(const char* label, float* zone,int maxv, const char* []);
@@ -839,6 +736,14 @@ public:
     void create_hslider(string id, Glib::ustring(label)) {
             addwidget(UiReglerWithCaption::create(*this, new Gxw::HSlider(), id, label, true));
         }
+
+    void create_master_slider(string id) {
+            addwidget(UiRackMasterRegler::create(*this, new Gxw::HSlider(), id));
+        }
+    void create_master_slider(string id, Glib::ustring(label)) {
+            addwidget(UiRackMasterRegler::create(*this, new Gxw::HSlider(), id, label));
+        }
+
     void create_minislider(string id) {
             addwidget(UiRegler::create(*this, new Gxw::MiniSlider(), id, false));
         }
@@ -896,10 +801,7 @@ public:
     void create_simple_spin_value(string id) {
             addwidget(UiRegler::create(*this, new Gxw::SimpleValueDisplay(), id, true));
         }
-
-    void create_selector(string id, const char *widget_name=0) {
-	addwidget(UiSelector::create(*this, id, widget_name));
-        }
+    void create_selector(string id, const char *widget_name=0);
     void create_switch_no_caption(const char *sw_type, string id) {
             addwidget(UiSwitch::create(*this, sw_type, id));
         }
@@ -963,7 +865,7 @@ string fformat(float value, float step);
 #ifndef NDEBUG
 // debug_check
 inline void check_zone(GtkWidget *w, void *zone) {
-    if (!parameter_map.hasZone(zone)) {
+    if (!gx_engine::parameter_map.hasZone(zone)) {
         gchar *p;
         gtk_widget_path(w, NULL, &p, NULL);
         cerr << "zone not found in definition of widget: "
@@ -971,7 +873,7 @@ inline void check_zone(GtkWidget *w, void *zone) {
         g_free(p);
         assert(false);
     }
-    parameter_map[zone].setUsed();
+    gx_engine::parameter_map[zone].setUsed();
 }
 #endif
 
@@ -980,16 +882,13 @@ inline void check_zone(GtkWidget *w, void *zone) {
 inline void connect_midi_controller(GtkWidget *w, void *zone) {
     debug_check(check_zone, w, zone);
     g_signal_connect(w, "button_press_event", G_CALLBACK(button_press_cb),
-                    (gpointer)&parameter_map[zone]);
+                    (gpointer)&gx_engine::parameter_map[zone]);
 }
 
 /****************************************************************/
 
 void conv_restart();
 
-Glib::RefPtr<Gtk::Builder> load_builder_from_file(Glib::ustring name, gx_ui::GxUI& ui);
-Glib::RefPtr<Gtk::Builder> load_builder_from_data(const char *xmldesc, gx_ui::GxUI& ui);
-
 /* -------------------------------------------------------------------------- */
 } /* end of gx_gui namespace */
 
