@@ -35,36 +35,6 @@ gboolean gx_check_cab_state(gpointer args);
 void cab_conv_restart();
 void contrast_conv_restart();
 
-/* -------- helper for level meter display -------- */
-inline float log_meter(float db) {
-    gfloat def = 0.0f; /* Meter deflection %age */
-
-    if (db < -70.0f) {
-        def = 0.0f;
-    } else if (db < -60.0f) {
-        def = (db + 70.0f) * 0.25f;
-    } else if (db < -50.0f) {
-        def = (db + 60.0f) * 0.5f + 2.5f;
-    } else if (db < -40.0f) {
-        def = (db + 50.0f) * 0.75f + 7.5f;
-    } else if (db < -30.0f) {
-        def = (db + 40.0f) * 1.5f + 15.0f;
-    } else if (db < -20.0f) {
-        def = (db + 30.0f) * 2.0f + 30.0f;
-    } else if (db < 6.0f) {
-        def = (db + 20.0f) * 2.5f + 50.0f;
-    } else {
-        def = 115.0f;
-    }
-
-    /* 115 is the deflection %age that would be
-       when db=6.0. this is an arbitrary
-       endpoint for our scaling.
-    */
-
-    return def/115.0f;
-}
-
 /* --------- calculate power (percent) to decibel -------- */
 // Note: could use fast_log10 (see ardour code) to make it faster
 inline float power2db(float power) {
